@@ -1,4 +1,4 @@
-<?php rcs_id('$Id: BlockParser.php,v 1.55 2005/01/29 21:08:41 rurban Exp $');
+<?php rcs_id('$Id: BlockParser.php,v 1.60 2007/01/07 18:41:32 rurban Exp $');
 /* Copyright (C) 2002 Geoffrey T. Dairiki <dairiki@dairiki.org>
  * Copyright (C) 2004,2005 Reini Urban
  *
@@ -372,10 +372,17 @@ class ParsedBlock extends Block_HtmlElement {
     	static $_regexpset, $_block_types;
 
     	if (!is_object($_regexpset)) {
-            foreach (array('oldlists', 'list', 'dl', 'table_dl',
-                           'blockquote', 'heading', 'hr', 'pre', 'email_blockquote',
-                           'plugin', 'p')
-                     as $type) {
+	    $Block_types = array
+		    ('oldlists', 'list', 'dl', 'table_dl',
+                     'blockquote', 'heading', 'hr', 'pre', 'email_blockquote',
+		     'plugin', 'p');
+            // insert it before p!
+            if (ENABLE_MARKUP_DIVSPAN) {
+            	array_pop($Block_types);
+ 		$Block_types[] = 'divspan';
+ 		$Block_types[] = 'p';
+            }
+            foreach ($Block_types as $type) {
                 $class = "Block_$type";
                 $proto = new $class;
                 $this->_block_types[] = $proto;
@@ -384,6 +391,7 @@ class ParsedBlock extends Block_HtmlElement {
             $this->_regexpset = new AnchoredRegexpSet($this->_regexps);
             $_regexpset = $this->_regexpset;
             $_block_types = $this->_block_types;
+            unset($Block_types);
     	} else {
              $this->_regexpset = $_regexpset;
              $this->_block_types = $_block_types;
@@ -502,6 +510,7 @@ class Block_blockquote extends BlockMarkup
         if (get_class($nextBlock) == get_class($this)) {
             assert ($nextBlock->_depth < $this->_depth);
             $nextBlock->_element->unshiftContent($this->_element);
+	    if (!empty($this->_tight_top))
             $nextBlock->_tight_top = $this->_tight_top;
             return $nextBlock;
         }
@@ -910,7 +919,7 @@ class Block_oldlists extends Block_list
 
 class Block_pre extends BlockMarkup
 {
-    var $_re = '<(?:pre|verbatim)>';
+    var $_re = '<(?:pre|verbatim|nowiki)>';
 
     function _match (&$input, $m) {
         $endtag = '</' . substr($m->match, 1);
@@ -927,14 +936,21 @@ class Block_pre extends BlockMarkup
         }
         $input->advance();
         
-        $text = join("\n", $text);
+	if ($m->match == '<nowiki>')
+	    $text = join("<br>\n", $text);
+	else
+            $text = join("\n", $text);
         
         // FIXME: no <img>, <big>, <small>, <sup>, or <sub>'s allowed
         // in a <pre>.
         if ($m->match == '<pre>')
             $text = TransformInline($text);
-
-        $this->_element = new Block_HtmlElement('pre', false, $text);
+	if ($m->match == '<nowiki>') {
+            $text = TransformInlineNowiki($text);
+	    $this->_element = new Block_HtmlElement('p', false, $text);
+	} else {
+            $this->_element = new Block_HtmlElement('pre', false, $text);
+	}
         return true;
     }
 }
@@ -1051,6 +1067,48 @@ class Block_p extends BlockMarkup
     }
 }
 
+class Block_divspan extends BlockMarkup
+{
+    var $_re = '<(?im)(?: div|span)(?:[^>]*)?>';
+
+    function _match (&$input, $m) {
+    	if (substr($m->match,1,4) == 'span') {
+    	    $tag = 'span';
+	} else {
+    	    $tag = 'div';
+	}
+	// without last >
+        $argstr = substr(trim(substr($m->match,strlen($tag)+1)),0,-1); 
+        $pos = $input->getPos();
+        $pi  = $content = $m->postmatch;
+        while (!preg_match('/^(.*)\<\/'.$tag.'\>(.*)$/i', $pi, $me)) {
+            $content .= "\n$pi";;
+            if (($pi = $input->nextLine()) === false) {
+                $input->setPos($pos);
+                return false;
+            }
+        }
+        $content .= $me[1]; // prematch
+        $input->advance();
+        $content = TransformInline(trim($content));
+        if (!$argstr) $args = false;
+        else {
+            foreach (preg_split("/\s+/", $argstr) as $a) {
+                @list($k, $v) = preg_split("/\s*=\s*/", $a);
+                if ($v) $v = preg_replace(array("/^[\"']/","/['\"]$/"), array('',''), $v);
+                if (trim($k) and trim($v)) $args[$k] = $v;
+            }
+        }
+        $this->_element = new Block_HtmlElement($tag, $args, $content);
+        //$this->_element->setTightness($tag == 'span', $tag == 'span');
+        return true;
+    }
+    function _setTightness($top, $bot) {
+	// Don't tighten user <div|span>
+    }
+}
+
+
 ////////////////////////////////////////////////////////////////
 //
 
@@ -1092,6 +1150,21 @@ function TransformText ($text, $markup = 2.0, $basepage=false) {
 }
 
 // $Log: BlockParser.php,v $
+// Revision 1.60  2007/01/07 18:41:32  rurban
+// Fix ENABLE_DIVSPAN
+//
+// Revision 1.59  2006/12/22 16:21:29  rurban
+// silence one BlockParser use-case
+//
+// Revision 1.58  2006/11/19 13:57:14  rurban
+// fix Regex Syntax Error
+//
+// Revision 1.57  2006/10/12 06:32:30  rurban
+// Optionally support new tags <div>, <span> with ENABLE_MARKUP_DIVSPAN (in work)
+//
+// Revision 1.56  2006/07/23 14:03:18  rurban
+// add new feature: DISABLE_MARKUP_WIKIWORD
+//
 // Revision 1.55  2005/01/29 21:08:41  rurban
 // update (C)
 //
@@ -1133,4 +1206,4 @@ function TransformText ($text, $markup = 2.0, $basepage=false) {
 // c-hanging-comment-ender-p: nil
 // indent-tabs-mode: nil
 // End:   
-?>
\ No newline at end of file
+?>
