@@ -1,4 +1,4 @@
-<?php rcs_id('$Id: ErrorManager.php,v 1.43 2005/04/11 19:41:23 rurban Exp $');
+<?php rcs_id('$Id: ErrorManager.php,v 1.50 2007/01/09 12:35:28 rurban Exp $');
 
 if (isset($GLOBALS['ErrorManager'])) return;
 
@@ -22,7 +22,8 @@ define ('EM_NOTICE_ERRORS', E_NOTICE | E_USER_NOTICE);
    Only where absolute speed is necessary you might want to turn 
    them off.
 */
-if (1 or (defined('DEBUG') and DEBUG))
+//also turn it on if phpwiki_version notes no release
+if (defined('DEBUG') and DEBUG)
     assert_options (ASSERT_ACTIVE, 1);
 else
     assert_options (ASSERT_ACTIVE, 0);
@@ -460,12 +461,16 @@ class PhpError {
         $errfile = preg_replace('|^' . preg_quote($dir) . '|', '', $this->errfile);
         $lines = explode("\n", $this->errstr);
         if (DEBUG & _DEBUG_VERBOSE) {
-          $msg = sprintf("%s:%d: %s[%d]: %s",
+          $msg = sprintf("%s:%d %s[%d]: %s",
                          $errfile, $this->errline,
                          $this->getDescription(), $this->errno,
                          array_shift($lines));
+        } elseif (! $this->isFatal()) {
+          $msg = sprintf("%s: \"%s\"",
+                         $this->getDescription(),
+                         array_shift($lines));
         } else {
-          $msg = sprintf("%s:%d: %s: \"%s\"",
+          $msg = sprintf("%s:%d %s: \"%s\"",
                          $errfile, $this->errline,
                          $this->getDescription(),
                          array_shift($lines));
@@ -591,12 +596,19 @@ class PhpErrorOnce extends PhpError {
 	        $lines = array($this->errstr->asXML());
         $errtype = (DEBUG & _DEBUG_VERBOSE) ? sprintf("%s[%d]", $this->getDescription(), $this->errno)
                                             : sprintf("%s", $this->getDescription());
-        $msg = sprintf("%s:%d: %s: %s %s",
+        if ((DEBUG & _DEBUG_VERBOSE) or $this->isFatal()) {
+	    $msg = sprintf("%s:%d %s: %s %s",
                        $errfile, $this->errline,
                        $errtype,
                        array_shift($lines),
                        $count > 1 ? sprintf(" (...repeated %d times)",$count) : ""
                        );
+	} else {
+          $msg = sprintf("%s: \"%s\" %s",
+			 $errtype,
+			 array_shift($lines),
+			 $count > 1 ? sprintf(" (...repeated %d times)",$count) : "");
+	}
         $html = HTML::div(array('class' => $this->getHtmlClass()), 
                           HTML::p($msg));
         if ($lines) {
@@ -617,6 +629,21 @@ if (!isset($GLOBALS['ErrorManager'])) {
 }
 
 // $Log: ErrorManager.php,v $
+// Revision 1.50  2007/01/09 12:35:28  rurban
+// release ready: turn off assert
+//
+// Revision 1.49  2006/12/22 00:17:49  rurban
+// improve and unify error messages
+//
+// Revision 1.48  2006/03/19 14:29:40  rurban
+// sf.net patch #1438439 by Matt Brown: Only set no-cache headers when error output is generated
+//
+// Revision 1.47  2005/10/31 17:20:40  rurban
+// fix ConvertBefore
+//
+// Revision 1.46  2005/10/30 16:38:13  rurban
+// minor fixes
+//
 // Revision 1.45  2005/10/29 14:28:08  uckelman
 // existence of isa should be checked, not built-in is_a()
 //
@@ -707,4 +734,4 @@ if (!isset($GLOBALS['ErrorManager'])) {
 // c-hanging-comment-ender-p: nil
 // indent-tabs-mode: nil
 // End:
-?>
\ No newline at end of file
+?>
