@@ -1,4 +1,4 @@
-<?php rcs_id('$Id: HtmlElement.php,v 1.46 2005/01/25 06:50:33 rurban Exp $');
+<?php rcs_id('$Id: HtmlElement.php,v 1.49 2007/06/02 18:23:49 rurban Exp $');
 /**
  * Code for writing the HTML subset of XML.
  * @author: Jeff Dairiki
@@ -63,8 +63,9 @@ class HtmlElement extends XmlElement
      *
      * @param $tooltip_text string The tooltip text.
      */
-    function addTooltip ($tooltip_text) {
+    function addTooltip ($tooltip_text, $accesskey = null) {
         $this->setAttr('title', $tooltip_text);
+	if ($accesskey) $this->setAccesskey($accesskey);
 
         // FIXME: this should be initialized from title by an onLoad() function.
         //        (though, that may not be possible.)
@@ -75,6 +76,23 @@ class HtmlElement extends XmlElement
         $this->setAttr('onmouseout', "window.status='';return true;");
     }
 
+    function setAccesskey ($key) {
+	global $WikiTheme;
+	if (strlen($key) != 1) return;
+	$this->setAttr("accesskey", $key);
+
+        if (!empty($this->_attr['title'])) {
+	    if (preg_match("/\[(alt-)?(.)\]$/", $this->_attr['title'], $m))
+	    {
+		$this->_attr['title'] = preg_replace("/\[(alt-)?(.)\]$/", "[".$WikiTheme->tooltipAccessKeyPrefix()."-\\2]", $this->_attr['title']);
+	    } else  {
+		$this->_attr['title'] .= " [".$WikiTheme->tooltipAccessKeyPrefix()."-$key]";
+	    }
+	} else {
+	    $this->_attr['title'] = "[".$WikiTheme->tooltipAccessKeyPrefix()."-$key]";
+	}
+    }
+
     function emptyTag () {
         if (($this->_properties & HTMLTAG_EMPTY) == 0)
             return $this->startTag() . "</$this->_tag>";
@@ -404,6 +422,10 @@ class HTML extends HtmlElement {
         $el = new HtmlElement('embed');
         return $el->_init2(func_get_args());
     }
+    function param (/*...*/) {
+        $el = new HtmlElement('param');
+        return $el->_init2(func_get_args());
+    }
 }
 
 define('HTMLTAG_EMPTY', 1);
@@ -493,13 +515,13 @@ function JavaScript ($js, $script_args = false) {
     $script_args = $script_args ? array_merge($default_script_args, $script_args)
                                 : $default_script_args;
     if (empty($js))
-        return HTML::script($script_args);
+        return HTML(HTML::script($script_args),"\n");
     else
         // see http://devedge.netscape.com/viewsource/2003/xhtml-style-script/
-        return HTML::script($script_args,
+        return HTML(HTML::script($script_args,
                             new RawXml((ENABLE_XHTML_XML ? "\n//<![CDATA[" : "\n<!--//")
-                                       . "\n".rtrim($js)."\n"
-                                       . (ENABLE_XHTML_XML ? "//]]>\n" : "// -->")));
+                                       . "\n".trim($js)."\n"
+                                       . (ENABLE_XHTML_XML ? "//]]>\n" : "// -->"))),"\n");
 }
 
 /** Conditionally display content based of whether javascript is supported.
@@ -535,6 +557,15 @@ function IfJavaScript($if_content = false, $else_content = false) {
     
 /**
  $Log: HtmlElement.php,v $
+ Revision 1.49  2007/06/02 18:23:49  rurban
+ Added accesskeys
+
+ Revision 1.48  2007/05/24 18:51:35  rurban
+ add param (for YouTube)
+
+ Revision 1.47  2005/08/06 12:53:36  rurban
+ beautify SCRIPT lines
+
  Revision 1.46  2005/01/25 06:50:33  rurban
  added label
 
@@ -648,4 +679,4 @@ function IfJavaScript($if_content = false, $else_content = false) {
 // c-hanging-comment-ender-p: nil
 // indent-tabs-mode: nil
 // End:
-?>
\ No newline at end of file
+?>
