@@ -1,6 +1,5 @@
 <?php
-rcs_id('$Id: IniConfig.php,v 1.94 2005/09/15 05:56:12 rurban Exp $');
-
+rcs_id('$Id: IniConfig.php,v 1.115 2007/04/18 20:40:49 rurban Exp $');
 /**
  * A configurator intended to read its config from a PHP-style INI file,
  * instead of a PHP file.
@@ -14,7 +13,7 @@ rcs_id('$Id: IniConfig.php,v 1.94 2005/09/15 05:56:12 rurban Exp $');
  * @author: Joby Walker, Reini Urban, Matthew Palmer
  */
 /*
- * Copyright 2004,2005 $ThePhpWikiProgrammingTeam
+ * Copyright 2004,2005,2006,2007 $ThePhpWikiProgrammingTeam
  *
  * This file is part of PhpWiki.
  *
@@ -101,6 +100,13 @@ function save_dump($file) {
     fclose($fp);
 }
 
+function _check_int_constant(&$c) {
+  // if int value == string value, force int type
+  if (sprintf("%d",(int)$c) === $c) { // DEBUG & _DEBUG_bla
+    $c = (int)$c;
+  }
+}
+
 function IniConfig($file) {
 
     // check config/config.php dump for faster startup
@@ -148,8 +154,10 @@ function IniConfig($file) {
          'PLUGIN_CACHED_DATABASE', 'PLUGIN_CACHED_FILENAME_PREFIX',
          'PLUGIN_CACHED_HIGHWATER', 'PLUGIN_CACHED_LOWWATER', 'PLUGIN_CACHED_MAXLIFETIME',
          'PLUGIN_CACHED_MAXARGLEN', 'PLUGIN_CACHED_IMGTYPES',
+         'WYSIWYG_BACKEND', 'PLUGIN_MARKUP_MAP',
          // extra logic:
          'SERVER_NAME','SERVER_PORT','SCRIPT_NAME', 'DATA_PATH', 'PHPWIKI_DIR', 'VIRTUAL_PATH',
+	 'EXTERNAL_HTML2PDF_PAGELIST', 'PLUGIN_CACHED_CACHE_DIR'
          );
 
     // Optional values which need to be defined.
@@ -165,7 +173,7 @@ function IniConfig($file) {
          'GOOGLE_LICENSE_KEY','FORTUNE_DIR',
          'DISABLE_GETIMAGESIZE','DBADMIN_USER','DBADMIN_PASSWD',
          'SESSION_SAVE_PATH', 'TOOLBAR_PAGELINK_PULLDOWN', 'TOOLBAR_TEMPLATE_PULLDOWN',
-         'EXTERNAL_LINK_TARGET', 'ACCESS_LOG_SQL', 'ENABLE_MARKUP_TEMPLATE'
+         'EXTERNAL_LINK_TARGET', 'ACCESS_LOG_SQL', 'USE_EXTERNAL_HTML2PDF'
          );
 
     // List of all valid config options to be define()d which take booleans.
@@ -183,9 +191,12 @@ function IniConfig($file) {
          'WARN_NONPUBLIC_INTERWIKIMAP', 'USE_PATH_INFO',
          'DISABLE_HTTP_REDIRECT',
          'PLUGIN_CACHED_USECACHE', 'PLUGIN_CACHED_FORCE_SYNCMAP',
-         'BLOG_EMPTY_DEFAULT_PREFIX', 'DATABASE_PERSISTENT',
+         'BLOG_DEFAULT_EMPTY_PREFIX', 'DATABASE_PERSISTENT',
          'ENABLE_DISCUSSION_LINK', 'ENABLE_CAPTCHA',
-         'USE_CAPTCHA_RANDOM_WORD'
+         'ENABLE_WYSIWYG', 'WYSIWYG_DEFAULT_PAGETYPE_HTML',
+         'DISABLE_MARKUP_WIKIWORD', 'ENABLE_MARKUP_COLOR', 'ENABLE_MARKUP_TEMPLATE',
+         'ENABLE_MARKUP_DIVSPAN', 'USE_BYTEA', 'UPLOAD_USERDIR', 'DISABLE_UNITS',
+	 'ENABLE_SEARCHHIGHLIGHT', 'DISABLE_UPLOAD_ONLY_ALLOWED_EXTENSIONS'
          );
 
     $rs = @parse_ini_file($file);
@@ -205,6 +216,7 @@ function IniConfig($file) {
             continue;
         }
         if (array_key_exists($item, $rs)) {
+            _check_int_constant($rs[$item]);
             define($item, $rs[$item]);
             unset($rs[$item]);
         //} elseif (array_key_exists($item, $rsdef)) {
@@ -214,7 +226,7 @@ function IniConfig($file) {
                            array('DATABASE_PREFIX', 'SERVER_NAME', 'SERVER_PORT',
                                  'SCRIPT_NAME', 'DATA_PATH', 'PHPWIKI_DIR', 'VIRTUAL_PATH',
                                  'LDAP_AUTH_HOST','IMAP_AUTH_HOST','POP3_AUTH_HOST',
-                                 'PLUGIN_CACHED_CACHE_DIR'))) 
+                                 'PLUGIN_CACHED_CACHE_DIR','EXTERNAL_HTML2PDF_PAGELIST'))) 
         {
             ;
         } elseif (!defined("_PHPWIKI_INSTALL_RUNNING")) {
@@ -248,7 +260,7 @@ function IniConfig($file) {
                                   'ALLOW_IMAP_LOGIN', 'ALLOW_USER_LOGIN',
                                   'REQUIRE_SIGNIN_BEFORE_EDIT',
                                   'WIKIDB_NOCACHE_MARKUP',
-                                  'COMPRESS_OUTPUT'
+                                  'COMPRESS_OUTPUT', 'USE_BYTEA'
                                   )))
         {
             ;
@@ -294,10 +306,12 @@ function IniConfig($file) {
             unset($rsdef[$item]);
         }
     }
-    if (!in_array(DATABASE_TYPE, array('SQL','ADODB','PDO','dba','file','cvs')))
+    $valid_database_types = array('SQL','ADODB','PDO','dba','file','flatfile','cvs','cvsclient');
+    if (!in_array(DATABASE_TYPE, $valid_database_types))
         trigger_error(sprintf("Invalid DATABASE_TYPE=%s. Choose one of %s", 
-                              DATABASE_TYPE, "SQL,ADODB,PDO,dba,file,cvs"), 
+                              DATABASE_TYPE, join(",", $valid_database_types)), 
                       E_USER_ERROR);
+    unset($valid_database_types);                  
     if (DATABASE_TYPE == 'PDO') {
         if (!check_php_version(5))
             trigger_error("Invalid DATABASE_TYPE=PDO. PDO requires at least php-5.0!", 
@@ -345,12 +359,13 @@ function IniConfig($file) {
     unset($item); unset($major); unset($max); 
     
     // User authentication
-    if (!isset($GLOBALS['USER_AUTH_ORDER']))
+    if (!isset($GLOBALS['USER_AUTH_ORDER'])) {
         if (isset($rs['USER_AUTH_ORDER']))
             $GLOBALS['USER_AUTH_ORDER'] = preg_split('/\s*:\s*/', 
                                                      $rs['USER_AUTH_ORDER']);
         else 
             $GLOBALS['USER_AUTH_ORDER'] = array("PersonalPage");
+    }
 
     // Now it's the external DB authentication stuff's turn
     if (in_array('Db', $GLOBALS['USER_AUTH_ORDER']) && empty($rs['DBAUTH_AUTH_DSN'])) {
@@ -385,7 +400,7 @@ function IniConfig($file) {
     }
     unset($rskey); unset($apkey);
 
-    // TODO: Currently unsupported on non-SQL
+    // TODO: Currently unsupported on non-SQL. Nice to have for RhNavPlugin
     // CHECKME: PDO
     if (array_key_exists('ACCESS_LOG_SQL', $rs)) {
     	// WikiDB_backend::isSql() not yet loaded
@@ -399,6 +414,23 @@ function IniConfig($file) {
                in_array(DATABASE_TYPE, array('SQL','ADODB','PDO')) ? 2 : 0);
     }
 
+    global $PLUGIN_MARKUP_MAP;
+    $PLUGIN_MARKUP_MAP = array();
+    if (defined('PLUGIN_MARKUP_MAP') and trim(PLUGIN_MARKUP_MAP) != "") {
+	$_map = preg_split('/\s+/', PLUGIN_MARKUP_MAP);
+	foreach ($_map as $v) {
+	    list($xml,$plugin) = split(':', $v);
+	    if (!empty($xml) and !empty($plugin))
+	        $PLUGIN_MARKUP_MAP[$xml] = $plugin;
+	}
+	unset($_map); unset($xml); unset($plugin); unset($v);
+    }
+
+    if (empty($rs['TEMP_DIR'])) {
+	$rs['TEMP_DIR'] = "/tmp";
+	if (getenv("TEMP"))
+	    $rs['TEMP_DIR'] = getenv("TEMP");
+    }
     // optional values will be set to '' to simplify the logic.
     foreach ($_IC_OPTIONAL_VALUE as $item) {
         if (defined($item)) {
@@ -406,13 +438,26 @@ function IniConfig($file) {
             continue;
         }
         if (array_key_exists($item, $rs)) {
+	    _check_int_constant($rs[$item]);
             define($item, $rs[$item]);
             unset($rs[$item]);
         } else 
             define($item, '');
     }
+
+    if (USE_EXTERNAL_HTML2PDF) {
+	$item = 'EXTERNAL_HTML2PDF_PAGELIST';
+        if (defined($item)) {
+            unset($rs[$item]);
+        } elseif (array_key_exists($item, $rs)) {
+            define($item, $rs[$item]);
+            unset($rs[$item]);
+        } elseif (array_key_exists($item, $rsdef)) {
+            define($item, $rsdef[$item]);
+	}
+    }
     unset($item); 
-    
+        
     // LDAP bind options
     global $LDAP_SET_OPTION;
     if (defined('LDAP_SET_OPTION') and LDAP_SET_OPTION) {
@@ -462,28 +507,25 @@ function IniConfig($file) {
     global $PLUGIN_CACHED_IMGTYPES;
     $PLUGIN_CACHED_IMGTYPES = preg_split('/\s*[|:]\s*/', PLUGIN_CACHED_IMGTYPES);
 
-    if (empty($rs['PLUGIN_CACHED_CACHE_DIR']) and !empty($rsdef['PLUGIN_CACHED_CACHE_DIR']))
-        $rs['PLUGIN_CACHED_CACHE_DIR'] = $rsdef['PLUGIN_CACHED_CACHE_DIR'];
-    if (empty($rs['PLUGIN_CACHED_CACHE_DIR'])) {
-        if (!empty($rs['INCLUDE_PATH'])) {
-            @ini_set('include_path', $rs['INCLUDE_PATH']);
+    if (!defined('PLUGIN_CACHED_CACHE_DIR')) {
+        if (empty($rs['PLUGIN_CACHED_CACHE_DIR']) and !empty($rsdef['PLUGIN_CACHED_CACHE_DIR']))
+            $rs['PLUGIN_CACHED_CACHE_DIR'] = $rsdef['PLUGIN_CACHED_CACHE_DIR'];
+        if (empty($rs['PLUGIN_CACHED_CACHE_DIR'])) {
+            if (!empty($rs['INCLUDE_PATH'])) {
+                @ini_set('include_path', $rs['INCLUDE_PATH']);
+            }
+            $rs['PLUGIN_CACHED_CACHE_DIR'] = TEMP_DIR . '/cache';
+            if (!FindFile($rs['PLUGIN_CACHED_CACHE_DIR'], 1)) { // [29ms]
+                FindFile(TEMP_DIR, false, 1);            // TEMP must exist!
+                mkdir($rs['PLUGIN_CACHED_CACHE_DIR'], 777);
+            }
+            // will throw an error if not exists.
+            define('PLUGIN_CACHED_CACHE_DIR', FindFile($rs['PLUGIN_CACHED_CACHE_DIR'],false,1)); 
+        } else {
+            define('PLUGIN_CACHED_CACHE_DIR', $rs['PLUGIN_CACHED_CACHE_DIR']);
+            // will throw an error if not exists.
+            FindFile(PLUGIN_CACHED_CACHE_DIR);
         }
-        if (empty($rs['TEMP_DIR'])) {
-            $rs['TEMP_DIR'] = "/tmp";
-            if (getenv("TEMP"))
-                $rs['TEMP_DIR'] = getenv("TEMP");
-        }
-        $rs['PLUGIN_CACHED_CACHE_DIR'] = $rs['TEMP_DIR'] . '/cache';
-        if (!FindFile($rs['PLUGIN_CACHED_CACHE_DIR'], 1)) { // [29ms]
-            FindFile($rs['TEMP_DIR'], false, 1);            // TEMP must exist!
-            mkdir($rs['PLUGIN_CACHED_CACHE_DIR'], 777);
-        }
-        // will throw an error if not exists.
-        define('PLUGIN_CACHED_CACHE_DIR', FindFile($rs['PLUGIN_CACHED_CACHE_DIR'],false,1)); 
-    } else {
-        define('PLUGIN_CACHED_CACHE_DIR', $rs['PLUGIN_CACHED_CACHE_DIR']);
-        // will throw an error if not exists.
-        FindFile(PLUGIN_CACHED_CACHE_DIR);
     }
 
     // process the rest of the config.ini settings:
@@ -491,6 +533,7 @@ function IniConfig($file) {
         if (defined($item)) {
             continue;
         } else {
+	    _check_int_constant($v);
             define($item, $v);
         }
     }
@@ -509,10 +552,20 @@ function IniConfig($file) {
     fixup_dynamic_configs($file); // [100ms]
 }
 
+function _ignore_unknown_charset_warning(&$error) {
+    //htmlspecialchars(): charset `iso-8859-2' not supported, assuming iso-8859-1
+    if (preg_match('/^htmlspecialchars\(\): charset \`.+\' not supported, assuming iso-8859-1/',
+                   $error->errstr)) {
+        $error->errno = 0;
+        return true;  // Ignore error
+    }
+    return false;
+}
+
 // moved from lib/config.php [1ms]
 function fixup_static_configs($file) {
     global $FieldSeparator, $charset, $WikiNameRegexp, $AllActionPages;
-    global $HTTP_SERVER_VARS, $DBParams, $LANG;
+    global $HTTP_SERVER_VARS, $DBParams, $LANG, $ErrorManager;
     // init FileFinder to add proper include paths
     FindFile("lib/interwiki.map",true);
     
@@ -520,7 +573,7 @@ function fixup_static_configs($file) {
     // chars in iso-8859-*
     // $FieldSeparator = "\263"; // this is a superscript 3 in ISO-8859-1.
     // $FieldSeparator = "\xFF"; // this byte should never appear in utf-8
-    // FIXME: get rid of constant. pref is dynamic and language specific
+    // Get rid of constant. pref is dynamic and language specific
     $charset = CHARSET;
     // Disabled: Let the admin decide which charset.
     //if (isset($LANG) and in_array($LANG,array('zh')))
@@ -530,18 +583,32 @@ function fixup_static_configs($file) {
     else
         $FieldSeparator = "\x81";
 
+    // Some exotic charsets are not supported by htmlspecialchars, which just prints an E_WARNING.
+    // Even on simple 8bit charsets, where just <>& need to be replaced. For iso-8859-[2-4] e.g.
+    // See <php-src>/ext/standard/html.c
+    // For performance reasons we require a magic constant to ignore this warning.
+    if (defined('IGNORE_CHARSET_NOT_SUPPORTED_WARNING') and IGNORE_CHARSET_NOT_SUPPORTED_WARNING) {
+        $ErrorManager->pushErrorHandler(new WikiFunctionCb('_ignore_unknown_charset_warning'));
+    }
+    // Used by SetupWiki to pull in required pages, if not translated, then in english.
+    // Also used by _WikiTranslation. Really important are only those which return pagelists 
+    // or contain basic functionality.
+    /*
+      All pages containing plugins of the same name as the filename:
+      cd pgsrc
+      grep -l '\?plugin ' *| perl -ne'$/=0;chop; s/%([\da-fA-F]{2})/pack("C",hex($1))/ge; next LINE if m{^(Help/|Template|Pgsrc)}; print "$_\n"; {local $/;open F,"<$_"; $f=join("",<F>);} push @a,$_ if $f=~/plugin $_/; END{print join(":",@a)};'
+     */
     $AllActionPages = explode(':',
-                              'AllPages:BackLinks:CreatePage:DebugInfo:EditMetaData:FindPage:'
-                              .'FullRecentChanges:FullTextSearch:FuzzyPages:InterWikiSearch:'
-                              .'LikePages:MostPopular:'
-                              .'OrphanedPages:PageDump:PageHistory:PageInfo:RandomPage:RateIt:'
-                              .'RecentChanges:RecentEdits:RecentComments:RelatedChanges:TitleSearch:'
-                              .'TranslateText:UpLoad:UserPreferences:WantedPages:WhoIsOnline:'
+			      'AllPages:AllUsers:AppendText:AuthorHistory:BackLinks:CreatePage:EditMetaData:FindPage:FullTextSearch:'
+			      .'FuzzyPages:InterWikiSearch:LikePages:LinkDatabase:LinkSearch:ListRelations:ModeratedPage:'
+			      .'MostPopular:OrphanedPages:PageDump:PageHistory:PageInfo:PasswordReset:PluginManager:RandomPage:'
+			      .'RateIt:RecentChanges:RecentComments:RelatedChanges:SearchHighlight:SemanticRelations:SemanticSearch:'
+			      .'TitleSearch:TranslateText:UpLoad:UserPreferences:WantedPages:WatchPage:WhoIsOnline:WikiAdminSelect'
+			      // plus some derivations
+			      .':DebugInfo:LeastPopular:RecentEdits:AllPagesCreatedByMe:AllPagesLastEditedByMe:AllPagesOwnedByMe:'
                               .'PhpWikiAdministration/Remove:PhpWikiAdministration/Chmod:'
                               .'PhpWikiAdministration/Rename:PhpWikiAdministration/Replace:'
-                              .'PhpWikiAdministration/SetAcl:PhpWikiAdministration/Chown'
-                              );
-
+                              .'PhpWikiAdministration/SetAcl:PhpWikiAdministration/Chown');
     // If user has not defined PHPWIKI_DIR, and we need it
     if (!defined('PHPWIKI_DIR') and !file_exists("themes/default")) {
     	$themes_dir = FindFile("themes");
@@ -742,7 +809,14 @@ function fixup_dynamic_configs($file) {
     foreach (array('SERVER_NAME','SERVER_PORT') as $var) {
         //FIXME: for CGI without _SERVER
         if (!defined($var) and !empty($HTTP_SERVER_VARS[$var]))
-            define($var, $HTTP_SERVER_VARS[$var]);
+            // IPV6 fix by matt brown, #1546571
+            // An IPv6 address must be surrounded by square brackets to form a valid server name.
+            if ($var == 'SERVER_NAME' &&
+                    strstr($HTTP_SERVER_VARS[$var], ':')) {
+                define($var, '[' . $HTTP_SERVER_VARS[$var] . ']');
+            } else {
+                define($var, $HTTP_SERVER_VARS[$var]);
+            }
     }
     if (!defined('SERVER_NAME')) define('SERVER_NAME', '127.0.0.1');
     if (!defined('SERVER_PORT')) define('SERVER_PORT', 80);
@@ -861,12 +935,74 @@ function fixup_dynamic_configs($file) {
 }
 
 // $Log: IniConfig.php,v $
+// Revision 1.115  2007/04/18 20:40:49  rurban
+// added DISABLE_UPLOAD_ONLY_ALLOWED_EXTENSIONS
+//
+// Revision 1.114  2007/02/17 22:49:24  rurban
+// enable TEMP_DIR, fixes bug 1646080
+//
+// Revision 1.113  2007/01/27 21:52:54  rurban
+// Define TEMP_DIR always
+//
+// Revision 1.112  2007/01/22 23:59:52  rurban
+// protect against PLUGIN_CACHED_CACHE_DIR definition
+//
+// Revision 1.111  2007/01/20 15:54:04  rurban
+// rename USE_SEARCHHIGHLIGHT to ENABLE_SEARCHHIGHLIGHT
+//
+// Revision 1.110  2007/01/07 18:42:39  rurban
+// Fix typo: BLOG_EMPTY_DEFAULT_PREFIX => BLOG_DEFAULT_EMPTY_PREFIX. Add USE_SEARCHHIGHLIGHT. Add EXTERNAL_HTML2PDF_PAGELIST logic. Update $AllActionPages.
+//
+// Revision 1.109  2007/01/04 16:40:47  rurban
+// Add more action pages
+//
+// Revision 1.108  2007/01/03 21:22:22  rurban
+// Add DISABLE_UNITS.
+//
+// Revision 1.107  2006/12/22 17:45:28  rurban
+// add USE_BYTEA
+//
+// Revision 1.106  2006/11/19 13:52:18  rurban
+// fix and stabilize MAP splitting
+//
+// Revision 1.105  2006/11/19 11:24:11  rurban
+// IPV6 fix by matt brown, #1546571
+// An IPv6 address must be surrounded by square brackets to form a valid server name.
+//
+// Revision 1.104  2006/10/12 06:36:09  rurban
+// Guard against unwanted DEBUG="DEBUG" logic. In detail (WikiDB),
+// and generally by forcing all int constants to be defined as int.
+//
+// Revision 1.103  2006/08/15 13:36:23  rurban
+// support iso-8859-2
+//
+// Revision 1.102  2006/05/14 18:02:27  rurban
+// patch from bug #1480077 by Kai Krakow
+//
+// Revision 1.101  2006/05/13 19:59:54  rurban
+// added wysiwyg_editor-1.3a feature by Jean-Nicolas GEREONE <jean-nicolas.gereone@st.com>
+// converted wysiwyg_editor-1.3a js to WysiwygEdit framework
+// changed default ENABLE_WYSIWYG = true and added WYSIWYG_BACKEND = Wikiwyg
+//
+// Revision 1.100  2006/03/19 14:23:51  rurban
+// sf.net patch #1377011 by Matt Brown: add DATABASE_OPTIMISE_FREQUENCY
+//
+// Revision 1.99  2006/03/07 20:42:34  rurban
+// add DBAUTH_PREF_INSERT support
+//
+// Revision 1.98  2006/01/12 16:22:47  rurban
+// Add flatfile and cvsclient to $valid_database_types (internal testing)
+// Fix bug #1381464 "Duplicate functions ver. 1.3.11p1"
+//
 // Revision 1.97  2005/10/29 14:16:38  rurban
 // fix broken locale update
 //
 // Revision 1.96  2005/09/26 06:27:33  rurban
 // default locale fix Thomas Harding
 //
+// Revision 1.95  2005/09/18 15:15:53  rurban
+// add a proper Content-Encoding: gzip if compressed, and omit Content-Length then.
+//
 // Revision 1.94  2005/09/15 05:56:12  rurban
 // read configurator desc from config-dist.ini, update desc, fix some warnings
 //
@@ -1253,4 +1389,4 @@ function fixup_dynamic_configs($file) {
 // c-hanging-comment-ender-p: nil
 // indent-tabs-mode: nil
 // End:   
-?>
\ No newline at end of file
+?>
