@@ -1,4 +1,4 @@
-<?php rcs_id('$Id: PageList.php,v 1.135 2005/09/14 05:59:03 rurban Exp $');
+<?php rcs_id('$Id: PageList.php,v 1.142 2007/07/01 09:09:19 rurban Exp $');
 
 /**
  * List a number of pagenames, optionally as table with various columns.
@@ -48,15 +48,18 @@
  * DONE: 
  *   paging support: limit, offset args
  *   check PagePerm "list" access-type,
- *   all columns are sortable (Thanks to the wikilens team).
+ *   all columns are sortable. Thanks to the wikilens team.
  *   cols > 1, comma, azhead, ordered (OL lists)
  *   ->supportedArgs() which arguments are supported, so that the plugin 
  *                     doesn't explictly need to declare it 
+ * TODO:
+ *   fix sortby logic, fix multiple sortby and other paging args per page.
+ *   info=relation,linkto nopage=1
+ *   use custom format method (RecentChanges, rss, ...)
  *
  * FIXED: 
  *   fix memory exhaustion on large pagelists with old --memory-limit php's only. 
  *   Status: improved 2004-06-25 16:19:36 rurban 
- *     but needs further testing.
  */
 class _PageList_Column_base {
     var $_tdattr = array();
@@ -74,10 +77,11 @@ class _PageList_Column_base {
     }
 
     function format ($pagelist, $page_handle, &$revision_handle) {
+    	$nbsp = HTML::raw('&nbsp;');
         return HTML::td($this->_tdattr,
-                        HTML::raw('&nbsp;'),
+                        $nbsp,
                         $this->_getValue($page_handle, $revision_handle),
-                        HTML::raw('&nbsp;'));
+                        $nbsp);
     }
 
     function getHeading () {
@@ -90,6 +94,8 @@ class _PageList_Column_base {
 
     // old-style heading
     function heading () {
+    	global $request;
+        $nbsp = HTML::raw('&nbsp;');
         // allow sorting?
         if (1 /* or in_array($this->_field, PageList::sortable_columns())*/) {
             // multiple comma-delimited sortby args: "+hits,+pagename"
@@ -98,49 +104,47 @@ class _PageList_Column_base {
             //Fixme: pass all also other GET args along. (limit, p[])
             //TODO: support GET and POST
             $s = HTML::a(array('href' => 
-                               $GLOBALS['request']->GetURLtoSelf(array('sortby' => $sortby,
-                                                                       'nocache' => '1')),
+                               $request->GetURLtoSelf(array('sortby' => $sortby)),
                                'class' => 'pagetitle',
                                'title' => sprintf(_("Sort by %s"), $this->_field)), 
-                         HTML::raw('&nbsp;'), HTML::u($this->_heading), HTML::raw('&nbsp;'));
+                         $nbsp, HTML::u($this->_heading), $nbsp);
         } else {
-            $s = HTML(HTML::raw('&nbsp;'), HTML::u($this->_heading), HTML::raw('&nbsp;'));
+            $s = HTML($nbsp, HTML::u($this->_heading), $nbsp);
         }
         return HTML::th(array('align' => 'center'),$s);
     }
 
     // new grid-style sortable heading
-    // see activeui.js 
-    function button_heading ($pagelist, $colNum) {
+    // TODO: via activeui.js ? (fast dhtml sorting)
+    function button_heading (&$pagelist, $colNum) {
         global $WikiTheme, $request;
         // allow sorting?
+        $nbsp = HTML::raw('&nbsp;');
         if (1 /* or in_array($this->_field, PageList::sortable_columns()) */) {
-            // multiple comma-delimited sortby args: "+hits,+pagename"
+            // TODO: add to multiple comma-delimited sortby args: "+hits,+pagename"
             $src = false; 
             $noimg_src = $WikiTheme->getButtonURL('no_order');
             if ($noimg_src)
-                $noimg = HTML::img(array('src' => $noimg_src,
-                                         'width' => '7', 
+                $noimg = HTML::img(array('src'    => $noimg_src,
+                                         'width'  => '7', 
                                          'height' => '7',
                                          'border' => 0,
                                          'alt'    => '.'));
             else 
-                $noimg = HTML::raw('&nbsp;');
-            if ($request->getArg('sortby')) {
-                if ($pagelist->sortby($colNum, 'check')) { // show icon?
-                    $sortby = $pagelist->sortby($request->getArg('sortby'), 'flip_order');
-                    //$request->setArg('sortby', $sortby);
-                    $desc = (substr($sortby,0,1) == '-'); // asc or desc? (+pagename, -pagename)
-                    $src = $WikiTheme->getButtonURL($desc ? 'asc_order' : 'desc_order');
-                } else {
-                    $sortby = $pagelist->sortby($colNum, 'init');
-                }
+                $noimg = $nbsp;
+            if ($pagelist->sortby($colNum, 'check')) { // show icon? request or plugin arg
+                $sortby = $pagelist->sortby($colNum, 'flip_order');
+                $desc = (substr($sortby,0,1) == '-'); // +pagename or -pagename
+                $src = $WikiTheme->getButtonURL($desc ? 'asc_order' : 'desc_order');
+                $reverse = $desc ? _("reverse")." " : "";
             } else {
-                $sortby = $pagelist->sortby($colNum, 'init');
+            	// initially unsorted
+            	$sortby = $pagelist->sortby($colNum, 'get');
             }
             if (!$src) {
                 $img = $noimg;
-                //$img->setAttr('alt', _("Click to sort"));
+                $reverse = "";
+                $img->setAttr('alt', ".");
             } else {
                 $img = HTML::img(array('src' => $src, 
                                        'width' => '7', 
@@ -149,21 +153,17 @@ class _PageList_Column_base {
                                        'alt' => _("Click to reverse sort order")));
             }
             $s = HTML::a(array('href' => 
-                               //Fixme: pass all also other GET args along. (limit is ok, p[])
-                               //Fixme: convert to POST submit[sortby]
-                               $request->GetURLtoSelf(array('sortby' => $sortby,
-                                                            /*'nocache' => '1'*/)),
+                                 //Fixme: pass all also other GET args along. (limit is ok, p[])
+                                 $request->GetURLtoSelf(array('sortby' => $sortby, 
+                                                              'id' => $pagelist->id)),
                                'class' => 'gridbutton', 
-                               'title' => sprintf(_("Click to sort by %s"), $this->_field)),
-                         HTML::raw('&nbsp;'),
-                         $noimg,
-                         HTML::raw('&nbsp;'),
-                         $this->_heading,
-                         HTML::raw('&nbsp;'),
-                         $img,
-                         HTML::raw('&nbsp;'));
+                               'title' => sprintf(_("Click to sort by %s"), $reverse . $this->_field)),
+                         $nbsp, $noimg,
+                         $nbsp, $this->_heading,
+                         $nbsp, $img,
+                         $nbsp);
         } else {
-            $s = HTML(HTML::raw('&nbsp;'), $this->_heading, HTML::raw('&nbsp;'));
+            $s = HTML($nbsp, $this->_heading, $nbsp);
         }
         return HTML::th(array('align' => 'center', 'valign' => 'middle', 
                               'class' => 'gridbutton'), $s);
@@ -345,6 +345,10 @@ class _PageList_Column_time extends _PageList_Column {
         $time = _PageList_Column::_getValue($page_handle, $revision_handle);
         return $this->Theme->formatDateTime($time);
     }
+
+    function _getSortableValue ($page_handle, &$revision_handle) {
+        return _PageList_Column::_getValue($page_handle, $revision_handle);
+    }
 };
 
 class _PageList_Column_version extends _PageList_Column {
@@ -432,31 +436,41 @@ class _PageList_Column_author extends _PageList_Column {
 
     function _getValue ($page_handle, &$revision_handle) {
         $author = _PageList_Column::_getValue($page_handle, $revision_handle);
-        if (isWikiWord($author) && $this->dbi->isWikiPage($author))
+        if ($this->dbi->isWikiPage($author))
             return WikiLink($author);
         else
             return $author;
     }
+    
+    function _getSortableValue ($page_handle, &$revision_handle) {
+        return _PageList_Column::_getValue($page_handle, $revision_handle);
+    }
 };
 
 class _PageList_Column_owner extends _PageList_Column_author {
     function _getValue ($page_handle, &$revision_handle) {
         $author = $page_handle->getOwner();
-        if (isWikiWord($author) && $this->dbi->isWikiPage($author))
+        if ($this->dbi->isWikiPage($author))
             return WikiLink($author);
         else
             return $author;
     }
+    function _getSortableValue ($page_handle, &$revision_handle) {
+        return _PageList_Column::_getValue($page_handle, $revision_handle);
+    }
 };
 
 class _PageList_Column_creator extends _PageList_Column_author {
     function _getValue ($page_handle, &$revision_handle) {
         $author = $page_handle->getCreator();
-        if (isWikiWord($author) && $this->dbi->isWikiPage($author))
+        if ($this->dbi->isWikiPage($author))
             return WikiLink($author);
         else
             return $author;
     }
+    function _getSortableValue ($page_handle, &$revision_handle) {
+        return _PageList_Column::_getValue($page_handle, $revision_handle);
+    }
 };
 
 class _PageList_Column_pagename extends _PageList_Column_base {
@@ -502,15 +516,17 @@ class PageList {
     var $_maxlen = 0;
 
     function PageList ($columns = false, $exclude = false, $options = false) {
+    	// unique id per pagelist on each page.
+    	if (!isset($GLOBALS['request']->_pagelist))
+    	    $GLOBALS['request']->_pagelist = 0;
+    	else 
+    	    $GLOBALS['request']->_pagelist++;    
+    	$this->id = $GLOBALS['request']->_pagelist;
         if ($options)
             $this->_options = $options;
 
-        // let plugins predefine only certain objects, such its own custom pagelist columns
-        if (!empty($this->_options['types'])) {
-            $this->_types = $this->_options['types'];
-            unset($this->_options['types']);
-        }
         $this->_initAvailableColumns();
+        // let plugins predefine only certain objects, such its own custom pagelist columns
         $symbolic_columns = 
             array(
                   'all' =>  array_diff(array_keys($this->_types), // all but...
@@ -528,23 +544,61 @@ class PageList {
                     $columns = array_diff(array_merge($columns,$cols),array($symbol));
                 }
             }
-            if (!in_array('pagename',$columns))
+            if (empty($this->_options['nopage']) and !in_array('pagename',$columns))
                 $this->_addColumn('pagename');
             foreach ($columns as $col) {
-                $this->_addColumn($col);
+		if (!empty($col))
+		    $this->_addColumn($col);
             }
         }
         // If 'pagename' is already present, _addColumn() will not add it again
-        $this->_addColumn('pagename');
+	if (empty($this->_options['nopage']))
+	    $this->_addColumn('pagename');
 
-        foreach (array('sortby','limit','paging','count','dosort') as $key) {
-          if (!empty($options) and !empty($options[$key])) {
-            $this->_options[$key] = $options[$key];
-          } else {
-            $this->_options[$key] = $GLOBALS['request']->getArg($key);
-          }
+	if (!empty($this->_options['types'])) {
+            foreach ($this->_options['types'] as $type) {	
+            	$this->_types[$type->_field] = $type;
+                $this->_addColumn($type->_field);
+            }
+            unset($this->_options['types']);
+        }
+
+        global $request;
+        // explicit header options: ?id=x&sortby=... override options[]
+        // support multiple sorts. check multiple, no nested elseif
+        if (($this->id == $request->getArg("id")) 
+             and $request->getArg('sortby')) 
+        {
+            // add it to the front of the sortby array
+            $this->sortby($request->getArg('sortby'), 'init');
+            $this->_options['sortby'] = $request->getArg('sortby');
+        } // plugin options
+        if (!empty($options['sortby'])) {
+	    if (empty($this->_options['sortby']))
+	        $this->_options['sortby'] = $options['sortby'];
+            $this->sortby($options['sortby'], 'init');
+	} // global options 
+	if (!isset($request->args["id"]) and $request->getArg('sortby') 
+	     and empty($this->_options['sortby'])) 
+	{
+	    $this->_options['sortby'] = $request->getArg('sortby');
+            $this->sortby($this->_options['sortby'], 'init');
+	}
+	// same as above but without the special sortby push, and mutually exclusive (elseif)
+        foreach ($this->pagingArgs() as $key) {
+            if ($key == 'sortby') continue;	
+            if (($this->id == $request->getArg("id")) 
+                and $request->getArg($key)) 
+            {
+                $this->_options[$key] = $request->getArg($key);
+            } // plugin options
+            elseif (!empty($options) and !empty($options[$key])) {
+		$this->_options[$key] = $options[$key];
+	    } // global options 
+	    elseif (!isset($request->args["id"]) and $request->getArg($key)) {
+		$this->_options[$key] = $request->getArg($key);
+	    }
         }
-        $this->_options['sortby'] = $this->sortby($this->_options['sortby'], 'init');
         if ($exclude) {
             if (is_string($exclude) and !is_array($exclude))
                 $exclude = $this->explodePageList($exclude, false,
@@ -593,13 +647,25 @@ class PageList {
                      			    // 1 if without links, 2 if with
                      'commasep' => false,   // Default: ', '
                      'ordered'  => false,   // OL or just UL lists (ignored for comma)
+		     'linkmore' => '',      // If count>0 and limit>0 display a link with 
+		     // the number of all results, linked to the given pagename.
+		     
+		     'nopage'   => false,   // for info=col omit the pagename column
                      );
     }
+    
+    function pagingArgs() {
+    	return array('sortby','limit','paging','count','dosort');
+    }
 
     function setCaption ($caption_string) {
         $this->_caption = $caption_string;
     }
 
+    function addCaption ($caption_string) {
+        $this->_caption = HTML($this->_caption," ",$caption_string);
+    }
+
     function getCaption () {
         // put the total into the caption if needed
         if (is_string($this->_caption) && strstr($this->_caption, '%d'))
@@ -633,8 +699,11 @@ class PageList {
 
     function pageNames() {
         $pages = array();
+	$limit = @$this->_options['limit'];
         foreach ($this->_pages as $page_handle) {
             $pages[] = $page_handle->getName();
+	    if ($limit and count($pages) > $limit)
+		break;
         }
         return $pages;
     }
@@ -642,7 +711,7 @@ class PageList {
     function _getPageFromHandle($page_handle) {
         if (is_string($page_handle)) {
             if (empty($page_handle)) return $page_handle;
-            //$dbi = $GLOBALS['request']->getDbh(); // no, safe memory!
+            //$dbi = $GLOBALS['request']->getDbh(); // no, safe some memory!
             $page_handle = $GLOBALS['request']->_dbi->getPage($page_handle);
         }
         return $page_handle;
@@ -676,10 +745,17 @@ class PageList {
 
         if (count($this->_columns) > 1) {
             $row = HTML::tr(array('class' => $class));
-            foreach ($this->_columns as $col)
+            $j = 0;
+            foreach ($this->_columns as $col) {
+	        $col->current_row = $i;
+	        $col->current_column = $j;
                 $row->pushContent($col->format($this, $page_handle, $revision_handle));
+                $j++;
+            }
         } else {
             $col = $this->_columns[0];
+	    $col->current_row = $i;
+	    $col->current_column = 0;
             $row = $col->_getValue($page_handle, $revision_handle);
         }
 
@@ -783,20 +859,44 @@ class PageList {
             $order = '-'; $column = substr($column,1);
         }
         // default initial order: +pagename, -mtime, -hits
-        if (empty($order))
-            if (in_array($column, array('mtime','hits')))
-                $order = '-';
-            else
-                $order = '+';
-        if ($action == 'flip_order') {
-            return ($order == '+' ? '-' : '+') . $column;
-        } elseif ($action == 'init') {
-            $this->_sortby[$column] = $order;
+        if (empty($order)) {
+            if (!empty($this->_sortby[$column]))
+                $order = $this->_sortby[$column];
+            else {
+                if (in_array($column, array('mtime','hits')))
+                    $order = '-';
+                else
+                    $order = '+';
+            }
+        }
+        if ($action == 'get') {
             return $order . $column;
-        } elseif ($action == 'check') {
-            return (!empty($this->_sortby[$column])
-                    or ($request->getArg('sortby')
-                        and strstr($request->getArg('sortby'),$column)));
+        } elseif ($action == 'flip_order') {
+            if (0 and DEBUG)
+            	trigger_error("flip $order $column ".$this->id,  E_USER_NOTICE); 
+            return ($order == '+' ? '-' : '+') . $column;
+        } elseif ($action == 'init') { // only allowed from PageList::PageList
+            if ($this->sortby($column, 'clicked')) {
+            	if (0 and DEBUG)
+            	    trigger_error("clicked $order $column $this->id",  E_USER_NOTICE); 
+                //$order = ($order == '+' ? '-' : '+'); // $this->sortby($sortby, 'flip_order');
+            }
+            $this->_sortby[$column] = $order; // forces show icon
+            return $order . $column;
+        } elseif ($action == 'check') {   // show icon?
+            //if specified via arg or if clicked
+            $show = (!empty($this->_sortby[$column]) or $this->sortby($column, 'clicked'));
+            if (0 and $show and DEBUG) {
+                trigger_error("show $order $column ".$this->id, E_USER_NOTICE);
+            }
+            return $show;	     
+        } elseif ($action == 'clicked') { // flip sort order?
+            global $request;
+            $arg = $request->getArg('sortby');
+            return ($arg
+                    and strstr($arg, $column)
+                    and (!isset($request->args['id']) 
+                         or $this->id == $request->getArg('id')));
         } elseif ($action == 'db') {
             // Performance enhancement: use native DB sort if possible.
             if (($valid_fields and in_array($column, $valid_fields))
@@ -813,8 +913,8 @@ class PageList {
     }
 
     // echo implode(":",explodeList("Test*",array("xx","Test1","Test2")));
-    function explodePageList($input, $include_empty=false, $sortby=false, 
-                             $limit=false, $exclude=false) 
+    function explodePageList($input, $include_empty=false, $sortby='', 
+                             $limit='', $exclude='') 
     {
         if (empty($input)) return array();
         // expand wildcards from list of all pages
@@ -841,10 +941,12 @@ class PageList {
             //TODO: do the sorting, normally not needed if used for exclude only
             return explode(',', $input);
         }
-    } 
+    }
 
-    function allPagesByAuthor($wildcard, $include_empty=false, $sortby=false, 
-                              $limit=false, $exclude=false) {
+    // TODO: optimize getTotal => store in count
+    function allPagesByAuthor($wildcard, $include_empty=false, $sortby='', 
+                              $limit='', $exclude='') 
+    {
         $dbi = $GLOBALS['request']->getDbh();
         $allPagehandles = $dbi->getAllPages($include_empty, $sortby, $limit, $exclude);
         $allPages = array();
@@ -869,8 +971,8 @@ class PageList {
         return $allPages;
     }
 
-    function allPagesByOwner($wildcard, $include_empty=false, $sortby=false, 
-                             $limit=false, $exclude=false) {
+    function allPagesByOwner($wildcard, $include_empty=false, $sortby='', 
+                             $limit='', $exclude='') {
         $dbi = $GLOBALS['request']->getDbh();
         $allPagehandles = $dbi->getAllPages($include_empty, $sortby, $limit, $exclude);
         $allPages = array();
@@ -894,8 +996,8 @@ class PageList {
         return $allPages;
     }
 
-    function allPagesByCreator($wildcard, $include_empty=false, $sortby=false, 
-                               $limit=false, $exclude=false) {
+    function allPagesByCreator($wildcard, $include_empty=false, $sortby='', 
+                               $limit='', $exclude='') {
         $dbi = $GLOBALS['request']->getDbh();
         $allPagehandles = $dbi->getAllPages($include_empty, $sortby, $limit, $exclude);
         $allPages = array();
@@ -1045,7 +1147,8 @@ class PageList {
                                      'rating',/*'ratingwidget',*/
                                      'coagreement', 'minmisery',
                                      /*'prediction',*/
-                                     'averagerating', 'top3recs');
+                                     'averagerating', 'top3recs', 
+                                     'relation', 'linkto');
             if (!in_array($column, $silently_ignore))
                 trigger_error(sprintf("%s: Bad column", $column), E_USER_NOTICE);
             return false;
@@ -1101,6 +1204,7 @@ class PageList {
                 assert(isset($col));
                 $revision_handle = false;
                 $aval = $col->_getSortableValue($pagea, $revision_handle);
+                $revision_handle = false;
                 $bval = $col->_getSortableValue($pageb, $revision_handle);
 
                 $cmp = $col->_compare($aval, $bval);
@@ -1123,11 +1227,13 @@ class PageList {
     function _sortPages() {
         if (count($this->_sortby) > 0) {
             $need_sort = $this->_options['dosort'];
-            foreach ($this->_sortby as $col => $dir) {
+            if (!$need_sort)
+              foreach ($this->_sortby as $col => $dir) {
                 if (! $this->sortby($col, 'db'))
                     $need_sort = true;
-            }
+              }
             if ($need_sort) { // There are some columns to sort by
+		// TODO: consider nopage
                 usort($this->_pages, array($this, '_pageCompare'));
             }
         }
@@ -1197,28 +1303,40 @@ class PageList {
     // make a table given the caption
     function _generateTable($caption) {
         if (count($this->_sortby) > 0) $this->_sortPages();
-
-        $rows = array(); $i = 0;
+        
+        // wikiadminutils hack. that's a way to pagelist non-pages
+        $rows = isset($this->_rows) ? $this->_rows : array(); $i = 0;
+        $count = $this->getTotal();
+        $do_paging = ( isset($this->_options['paging']) 
+        	       and !empty($this->_options['limit']) 
+        	       and $count 
+        	       and $this->_options['paging'] != 'none' );
+        if ($do_paging) {
+            $tokens = $this->pagingTokens($count, 
+                                           count($this->_columns), 
+                                           $this->_options['limit']);
+            if ($tokens)                               
+                $this->_pages = array_slice($this->_pages, $tokens['OFFSET'], $tokens['NUMPAGES']);
+        }
         foreach ($this->_pages as $pagenum => $page) {
             $rows[] = $this->_renderPageRow($page, $i++);
         }
-
         $table = HTML::table(array('cellpadding' => 0,
                                    'cellspacing' => 1,
                                    'border'      => 0,
-                                   'class'       => 'pagelist'));
-        if ($caption)
+                                   'class'       => 'pagelist', 
+				   ));
+        if ($caption) {
             $table->pushContent(HTML::caption(array('align'=>'top'), $caption));
+            $table->setAttr('width', '100%');
+	}
 
         //Warning: This is quite fragile. It depends solely on a private variable
         //         in ->_addColumn()
         if (!empty($this->_columns_seen['checkbox'])) {
             $table->pushContent($this->_jsFlipAll());
         }
-        $do_paging = ( isset($this->_options['paging']) 
-        	       and !empty($this->_options['limit']) 
-        	       and $this->getTotal() 
-        	       and $this->_options['paging'] != 'none' );
+
         $row = HTML::tr();
         $table_summary = array();
         $i = 1; // start with 1!
@@ -1240,9 +1358,6 @@ class PageList {
                                            join(", ", $table_summary)));
         $table->pushContent(HTML::colgroup(array('span' => count($this->_columns))));
         if ( $do_paging ) {
-            $tokens = $this->pagingTokens($this->getTotal(), 
-                                           count($this->_columns), 
-                                           $this->_options['limit']);
             if ($tokens === false) {
                 $table->pushContent(HTML::thead($row),
                                     HTML::tbody(false, $rows));
@@ -1303,11 +1418,16 @@ function flipAll(formObj) {
     // 'azhead' - support <h3> grouping into initials
     // 'ordered' - OL or UL list (not yet inherited to all plugins)
     // 'comma'  - condensed comma-list only, 1: no links, >1: with links
+    // FIXME: only unique list entries, esp. with nopage
     function _generateList($caption='') {
     	if (empty($this->_pages)) return; // stop recursion
         $out = HTML();
         if ($caption)
             $out->pushContent(HTML::p($caption));
+	// Semantic Search et al: only unique list entries, esp. with nopage
+	if (!is_array($this->_pages[0]) and is_string($this->_pages[0])) {
+	    $this->_pages = array_unique($this->_pages);
+	}
 
         // need a recursive switch here for the azhead and cols grouping.
         if (!empty($this->_options['cols']) and $this->_options['cols'] > 1) {
@@ -1386,10 +1506,11 @@ function flipAll(formObj) {
 	else    
             $list = HTML::ul(array('class' => 'pagelist'));
         $i = 0;
-        //TODO: currently we ignore limit here and hope tha the backend didn't ignore it. (BackLinks)
+        //TODO: currently we ignore limit here and hope that the backend didn't ignore it. (BackLinks)
         if (!empty($this->_options['limit']))
             list($offset, $pagesize) = $this->limit($this->_options['limit']);
-        else $pagesize=0;
+        else 
+	    $pagesize=0;
         foreach ($this->_pages as $pagenum => $page) {
             $pagehtml = $this->_renderPageRow($page);
             $group = ($i++ / $this->_group_rows);
@@ -1409,6 +1530,7 @@ function flipAll(formObj) {
     // comma=1
     // Condense list without a href links: "Page1, Page2, ..." 
     // Alternative $seperator = HTML::Raw(' &middot; ')
+    // FIXME: only unique list entries, esp. with nopage
     function _generateCommaListAsString() {
     	if (defined($this->_options['commasep']))
     	    $seperator = $this->_options['commasep'];
@@ -1426,6 +1548,7 @@ function flipAll(formObj) {
     // Normal WikiLink list.
     // Future: 1 = reserved for plain string (see above)
     //         2 and more => HTML link specialization?
+    // FIXME: only unique list entries, esp. with nopage
     function _generateCommaList($style = false) {
     	if (defined($this->_options['commasep']))
     	    $seperator = HTLM::Raw($this->_options['commasep']);
@@ -1459,7 +1582,7 @@ function flipAll(formObj) {
 class PageList_Selectable
 extends PageList {
 
-    function PageList_Selectable ($columns=false, $exclude=false, $options = false) {
+    function PageList_Selectable ($columns=false, $exclude='', $options = false) {
         if ($columns) {
             if (!is_array($columns))
                 $columns = explode(',', $columns);
@@ -1484,6 +1607,27 @@ extends PageList {
 }
 
 // $Log: PageList.php,v $
+// Revision 1.142  2007/07/01 09:09:19  rurban
+// fix PageList with multiple lists: added id, fixed sortby REQUEST logic
+//
+// Revision 1.141  2007/05/24 18:40:55  rurban
+// display list with tokens problem
+//
+// Revision 1.140  2007/05/13 18:12:55  rurban
+// force adding a options[type] column: fixes LinkDatabase
+//
+// Revision 1.139  2007/01/25 07:42:01  rurban
+// Support nopage
+//
+// Revision 1.138  2007/01/20 11:24:15  rurban
+// Support paging limit if ->_pages is not yet limited by the backend (AllPagesByMe)
+//
+// Revision 1.137  2007/01/07 18:43:08  rurban
+// Honor predefined ->_rows: hack to (re-)allow non-pagenames, used by WikiAdminUtils
+//
+// Revision 1.136  2007/01/02 13:18:46  rurban
+// filter pageNames through limit, needed for xmlrpc. publish col->current_row and col->current_column counters during iteration. use table width=100% with captions. Clarify API: sortby,limit and exclude are strings.
+//
 // Revision 1.135  2005/09/14 05:59:03  rurban
 // optimized explodePageList to use SQL when available
 //   (titleSearch instead of getAllPages)
@@ -1731,4 +1875,4 @@ extends PageList {
 // c-hanging-comment-ender-p: nil
 // indent-tabs-mode: nil
 // End:
-?>
\ No newline at end of file
+?>
