@@ -86,7 +86,7 @@ function pagePermissions($pagename) {
             return array('inherited', pagePermissions(getParentPage($pagename)));
         }
     } elseif ($perm = getPagePermissions($page)) {
-        return array('page',$perm);
+        return array('page', $perm);
     // or no permissions defined; returned inherited permissions, to be displayed in gray
     } elseif ($pagename == '.') { // stop recursion in pathological case. 
     	// "." defined, without any acl
@@ -264,7 +264,7 @@ function _requiredAuthorityForPagename($access, $pagename) {
  */
 function getParentPage($pagename) {
     if (isSubPage($pagename)) {
-        return subPageSlice($pagename,0);
+        return subPageSlice($pagename, 0);
     } else {
         return '.';
     }
@@ -359,15 +359,18 @@ class PagePermission {
      * Must translate the various special groups to the actual users settings 
      * (userid, group membership).
      */
-    function isAuthorized($access,$user) {
+    function isAuthorized($access, $user) {
         if (!empty($this->perm{$access})) {
+            $allow = -1;
             foreach ($this->perm[$access] as $group => $bool) {
-                if ($this->isMember($user,$group)) {
+                if ($this->isMember($user, $group)) {
                     return $bool;
+                } elseif ($allow == -1) { // not a member and undecided: check other groups
+                    $allow = !$bool;
                 }
             }
         }
-        return -1; // undecided
+        return $allow; // undecided
     }
 
     /**
