@@ -1,7 +1,7 @@
 <?php // -*-php-*-
-rcs_id('$Id: PageType.php,v 1.45 2005/05/06 16:48:41 rurban Exp $');
+rcs_id('$Id: PageType.php,v 1.52 2007/02/17 14:17:41 rurban Exp $');
 /*
- Copyright 1999,2000,2001,2002,2003,2004,2005 $ThePhpWikiProgrammingTeam
+ Copyright 1999,2000,2001,2002,2003,2004,2005,2006 $ThePhpWikiProgrammingTeam
 
  This file is part of PhpWiki.
 
@@ -119,9 +119,9 @@ class PageType_comment extends PageType {}
 class PageType_wikiforum extends PageType {}
 
 /* To prevent from PHP5 Fatal error: Using $this when not in object context */
-function getInterwikiMap ($pagetext = false) {
+function getInterwikiMap ($pagetext=false, $force=false) {
     static $map;
-    if (empty($map))
+    if (empty($map) or $force)
         $map = new PageType_interwikimap($pagetext);
     return $map;
 }
@@ -167,6 +167,7 @@ class PageType_interwikimap extends PageType
     }
 
     function link ($link, $linktext = false) {
+	global $WikiTheme;
         list ($moniker, $page) = split (":", $link, 2);
         
         if (!isset($this->_map[$moniker])) {
@@ -175,6 +176,20 @@ class PageType_interwikimap extends PageType
         }
 
         $url = $this->_map[$moniker];
+	// localize Upload:links for WIKIDUMP
+	if (!empty($WikiTheme->DUMP_MODE) and $moniker == 'Upload') {
+	    global $request;
+	    $url = getUploadFilePath();
+	    //calculate to a relative local path to /uploads for pdf images.
+	    $doc_root = $request->get("DOCUMENT_ROOT");
+    	    $ldir = NormalizeLocalFileName($url);
+            $wikiroot = NormalizeLocalFileName('');
+    	    if (string_starts_with($ldir, $doc_root)) {
+        	$link_prefix = substr($url, strlen($doc_root));
+    	    } elseif (string_starts_with($ldir, $wikiroot)) {
+        	$link_prefix = NormalizeWebFileName(substr($url, strlen($wikiroot)));
+	    }
+	}
         
         // Urlencode page only if it's a query arg.
         // FIXME: this is a somewhat broken heuristic.
@@ -187,7 +202,7 @@ class PageType_interwikimap extends PageType
 
         $link = HTML::a(array('href' => $url));
 
-        if (!$linktext) {
+	if (!$linktext) {
             $link->pushContent(PossiblyGlueIconToText('interwiki', "$moniker:"),
                                HTML::span(array('class' => 'wikipage'), $page));
             $link->setAttr('class', 'interwiki');
@@ -202,7 +217,7 @@ class PageType_interwikimap extends PageType
 
 
     function _parseMap ($text) {
-        if (!preg_match_all("/^\s*(\S+)\s+(\S+)/m",
+        if (!preg_match_all("/^\s*(\S+)\s+(.+)$/m",
                             $text, $matches, PREG_SET_ORDER))
             return false;
 
@@ -210,7 +225,9 @@ class PageType_interwikimap extends PageType
             $map[$m[1]] = $m[2];
         }
 
-        // Add virtual monikers: Upload:, Talk:, User:
+        // Add virtual monikers: "Upload:" "Talk:" "User:", ":"
+        // and expand special variables %u, %b, %d
+
         // Upload: Should be expanded later to user-specific upload dirs. 
         // In the Upload plugin, not here: Upload:ReiniUrban/uploaded-file.png
         if (empty($map['Upload'])) {
@@ -222,8 +239,11 @@ class PageType_interwikimap extends PageType
         if (empty($map["User"])) {
             $map["User"] = "%s";
         }
+        // Talk:UserName => UserName/Discussion
+        // Talk:PageName => PageName/Discussion as default, which might be overridden
         if (empty($map["Talk"])) {
             $pagename = $GLOBALS['request']->getArg('pagename');
+            // against PageName/Discussion/Discussion
             if (string_ends_with($pagename, SUBPAGE_SEPARATOR._("Discussion")))
                 $map["Talk"] = "%s";
             else
@@ -251,10 +271,6 @@ class PageType_interwikimap extends PageType
                                              Iso8601DateTime());
         }
 
-        // Maybe add other monikers also (SemanticWeb link predicates?)
-        // Should they be defined in a RDF? (strict mode)
-        // Or should the SemanticWeb lib add it by itself? 
-        // (adding only a subset dependent on the context = model)
         return $map;
     }
 
@@ -353,7 +369,7 @@ class PageFormatter_interwikimap extends PageFormatter
     }
     
     function _getMap($pagetext) {
-        $map = getInterwikiMap($pagetext);
+        $map = getInterwikiMap($pagetext, 'force');
         return $map->_map;
     }
     
@@ -498,7 +514,29 @@ class PageFormatter_pdf extends PageFormatter
         return $pdf;
     }
 }
+
 // $Log: PageType.php,v $
+// Revision 1.52  2007/02/17 14:17:41  rurban
+// localize Upload:links for WIKIDUMP: esp. for pdf images
+//
+// Revision 1.51  2007/01/07 18:43:17  rurban
+// Disallow ":" as interwikmap and use it as proper LinkedBracket match.
+//
+// Revision 1.50  2007/01/04 16:44:57  rurban
+// Force interwiki updates and page edits
+//
+// Revision 1.49  2006/10/12 06:25:09  rurban
+// use the same class for $moniker == ""
+//
+// Revision 1.48  2006/10/08 12:38:11  rurban
+// New special interwiki link markup [:LinkTo] without storing the backlink
+//
+// Revision 1.47  2005/08/07 09:14:38  rurban
+// fix comments
+//
+// Revision 1.46  2005/08/06 13:09:33  rurban
+// allow spaces in interwiki paths, even implicitly. fixes bug #1218733
+//
 // Revision 1.45  2005/05/06 16:48:41  rurban
 // support %u, %b, %d expansion for Upload: User: and Talk: interwiki monikers
 //
@@ -539,4 +577,4 @@ class PageFormatter_pdf extends PageFormatter
 // c-hanging-comment-ender-p: nil
 // indent-tabs-mode: nil
 // End:
-?>
\ No newline at end of file
+?>
