@@ -1,5 +1,5 @@
-<?php rcs_id('$Id: Theme.php,v 1.131 2005/06/10 06:09:06 rurban Exp $');
-/* Copyright (C) 2002,2004,2005 $ThePhpWikiProgrammingTeam
+<?php rcs_id('$Id: Theme.php,v 1.142 2007/07/01 09:36:09 rurban Exp $');
+/* Copyright (C) 2002,2004,2005,2006 $ThePhpWikiProgrammingTeam
  *
  * This file is part of PhpWiki.
  * 
@@ -179,29 +179,35 @@ function WikiLink ($page_or_rev, $type = 'known', $label = false) {
  * </dl>
  * ($Page_or_rev is ignored for submit buttons.)
  */
-function Button ($action, $label = false, $page_or_rev = false) {
+function Button ($action, $label = false, $page_or_rev = false, $options = false) {
     global $WikiTheme;
 
     if (!is_array($action) && preg_match('/^submit:(.*)/', $action, $m))
-        return $WikiTheme->makeSubmitButton($label, $m[1], $page_or_rev);
+        return $WikiTheme->makeSubmitButton($label, $m[1], $page_or_rev, $options);
     else
-        return $WikiTheme->makeActionButton($action, $label, $page_or_rev);
+        return $WikiTheme->makeActionButton($action, $label, $page_or_rev, $options);
 }
 
-
 class Theme {
     var $HTML_DUMP_SUFFIX = '';
     var $DUMP_MODE = false, $dumped_images, $dumped_css; 
 
-    function Theme ($theme_name = 'default') {
+    /**
+     * noinit: Do not initialize unnecessary items in default_theme fallback twice.
+     */
+    function Theme ($theme_name = 'default', $noinit = false) {
         $this->_name = $theme_name;
         $this->_themes_dir = NormalizeLocalFileName("themes");
         $this->_path  = defined('PHPWIKI_DIR') ? NormalizeLocalFileName("") : "";
         $this->_theme = "themes/$theme_name";
 
         if ($theme_name != 'default')
-            $this->_default_theme = new Theme;
+            $this->_default_theme = new Theme('default',true);
+        if ($noinit) return;
 
+	$this->addMoreHeaders(JavaScript("var data_path = '". DATA_PATH ."'\n"
+					."var pagename  = '". $GLOBALS['request']->getArg('pagename') ."'\n"
+					."var stylepath = '". DATA_PATH . '/'.$this->_theme."/'\n"));
         // by pixels
         if ((is_object($GLOBALS['request']) // guard against unittests
              and $GLOBALS['request']->getPref('doubleClickEdit'))
@@ -212,6 +218,10 @@ class Theme {
         if (ENABLE_LIVESEARCH) { // by bitflux.ch
             $this->initLiveSearch();
         }
+        // replaces external LiveSearch
+        if (defined("ENABLE_ACDROPDOWN") and ENABLE_ACDROPDOWN) { 
+            $this->initMoAcDropDown();
+        }
         $this->_css = array();
     }
 
@@ -459,16 +469,17 @@ class Theme {
         }
     }
 
-    function getAuthorMessage ($revision, $only_authenticated = true) {
+    /* New behaviour: (by Matt Brown)
+       Prefer author (name) over internal author_id (IP) */
+    function getAuthorMessage ($revision) {
         if (!$revision) return '';
         $dbi =& $GLOBALS['request']->_dbi;
-        $author = $revision->get('author_id');
-        if ( $author or $only_authenticated ) {
-            if (!$author) $author = $revision->get('author');
+        $author = $revision->get('author');
+        if (!$author) $author = $revision->get('author_id');
             if (!$author) return '';
-            if ( $dbi->isWikiPage($author) )
+        if ( $dbi->isWikiPage($author) ) {
                 return fmt("by %s", WikiLink($author));
-            else
+        } else {
                 return fmt("by %s", '"'.$author.'"');
         }
     }
@@ -640,7 +651,9 @@ class Theme {
      */
     function addImageAlias ($alias, $image_name) {
         // fall back to the PhpWiki-supplied image if not found
-        if ($this->_findFile("images/$image_name", true))
+	if ((empty($this->_imageAliases[$alias])
+	       and $this->_findFile("images/$image_name", true))
+	    or $image_name === false)
             $this->_imageAliases[$alias] = $image_name;
     }
 
@@ -799,28 +812,29 @@ class Theme {
     //
     ////////////////////////////////////////////////////////////////
 
-    function makeButton ($text, $url, $class = false) {
+    function makeButton ($text, $url, $class = false, $options = false) {
         // FIXME: don't always try for image button?
 
         // Special case: URLs like 'submit:preview' generate form
         // submission buttons.
         if (preg_match('/^submit:(.*)$/', $url, $m))
-            return $this->makeSubmitButton($text, $m[1], $class);
+            return $this->makeSubmitButton($text, $m[1], $class, $options);
 
         $imgurl = $this->getButtonURL($text);
         if ($imgurl)
-            return new ImageButton($text, $url, $class, $imgurl);
+            return new ImageButton($text, $url, $class, $imgurl, $options);
         else
-            return new Button($this->maybeSplitWikiWord($text), $url, $class);
+            return new Button($this->maybeSplitWikiWord($text), $url, 
+                              $class, $options);
     }
 
-    function makeSubmitButton ($text, $name, $class = false) {
+    function makeSubmitButton ($text, $name, $class = false, $options = false) {
         $imgurl = $this->getButtonURL($text);
 
         if ($imgurl)
-            return new SubmitImageButton($text, $name, $class, $imgurl);
+            return new SubmitImageButton($text, $name, $class, $imgurl, $options);
         else
-            return new SubmitButton($text, $name, $class);
+            return new SubmitButton($text, $name, $class, $options);
     }
 
     /**
@@ -847,7 +861,7 @@ class Theme {
      *
      * @return object A Button object.
      */
-    function makeActionButton ($action, $label = false, $page_or_rev = false) {
+    function makeActionButton ($action, $label = false, $page_or_rev = false, $options = false) {
         extract($this->_get_name_and_rev($page_or_rev));
 
         if (is_array($action)) {
@@ -867,9 +881,39 @@ class Theme {
         if ($action == 'browse')
             unset($attr['action']);
 
-        return $this->makeButton($label, WikiURL($pagename, $attr), $class);
+        $options = $this->fixAccesskey($options);
+
+        return $this->makeButton($label, WikiURL($pagename, $attr), $class, $options);
+    }
+    
+    function tooltipAccessKeyPrefix() {
+	static $tooltipAccessKeyPrefix = null;
+	if ($tooltipAccessKeyPrefix) return $tooltipAccessKeyPrefix;
+
+        $tooltipAccessKeyPrefix = 'alt';
+	if (isBrowserOpera()) $tooltipAccessKeyPrefix = 'shift-esc';
+	elseif (isBrowserSafari() or browserDetect("Mac") or isBrowserKonqueror()) 
+	    $tooltipAccessKeyPrefix = 'ctrl';
+	// ff2 win and x11 only
+	elseif ((browserDetect("firefox/2") or browserDetect("minefield/3") or browserDetect("SeaMonkey/1.1")) 
+		and ((browserDetect("windows") or browserDetect("x11"))))
+	    $tooltipAccessKeyPrefix = 'alt-shift'; 
+	return $tooltipAccessKeyPrefix;
     }
 
+    /** Define the accesskey in the title only, with ending [p] or [alt-p].
+     *  This fixes the prefix in the title and sets the accesskey.
+     */
+    function fixAccesskey($attrs) {
+        if (!empty($attrs['title']) and preg_match("/\[(alt-)?(.)\]$/", $attrs['title'], $m))
+        {
+    	    if (empty($attrs['accesskey'])) $attrs['accesskey'] = $m[2];
+    	    // firefox 'alt-shift', MSIE: 'alt', ... see wikibits.js
+    	    $attrs['title'] = preg_replace("/\[(alt-)?(.)\]$/", "[".$this->tooltipAccessKeyPrefix()."-\\2]", $attrs['title']);
+        }
+        return $attrs;
+    }
+    
     /**
      * Make a "button" which links to a wiki-page.
      *
@@ -1070,9 +1114,13 @@ class Theme {
         foreach ($this->_css as $title => $css_files) {
             ksort($css_files); // move $css_files[''] to front.
             foreach ($css_files as $media => $css_file) {
-                $css[] = $this->_CSSlink($title, $css_file, $media, $is_alt);
+		if (!empty($this->DUMP_MODE)) {
+		    if ($media == 'print')
+			$css[] = $this->_CSSlink($title, $css_file, '', $is_alt);
+		} else {
+		    $css[] = $this->_CSSlink($title, $css_file, $media, $is_alt);
+		}
                 if ($is_alt) break;
-                
             }
             $is_alt = true;
         }
@@ -1084,10 +1132,10 @@ class Theme {
             return $this->_path . $tmp;
         else {
             $f1 = $this->file("templates/$name.tmpl");
-            trigger_error("pwd: ".getcwd(), E_USER_ERROR);
+            //trigger_error("findTemplate($name) pwd: ".getcwd(), E_USER_ERROR);
             if (isset($this->_default_theme)) {
                $f2 = $this->_default_theme->file("templates/$name.tmpl");
-               trigger_error("$f1 nor $f2 found", E_USER_ERROR);
+               //trigger_error("$f1 nor $f2 found", E_USER_ERROR);
             } else 
                trigger_error("$f1 not found", E_USER_ERROR);
             return false;
@@ -1096,9 +1144,21 @@ class Theme {
 
     var $_MoreHeaders = array();
     function addMoreHeaders ($element) {
-        array_push($this->_MoreHeaders, $element);
+        $this->_MoreHeaders[] = $element;
+        if (!empty($this->_headers_printed) and $this->_headers_printed) {
+	    trigger_error(_("Some action(page) wanted to add more headers, but they were already printed.")
+			  ."\n". $element->asXML(),
+                           E_USER_NOTICE);
+        }
     }
+
+    /**
+      * Singleton. Only called once, by the head template. See the warning above.
+      */
     function getMoreHeaders () {
+    	// actionpages cannot add headers, because recursive template expansion
+    	// already expanded the head template before.
+    	$this->_headers_printed = 1;
         if (empty($this->_MoreHeaders))
             return '';
         $out = '';
@@ -1140,6 +1200,130 @@ class Theme {
         return $out;
     }
 
+    /**
+     * Common Initialisations
+     */
+
+    /**
+     * The ->load() method replaces the formerly global code in themeinfo.php.
+     * Without this you would not be able to derive from other themes.
+     */
+    function load() {
+
+	// CSS file defines fonts, colors and background images for this
+	// style.  The companion '*-heavy.css' file isn't defined, it's just
+	// expected to be in the same directory that the base style is in.
+
+	// This should result in phpwiki-printer.css being used when
+	// printing or print-previewing with style "PhpWiki" or "MacOSX" selected.
+	$this->setDefaultCSS('PhpWiki',
+			     array(''      => 'phpwiki.css',
+				   'print' => 'phpwiki-printer.css'));
+
+	// This allows one to manually select "Printer" style (when browsing page)
+	// to see what the printer style looks like.
+	$this->addAlternateCSS(_("Printer"), 'phpwiki-printer.css', 'print, screen');
+	$this->addAlternateCSS(_("Top & bottom toolbars"), 'phpwiki-topbottombars.css');
+	$this->addAlternateCSS(_("Modern"), 'phpwiki-modern.css');
+
+	if (isBrowserIE()) {
+	    $this->addMoreHeaders($this->_CSSlink(0,
+						  $this->_findFile('IEFixes.css'),'all'));
+	    $this->addMoreHeaders("\n");
+	}
+
+	/**
+	 * The logo image appears on every page and links to the HomePage.
+	 */
+	$this->addImageAlias('logo', WIKI_NAME . 'Logo.png');
+	
+	$this->addImageAlias('search', 'search.png');
+
+	/**
+	 * The Signature image is shown after saving an edited page. If this
+	 * is set to false then the "Thank you for editing..." screen will
+	 * be omitted.
+	 */
+
+	$this->addImageAlias('signature', WIKI_NAME . "Signature.png");
+	// Uncomment this next line to disable the signature.
+	//$this->addImageAlias('signature', false);
+
+	/*
+	 * Link icons.
+	 */
+	$this->setLinkIcon('http');
+	$this->setLinkIcon('https');
+	$this->setLinkIcon('ftp');
+	$this->setLinkIcon('mailto');
+	$this->setLinkIcon('interwiki');
+	$this->setLinkIcon('wikiuser');
+	$this->setLinkIcon('*', 'url');
+
+	$this->setButtonSeparator("\n | ");
+
+	/**
+	 * WikiWords can automatically be split by inserting spaces between
+	 * the words. The default is to leave WordsSmashedTogetherLikeSo.
+	 */
+	$this->setAutosplitWikiWords(false);
+
+	/**
+	 * Layout improvement with dangling links for mostly closed wiki's:
+	 * If false, only users with edit permissions will be presented the 
+	 * special wikiunknown class with "?" and Tooltip.
+	 * If true (default), any user will see the ?, but will be presented 
+	 * the PrintLoginForm on a click.
+	 */
+	//$this->setAnonEditUnknownLinks(false);
+
+	/*
+	 * You may adjust the formats used for formatting dates and times
+	 * below.  (These examples give the default formats.)
+	 * Formats are given as format strings to PHP strftime() function See
+	 * http://www.php.net/manual/en/function.strftime.php for details.
+	 * Do not include the server's zone (%Z), times are converted to the
+	 * user's time zone.
+	 *
+	 * Suggestion for french: 
+	 *   $this->setDateFormat("%A %e %B %Y");
+	 *   $this->setTimeFormat("%H:%M:%S");
+	 * Suggestion for capable php versions, using the server locale:
+	 *   $this->setDateFormat("%x");
+	 *   $this->setTimeFormat("%X");
+	 */
+	//$this->setDateFormat("%B %d, %Y");
+	//$this->setTimeFormat("%I:%M %p");
+
+	/*
+	 * To suppress times in the "Last edited on" messages, give a
+	 * give a second argument of false:
+	 */
+	//$this->setDateFormat("%B %d, %Y", false); 
+
+
+	/**
+	 * Custom UserPreferences:
+	 * A list of name => _UserPreference class pairs.
+	 * Rationale: Certain themes should be able to extend the predefined list 
+	 * of preferences. Display/editing is done in the theme specific userprefs.tmpl
+	 * but storage/sanification/update/... must be extended to the Get/SetPreferences methods.
+	 * See themes/wikilens/themeinfo.php
+	 */
+	//$this->customUserPreference(); 
+
+	/**
+	 * Register custom PageList type and define custom PageList classes.
+	 * Rationale: Certain themes should be able to extend the predefined list 
+	 * of pagelist types. E.g. certain plugins, like MostPopular might use 
+	 * info=pagename,hits,rating
+	 * which displays the rating column whenever the wikilens theme is active.
+	 * See themes/wikilens/themeinfo.php
+	 */
+	//$this->addPageListColumn(); 
+
+    } // end of load
+
     /**
      * Custom UserPreferences:
      * A list of name => _UserPreference class pairs.
@@ -1177,19 +1361,162 @@ class Theme {
             $this->addMoreAttr('body', 'DoubleClickEdit', HTML::Raw(" ondblclick=\"url = document.URL; url2 = url; if (url.indexOf('?') != -1) url2 = url.slice(0, url.indexOf('?')); if ((url.indexOf('action') == -1) || (url.indexOf('action=browse') != -1)) document.location = url2 + '?action=edit';\""));
     }
 
-    // Immediate title search results via XMLHTML(HttpRequest
+    // Immediate title search results via XMLHTML(HttpRequest)
     // by Bitflux GmbH, bitflux.ch. You need to install the livesearch.js seperately.
     // Google's or acdropdown is better.
     function initLiveSearch() {
-        if (!$this->HTML_DUMP_SUFFIX) {
+	//subclasses of Sidebar will init this twice 
+	static $already = 0;
+        if (!$this->HTML_DUMP_SUFFIX and !$already) {
             $this->addMoreAttr('body', 'LiveSearch', 
                                HTML::Raw(" onload=\"liveSearchInit()"));
             $this->addMoreHeaders(JavaScript('var liveSearchURI="'
                                              .WikiURL(_("TitleSearch"),false,true).'";'));
             $this->addMoreHeaders(JavaScript('', array
                                              ('src' => $this->_findData('livesearch.js'))));
+	    $already = 1;
         }
     }
+
+    // Immediate title search results via XMLHttpRequest
+    // using the shipped moacdropdown js-lib
+    function initMoAcDropDown() {
+	//subclasses of Sidebar will init this twice 
+	static $already = 0;
+        if (!$this->HTML_DUMP_SUFFIX and !$already) {
+            $dir = $this->_findData('moacdropdown');
+            // if autocomplete_remote is used: (getobject2 also for calc. the showlist width)
+	    if (DEBUG) {
+		foreach (array("mobrowser.js","modomevent3.js","modomt.js",
+			       "modomext.js","getobject2.js","xmlextras.js") as $js) 
+		{
+		    $this->addMoreHeaders(JavaScript('', array('src' => "$dir/js/$js")));
+		}
+		$this->addMoreHeaders(JavaScript('', array('src' => "$dir/js/acdropdown.js")));
+	    } else {
+		$this->addMoreHeaders(JavaScript('', array('src' => DATA_PATH . "/themes/default/moacdropdown.js")));
+	    }
+            //$this->addMoreHeaders($this->_CSSlink(0, 
+            //                      $this->_findFile('moacdropdown/css/dropdown.css'), 'all'));
+            $this->addMoreHeaders(HTML::style("  @import url( $dir/css/dropdown.css );\n"));
+	    /*
+	    // for local xmlrpc requests
+	    $xmlrpc_url = deduce_script_name();
+	    //if (1 or DATABASE_TYPE == 'dba')
+	    $xmlrpc_url = DATA_PATH . "/RPC2.php";
+	    if ((DEBUG & _DEBUG_REMOTE) and isset($_GET['start_debug']))
+		$xmlrpc_url .= ("?start_debug=".$_GET['start_debug']);
+            $this->addMoreHeaders(JavaScript("var xmlrpc_url = '$xmlrpc_url'"));
+	    */
+	    $already = 1;
+        }
+    }
+
+    function calendarLink($date = false) {
+        return $this->calendarBase() . SUBPAGE_SEPARATOR . 
+               strftime("%Y-%m-%d", $date ? $date : time());
+    }
+
+    function calendarBase() {
+        static $UserCalPageTitle = false;
+        global $request;
+
+        if (!$UserCalPageTitle) 
+            $UserCalPageTitle = $request->_user->getId() . 
+                                SUBPAGE_SEPARATOR . _("Calendar");
+        if (!$UserCalPageTitle)
+            $UserCalPageTitle = (BLOG_EMPTY_DEFAULT_PREFIX ? '' 
+                                 : ($request->_user->getId() . SUBPAGE_SEPARATOR)) . "Blog";
+        return $UserCalPageTitle;
+    }
+
+    function calendarInit($force = false) {
+        $dbi = $GLOBALS['request']->getDbh();
+        // display flat calender dhtml in the sidebar
+        if ($force or $dbi->isWikiPage($this->calendarBase())) {
+            $jslang = @$GLOBALS['LANG'];
+            $this->addMoreHeaders
+                (
+                 $this->_CSSlink(0, 
+                                 $this->_findFile('jscalendar/calendar-phpwiki.css'), 'all'));
+            $this->addMoreHeaders
+                (JavaScript('',
+                            array('src' => $this->_findData('jscalendar/calendar'.(DEBUG?'':'_stripped').'.js'))));
+            if (!($langfile = $this->_findData("jscalendar/lang/calendar-$jslang.js")))
+                $langfile = $this->_findData("jscalendar/lang/calendar-en.js");
+            $this->addMoreHeaders(JavaScript('',array('src' => $langfile)));
+            $this->addMoreHeaders
+                (JavaScript('',
+                            array('src' => 
+                                  $this->_findData('jscalendar/calendar-setup'.(DEBUG?'':'_stripped').'.js'))));
+
+            // Get existing date entries for the current user
+            require_once("lib/TextSearchQuery.php");
+            $iter = $dbi->titleSearch(new TextSearchQuery("^".$this->calendarBase().SUBPAGE_SEPARATOR, true, "auto"));
+            $existing = array();
+            while ($page = $iter->next()) {
+                if ($page->exists())
+                    $existing[] = basename($page->_pagename);
+            }
+            if (!empty($existing)) {
+                $js_exist = '{"'.join('":1,"',$existing).'":1}';
+                //var SPECIAL_DAYS = {"2004-05-11":1,"2004-05-12":1,"2004-06-01":1}
+                $this->addMoreHeaders(JavaScript('
+// This table holds the existing calender entries for the current user
+// calculated from the database
+var SPECIAL_DAYS = '.$js_exist.';
+// This function returns true if the date exists in SPECIAL_DAYS
+function dateExists(date, y, m, d) {
+    var year = date.getFullYear();
+    m = m + 1;
+    m = m < 10 ? "0" + m : m;  // integer, 0..11
+    d = d < 10 ? "0" + d : d;  // integer, 1..31
+    var date = year+"-"+m+"-"+d;
+    var exists = SPECIAL_DAYS[date];
+    if (!exists) return false;
+    else return true;
+}
+// This is the actual date status handler. 
+// Note that it receives the date object as well as separate 
+// values of year, month and date.
+function dateStatusFunc(date, y, m, d) {
+    if (dateExists(date, y, m, d)) return "existing";
+    else return false;
+}'));
+            }
+            else {
+                $this->addMoreHeaders(JavaScript('
+function dateStatusFunc(date, y, m, d) { return false;}'));
+            }
+        }
+    }
+
+    ////////////////////////////////////////////////////////////////
+    //
+    // Events
+    //
+    ////////////////////////////////////////////////////////////////
+
+    /* Callback when a new user creates or edits a page */
+    function CbNewUserEdit (&$request, $userid) {
+	; // i.e. create homepage with Template/UserPage
+    }
+
+    /* Callback when a new user logs in. 
+       What is new? We only record changes, not logins.
+       Should we track user actions?
+       Let's say user without homepage.
+    */
+    function CbNewUserLogin (&$request, $userid) {
+	; // do nothing
+    }
+
+    /* Callback when a user logs out
+    */
+    function CbUserLogout (&$request, $userid) {
+	; // do nothing
+    }
+
 };
 
 
@@ -1205,8 +1532,9 @@ class Button extends HtmlElement {
      * @param $text string The text for the button.
      * @param $url string The url (href) for the button.
      * @param $class string The CSS class for the button.
+     * @param $options array Additional attributes for the &lt;input&gt; tag.
      */
-    function Button ($text, $url, $class = false) {
+    function Button ($text, $url, $class=false, $options=false) {
         global $request;
         //php5 workaround
         if (check_php_version(5)) {
@@ -1218,6 +1546,10 @@ class Button extends HtmlElement {
             $this->setAttr('class', $class);
         if ($request->getArg('frame'))
             $this->setAttr('target', '_top');
+        if (!empty($options)) {
+            foreach ($options as $key => $val)
+                $this->setAttr($key, $val);
+        }
         // Google honors this
         if (in_array(strtolower($text), array('edit','create','diff','pdf'))
             and !$request->_user->isAuthenticated())
@@ -1240,7 +1572,7 @@ class ImageButton extends Button {
      * @param $img_url string URL for button's image.
      * @param $img_attr array Additional attributes for the &lt;img&gt; tag.
      */
-    function ImageButton ($text, $url, $class, $img_url, $img_attr = false) {
+    function ImageButton ($text, $url, $class, $img_url, $img_attr=false) {
         $this->__construct('a', array('href' => $url));
         if ($class)
             $this->setAttr('class', $class);
@@ -1268,14 +1600,19 @@ class SubmitButton extends HtmlElement {
      * @param $text string The text for the button.
      * @param $name string The name of the form field.
      * @param $class string The CSS class for the button.
+     * @param $options array Additional attributes for the &lt;input&gt; tag.
      */
-    function SubmitButton ($text, $name = false, $class = false) {
+    function SubmitButton ($text, $name=false, $class=false, $options=false) {
         $this->__construct('input', array('type' => 'submit',
                                           'value' => $text));
         if ($name)
             $this->setAttr('name', $name);
         if ($class)
             $this->setAttr('class', $class);
+        if (!empty($options)) {
+            foreach ($options as $key => $val)
+                $this->setAttr($key, $val);
+        }
     }
 
 };
@@ -1293,7 +1630,7 @@ class SubmitImageButton extends SubmitButton {
      * @param $img_url string URL for button's image.
      * @param $img_attr array Additional attributes for the &lt;img&gt; tag.
      */
-    function SubmitImageButton ($text, $name = false, $class = false, $img_url) {
+    function SubmitImageButton ($text, $name=false, $class=false, $img_url, $img_attr=false) {
         $this->__construct('input', array('type'  => 'image',
                                           'src'   => $img_url,
                                           'value' => $text,
@@ -1302,6 +1639,10 @@ class SubmitImageButton extends SubmitButton {
             $this->setAttr('name', $name);
         if ($class)
             $this->setAttr('class', $class);
+        if (!empty($img_attr)) {
+            foreach ($img_attr as $key => $val)
+                $this->setAttr($key, $val);
+        }
     }
 
 };
@@ -1341,6 +1682,8 @@ class PluginSidebarBox extends SidebarBox {
     var $_plugin, $_args = false, $_basepage = false;
 
     function PluginSidebarBox($name, $args = false, $basepage = false) {
+	require_once("lib/WikiPlugin.php");
+
         $loader = new WikiPluginLoader();
         $plugin = $loader->getPlugin($name);
         if (!$plugin) {
@@ -1376,8 +1719,9 @@ class RelatedLinksBox extends SidebarBox {
         $counter = 0;
         $sp = HTML::Raw('&middot; ');
         foreach ($page_content->getWikiPageLinks() as $link) {
-            if (!$request->_dbi->isWikiPage($link)) continue;
-            $this->body->pushContent($sp, WikiLink($link), HTML::br());
+            $linkto = $link['linkto'];
+            if (!$request->_dbi->isWikiPage($linkto)) continue;
+            $this->body->pushContent($sp, WikiLink($linkto), HTML::br());
             $counter++;
             if ($limit and $counter > $limit) continue;
         }
@@ -1394,8 +1738,9 @@ class RelatedExternalLinksBox extends SidebarBox {
         $counter = 0;
         $sp = HTML::Raw('&middot; ');
         foreach ($cache->getWikiPageLinks() as $link) {
-            if ($link) {
-                $this->body->pushContent($sp, WikiLink($link), HTML::br());
+            $linkto = $link['linkto'];
+            if ($linkto) {
+                $this->body->pushContent($sp, WikiLink($linkto), HTML::br());
                 $counter++;
                 if ($limit and $counter > $limit) continue;
             }
@@ -1411,9 +1756,9 @@ function listAvailableThemes() {
     $dir = dir($dir_root);
     if ($dir) {
         while($entry = $dir->read()) {
-            if (is_dir($dir_root.'/'.$entry)
-                && (substr($entry,0,1) != '.')
-                && $entry != 'CVS') {
+            if (is_dir($dir_root.'/'.$entry) 
+                and file_exists($dir_root.'/'.$entry.'/themeinfo.php')) 
+            {
                 array_push($available_themes, $entry);
             }
         }
@@ -1429,10 +1774,7 @@ function listAvailableLanguages() {
         $dir_root = PHPWIKI_DIR . "/$dir_root";
     if ($dir = dir($dir_root)) {
         while($entry = $dir->read()) {
-            if (is_dir($dir_root."/".$entry)
-                && (substr($entry,0,1) != '.')
-                && $entry != 'po'
-                && $entry != 'CVS') 
+            if (is_dir($dir_root."/".$entry) and is_dir($dir_root.'/'.$entry.'/LC_MESSAGES'))
             {
                 array_push($available_languages, $entry);
             }
@@ -1443,8 +1785,36 @@ function listAvailableLanguages() {
 }
 
 // $Log: Theme.php,v $
+// Revision 1.142  2007/07/01 09:36:09  rurban
+// themes are now easier derivable classes from other themes.
+// removed global code setters, switched to $WikiTheme->load() in main
+//
+// Revision 1.141  2007/06/02 18:24:30  rurban
+// Added accesskeys. Added js stylepath
+//
+// Revision 1.140  2007/03/10 18:27:20  rurban
+// Patch 1677965 by Erwann Penet
+//
+// Revision 1.138  2007/01/07 18:43:37  rurban
+// Remove fatals when no intermediate template was found. Deprecate subclasses of subthemes (blog of Sidebar). Warn about lost _MoreHeaders. $noinit: Do not initialize unnecessary items in default_theme fallback twice. Move over calendarInit from themes/Sidebar/themeinfo.php.
+//
+// Revision 1.137  2007/01/02 13:19:23  rurban
+// add getobject2.js to acdropdown to calculate the sizes. add global xmlrpc_url for local xml requests (interface simplification). stabilize theme and language detection: require magic files there
+//
+// Revision 1.136  2006/12/06 22:07:31  rurban
+// Add new Button argument to override any option.
+//
+// Revision 1.135  2006/04/17 17:28:21  rurban
+// honor getWikiPageLinks change linkto=>relation
+//
+// Revision 1.134  2006/03/19 16:24:38  rurban
+// fix syntax error with patch #1377650
+//
+// Revision 1.133  2006/03/19 14:24:38  rurban
+// sf.net patch #1377650 by Matt Brown:  Always prefer legible author name over author_id property
+//
 // Revision 1.132  2005/07/24 09:51:22  rurban
-// guard doubleClickEdit for unittests
+// guard doubleClickEdit for unittests, add AcDropDown support
 //
 // Revision 1.131  2005/06/10 06:09:06  rurban
 // enable getPref('doubleClickEdit')
@@ -1798,4 +2168,4 @@ function listAvailableLanguages() {
 // c-hanging-comment-ender-p: nil
 // indent-tabs-mode: nil
 // End:
-?>
\ No newline at end of file
+?>
