@@ -1,5 +1,5 @@
 <?php // -*-php-*-
-rcs_id('$Id: backend.php,v 1.25 2005/09/11 13:23:21 rurban Exp $');
+rcs_id('$Id: backend.php,v 1.33 2007/06/07 21:35:04 rurban Exp $');
 
 /*
   Pagedata
@@ -271,7 +271,7 @@ class WikiDB_backend
      * @return object A WikiDB_backend_iterator.
      */
     function get_links($pagename, $reversed, $include_empty=false,
-                       $sortby=false, $limit=false, $exclude=false) {
+                       $sortby='', $limit='', $exclude='') {
         //FIXME: implement simple (but slow) link finder.
         die("FIXME get_links");
     }
@@ -306,7 +306,7 @@ class WikiDB_backend
      *
      * @return object A WikiDB_backend_iterator.
      */
-    function get_all_pages($include_defaulted, $orderby=false, $limit=false, $exclude=false) {
+    function get_all_pages($include_defaulted, $orderby=false, $limit='', $exclude='') {
         trigger_error("virtual", E_USER_ERROR);
     }
         
@@ -328,7 +328,9 @@ class WikiDB_backend
      *
      * @see WikiDB::titleSearch
      */
-    function text_search($search, $fulltext=false, $sortby=false, $limit=false, $exclude=false) {
+    function text_search($search, $fulltext=false, $sortby='', 
+			 $limit='', $exclude='') 
+    {
         // This is method implements a simple linear search
         // through all the pages in the database.
         //
@@ -342,6 +344,25 @@ class WikiDB_backend
                                                             'exclude' => $exclude));
     }
 
+
+    /**
+     *
+     * @access protected
+     * @param $pages     object A TextSearchQuery object.
+     * @param $linkvalue object A TextSearchQuery object for the linkvalues 
+     *                          (linkto, relation or backlinks or attribute values).
+     * @param $linktype  string One of the 4 linktypes.
+     * @param $relation  object A TextSearchQuery object or false.
+     * @param $options   array Currently ignored. hash of sortby, limit, exclude.
+     * @return object A WikiDB_backend_iterator.
+     * @see WikiDB::linkSearch
+     */
+    function link_search( $pages, $linkvalue, $linktype, $relation=false, $options=array() ) {
+        include_once('lib/WikiDB/backend/dumb/LinkSearchIter.php');
+        $pageiter = $this->text_search($pages);
+        return new WikiDB_backend_dumb_LinkSearchIter($this, $pageiter, $linkvalue, $linktype, $relation, $options);
+    }
+
     /**
      * Find pages with highest hit counts.
      *
@@ -349,7 +370,7 @@ class WikiDB_backend
      * be returned in reverse order by hit count.
      *
      * @access protected
-     * @param $limit integer  No more than this many pages
+     * @param integer $limit No more than this many pages
      * @return object A WikiDB_backend_iterator.
      */
     function most_popular($limit, $sortby='-hits') {
@@ -384,7 +405,7 @@ class WikiDB_backend
         return new WikiDB_backend_dumb_MostRecentIter($this, $pages, $params);
     }
 
-    function wanted_pages($exclude_from='', $exclude='', $sortby=false, $limit=false) {
+    function wanted_pages($exclude_from='', $exclude='', $sortby='', $limit='') {
         include_once('lib/WikiDB/backend/dumb/WantedPagesIter.php');
         $allpages = $this->get_all_pages(true,false,false,$exclude_from);
         return new WikiDB_backend_dumb_WantedPagesIter($this, $allpages, $exclude, $sortby, $limit);
@@ -451,7 +472,8 @@ class WikiDB_backend
     }
 
     /**
-     * Put the database into a consistent state.
+     * Put the database into a consistent state 
+     * by reparsing and restoring all pages.
      *
      * This should put the database into a consistent state.
      * (I.e. rebuild indexes, etc...)
@@ -459,6 +481,24 @@ class WikiDB_backend
      * @return boolean True iff successful.
      */
     function rebuild() {
+	global $request;
+	$dbh = $request->getDbh();
+    	$iter = $dbh->getAllPages(false);
+        while ($page = $iter->next()) {
+	    $current = $page->getCurrentRevision(true);
+	    $pagename = $page->getName();
+	    $meta = $current->_data;
+	    $version = $current->getVersion();
+	    $content =& $meta['%content'];
+	    $formatted = new TransformedText($page, $content, $current->getMetaData());
+	    $type = $formatted->getType();
+	    $meta['pagetype'] = $type->getName();
+	    $links = $formatted->getWikiPageLinks(); // linkto => relation
+	    $this->lock(array('version','page','recent','link','nonempty'));
+	    $this->set_versiondata($pagename, $version, $meta);
+	    $this->set_links($pagename, $links);
+	    $this->unlock(array('version','page','recent','link','nonempty'));
+        }
     }
 
     function _parse_searchwords($search) {
@@ -560,6 +600,33 @@ class WikiDB_backend
     function isSQL () {
         return in_array(DATABASE_TYPE, array('SQL','ADODB','PDO'));
     }
+
+    function write_accesslog(&$entry) {
+        global $request;
+        $dbh = &$this->_dbh;
+        $log_tbl = $entry->_accesslog->logtable;
+        // duration problem: sprintf "%f" might use comma e.g. "100,201" in european locales
+        $dbh->query("INSERT INTO $log_tbl"
+                    . " (time_stamp,remote_host,remote_user,request_method,request_line,request_uri,"
+                    .   "request_args,request_time,status,bytes_sent,referer,agent,request_duration)"
+                    . " VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?)",
+                    array(
+                          // Problem: date formats are backend specific. Either use unixtime as %d (long),
+                          // or the native timestamp format.
+                          $entry->time,
+                          $entry->host, 
+                          $entry->user,
+                          $entry->request_method, 
+                          $entry->request, 
+                          $entry->request_uri,    
+                          $entry->request_args,
+                          $entry->_ncsa_time($entry->time), 
+                          $entry->status, 
+                          (int)$entry->size,
+                          $entry->referer,
+                          $entry->user_agent,
+                          $entry->duration));
+    }
 };
 
 /**
@@ -576,23 +643,41 @@ class WikiDB_backend_iterator
      *
      * This returns a hash. The hash may contain the following keys:
      * <dl>
-     * <dt> pagename <dt> (string) the page name
+     * <dt> pagename <dt> (string) the page name or linked page name on link iterators
      * <dt> version  <dt> (int) the version number
      * <dt> pagedata <dt> (hash) page meta-data (as returned from backend::get_pagedata().)
      * <dt> versiondata <dt> (hash) page meta-data (as returned from backend::get_versiondata().)
+     * <dt> linkrelation <dt> (string) the page naming the relation (e.g. isa:=page <=> isa)
      *
      * If this is a page iterator, it must contain the 'pagename' entry --- the others
      * are optional.
      *
      * If this is a version iterator, the 'pagename', 'version', <strong>and</strong> 'versiondata'
      * entries are mandatory.  ('pagedata' is optional.)
+     *
+     * If this is a link iterator, the 'pagename' is mandatory, 'linkrelation' is optional.
      */
     function next() {
         trigger_error("virtual", E_USER_ERROR);
     }
 
     function count() {
-        return count($this->_pages);
+    	if (!empty($this->_pages))
+	    return count($this->_pages);
+	else
+	    return 0;    
+    }
+
+    function asArray() {
+    	if (!empty($this->_pages)) {
+            reset($this->_pages);
+            return $this->_pages;
+    	} else {
+    	    $result = array();
+    	    while ($page = $this->next())
+    	        $result[] = $page;
+            return $result;
+    	}
     }
 
     /**
@@ -603,13 +688,15 @@ class WikiDB_backend_iterator
 };
 
 /**
- * search baseclass, pcre-specific. only overriden by sql backends.
+ * search baseclass, pcre-specific
  */
 class WikiDB_backend_search
 {
-    function WikiDB_backend_search(&$dbh, $search) {
+    function WikiDB_backend_search($search, &$dbh) {
         $this->_dbh = $dbh;
         $this->_case_exact = $search->_case_exact;
+        $this->_stoplist   =& $search->_stoplist;
+        $this->stoplisted = array();
     }
     function _quote($word) {
         return preg_quote($word, "/");
@@ -626,8 +713,67 @@ class WikiDB_backend_search
         $word = $this->$method($node->word);
         return "preg_match(\"/\".$word.\"/\"".($this->_case_exact ? "i":"").")";
     }
+    /* Eliminate stoplist words.
+     *  Keep a list of Stoplisted words to inform the poor user. 
+     */
+    function isStoplisted ($node) {
+    	// check only on WORD or EXACT fulltext search
+    	if ($node->op != 'WORD' and $node->op != 'EXACT')
+    	    return false;
+        if (preg_match("/^".$this->_stoplist."$/i", $node->word)) {
+            array_push($this->stoplisted, $node->word);
+            return true;
+        }
+        return false;
+    }
+    function getStoplisted($word) {
+        return $this->stoplisted;
+    }
 }
 
+/**
+ * search baseclass, sql-specific
+ */
+class WikiDB_backend_search_sql extends WikiDB_backend_search
+{
+    function _pagename_match_clause($node) {
+        // word already quoted by TextSearchQuery_node_word::_sql_quote()
+        $word = $node->sql();
+        if ($word == '%') // ALL shortcut
+            return "1=1";
+        else
+            return ($this->_case_exact 
+                    ? "pagename LIKE '$word'"
+                    : "LOWER(pagename) LIKE '$word'");
+    }
+    function _fulltext_match_clause($node) {
+        // force word-style %word% for fulltext search
+        $word = '%' . $node->_sql_quote($node->word) . '%';
+        // eliminate stoplist words
+        if ($this->isStoplisted($node))
+            return "1=1";  // and (pagename or 1) => and 1
+        else
+            return $this->_pagename_match_clause($node)
+                // probably convert this MATCH AGAINST or SUBSTR/POSITION without wildcards
+                . ($this->_case_exact ? " OR content LIKE '$word'" 
+                                      : " OR LOWER(content) LIKE '$word'");
+    }
+}
+
+// $Log: backend.php,v $
+// Revision 1.33  2007/06/07 21:35:04  rurban
+// fixed backend asArray access to iterators (DebugInfo with SQL)
+//
+// Revision 1.32  2007/02/17 14:14:41  rurban
+// enforce accesslog types
+//
+// Revision 1.31  2007/01/28 22:49:55  rurban
+// use backend specific SQL write_accesslog
+//
+// Revision 1.30  2007/01/02 13:20:26  rurban
+// added link_search. Clarify API: sortby,limit and exclude are strings.
+//
+
 // For emacs users
 // Local Variables:
 // mode: php
@@ -636,4 +782,4 @@ class WikiDB_backend_search
 // c-hanging-comment-ender-p: nil
 // indent-tabs-mode: nil
 // End:
-?>
\ No newline at end of file
+?>
