@@ -1,5 +1,5 @@
 <?php // -*-php-*-
-rcs_id('$Id: PearDB.php,v 1.92 2005/09/14 06:04:43 rurban Exp $');
+rcs_id('$Id: PearDB.php,v 1.108 2007/06/07 21:37:39 rurban Exp $');
 
 require_once('lib/WikiDB/backend.php');
 //require_once('lib/FileFinder.php');
@@ -35,7 +35,7 @@ extends WikiDB_backend
 
         // Install filter to handle bogus error notices from buggy DB.php's.
         // TODO: check the Pear_DB version, but how?
-        if (0) {
+        if (DEBUG) {
             global $ErrorManager;
             $ErrorManager->pushErrorHandler(new WikiMethodCb($this, '_pear_notice_filter'));
             $this->_pearerrhandler = true;
@@ -56,7 +56,10 @@ extends WikiDB_backend
         if (DB::isError($dbh)) {
             trigger_error(sprintf("Can't connect to database: %s",
                                   $this->_pear_error_message($dbh)),
-                          E_USER_ERROR);
+                          isset($dbparams['_tryroot_from_upgrade']) // hack!
+                            ? E_USER_WARNING : E_USER_ERROR);
+            if (isset($dbparams['_tryroot_from_upgrade']))
+                return;                
         }
         $dbh->setErrorHandling(PEAR_ERROR_CALLBACK,
                                array($this, '_pear_error_callback'));
@@ -212,10 +215,10 @@ extends WikiDB_backend
                             $dbh->escapeSimple($this->_serialize($data)),
                             $dbh->escapeSimple($pagename)));
         */
-        $sth = $dbh->query("UPDATE $page_tbl"
-                           . " SET hits=?, pagedata=?"
-                           . " WHERE pagename=?",
-                           array($hits, $this->_serialize($data), $pagename));
+        $dbh->query("UPDATE $page_tbl"
+                    . " SET hits=?, pagedata=?"
+                    . " WHERE pagename=?",
+                    array($hits, $this->_serialize($data), $pagename));
         $this->unlock(array($page_tbl));
     }
 
@@ -229,10 +232,10 @@ extends WikiDB_backend
     function set_cached_html($pagename, $data) {
         $dbh = &$this->_dbh;
         $page_tbl = $this->_table_names['page_tbl'];
-        $sth = $dbh->query("UPDATE $page_tbl"
-                           . " SET cached_html=?"
-                           . " WHERE pagename=?",
-                           array($data, $pagename));
+        $dbh->query("UPDATE $page_tbl"
+                    . " SET cached_html=?"
+                    . " WHERE pagename=?",
+                    array($data, $pagename));
     }
 
     function _get_pageid($pagename, $create_if_missing = false) {
@@ -246,6 +249,9 @@ extends WikiDB_backend
             }
         }
 
+	// attributes play this game.
+        if ($pagename === '') return 0;
+
         $dbh = &$this->_dbh;
         $page_tbl = $this->_table_names['page_tbl'];
         
@@ -260,6 +266,8 @@ extends WikiDB_backend
             $this->lock(array($page_tbl), true); // write lock
             $max_id = $dbh->getOne("SELECT MAX(id) FROM $page_tbl");
             $id = $max_id + 1;
+            // requires createSequence and on mysql lock the interim table ->getSequenceName
+            //$id = $dbh->nextId($page_tbl . "_id");
             $dbh->query(sprintf("INSERT INTO $page_tbl"
                                 . " (id,pagename,hits)"
                                 . " VALUES (%d,'%s',0)",
@@ -342,6 +350,13 @@ extends WikiDB_backend
         if (!$query_result)
             return false;
 
+        /* Earlier versions (<= 1.3.7) stored the version data in base64.
+           This could be done here or in upgrade.
+        */
+        if (!strstr($query_result['versiondata'], ":")) {
+            $query_result['versiondata'] = 
+                base64_decode($query_result['versiondata']);
+        }
         $data = $this->_unserialize($query_result['versiondata']);
         
         $data['mtime'] = $query_result['mtime'];
@@ -388,19 +403,9 @@ extends WikiDB_backend
         $this->lock();
         $id = $this->_get_pageid($pagename, true);
 
-        // FIXME: optimize: mysql can do this with one REPLACE INTO (I think).
         $dbh->query(sprintf("DELETE FROM $version_tbl"
                             . " WHERE id=%d AND version=%d",
                             $id, $version));
-
-        /* mysql optimized version. 
-        $dbh->query(sprintf("INSERT INTO $version_tbl"
-                            . " (id,version,mtime,minor_edit,content,versiondata)"
-                            . " VALUES(%d,%d,%d,%d,'%s','%s')",
-                            $id, $version, $mtime, $minor_edit,
-                            $dbh->quoteSmart($content),
-                            $dbh->quoteSmart($this->_serialize($data))));
-        */
         // generic slow PearDB bind eh quoting.
         $dbh->query("INSERT INTO $version_tbl"
                     . " (id,version,mtime,minor_edit,content,versiondata)"
@@ -425,6 +430,7 @@ extends WikiDB_backend
         if ( ($id = $this->_get_pageid($pagename)) ) {
             $dbh->query("DELETE FROM $version_tbl"
                         . " WHERE id=$id AND version=$version");
+
             $this->_update_recent_table($id);
             // This shouldn't be needed (as long as the latestversion
             // never gets deleted.)  But, let's be safe.
@@ -466,9 +472,9 @@ extends WikiDB_backend
         
         $this->lock();
         if ( ($id = $this->_get_pageid($pagename, false)) ) {
-            $dbh->query("DELETE FROM $version_tbl  WHERE id=$id");
-            $dbh->query("DELETE FROM $recent_tbl   WHERE id=$id");
             $dbh->query("DELETE FROM $nonempty_tbl WHERE id=$id");
+            $dbh->query("DELETE FROM $recent_tbl   WHERE id=$id");
+            $dbh->query("DELETE FROM $version_tbl  WHERE id=$id");
             $dbh->query("DELETE FROM $link_tbl     WHERE linkfrom=$id");
             $nlinks = $dbh->getOne("SELECT COUNT(*) FROM $link_tbl WHERE linkto=$id");
             if ($nlinks) {
@@ -507,27 +513,48 @@ extends WikiDB_backend
         $pageid = $this->_get_pageid($pagename, true);
 
         $dbh->query("DELETE FROM $link_tbl WHERE linkfrom=$pageid");
-
 	if ($links) {
-            foreach($links as $link) {
-                // avoid duplicates
-                if (isset($linkseen[$link]))
+	    $linkseen = array();
+            foreach ($links as $link) {
+                $linkto = $link['linkto'];
+                if ($linkto === "") { // ignore attributes
                     continue;
-                $linkseen[$link] = true;
-                $linkid = $this->_get_pageid($link, true);
+                }
+                if ($link['relation'])
+                    $relation = $this->_get_pageid($link['relation'], true);
+                else 
+                    $relation = 0;
+                // avoid duplicates
+                if (isset($linkseen[$linkto]) and !$relation)
+                    continue;
+                if (!$relation)
+                    $linkseen[$linkto] = true;
+                $linkid = $this->_get_pageid($linkto, true);
+              	if (!$linkid) {
+               	    echo("No link for $linkto on page $pagename");
+               	    //printSimpleTrace(debug_backtrace());
+               	    trigger_error("No link for $linkto on page $pagename");
+                }
                 assert($linkid);
-                $dbh->query("INSERT INTO $link_tbl (linkfrom, linkto)"
-                            . " VALUES ($pageid, $linkid)");
+                $dbh->query("INSERT INTO $link_tbl (linkfrom, linkto, relation)"
+                            . " VALUES ($pageid, $linkid, $relation)");
             }
+	    unset($linkseen);
 	}
         $this->unlock();
     }
     
     /**
      * Find pages which link to or are linked from a page.
+     *
+     * TESTME relations: get_links is responsible to add the relation to the pagehash 
+     * as 'linkrelation' key as pagename. See WikiDB_PageIterator::next 
+     *   if (isset($next['linkrelation']))
      */
     function get_links($pagename, $reversed=true, $include_empty=false,
-                       $sortby=false, $limit=false, $exclude='') {
+                       $sortby='', $limit='', $exclude='', 
+                       $want_relations = false)
+    {
         $dbh = &$this->_dbh;
         extract($this->_table_names);
 
@@ -543,9 +570,12 @@ extends WikiDB_backend
             $exclude='';
 
         $qpagename = $dbh->escapeSimple($pagename);
-        $sql = "SELECT $want.id AS id, $want.pagename AS pagename "
-            . " FROM $link_tbl, $page_tbl linker, $page_tbl linkee"
-            . (!$include_empty ? ", $nonempty_tbl" : '')
+        $sql = "SELECT $want.id AS id, $want.pagename AS pagename, "
+            . ($want_relations ? " related.pagename as linkrelation" : " $want.hits AS hits")
+            . " FROM "
+            . (!$include_empty ? "$nonempty_tbl, " : '')
+            . " $page_tbl linkee, $page_tbl linker, $link_tbl "
+            . ($want_relations ? " JOIN $page_tbl related ON ($link_tbl.relation=related.id)" : '')
             . " WHERE linkfrom=linker.id AND linkto=linkee.id"
             . " AND $have.pagename='$qpagename'"
             . (!$include_empty ? " AND $nonempty_tbl.id=$want.id" : "")
@@ -576,17 +606,15 @@ extends WikiDB_backend
             list($have, $want) = array('linker', 'linkee');
         $qpagename = $dbh->escapeSimple($pagename);
         $qlink = $dbh->escapeSimple($link);
-        $row = $dbh->GetRow
-            ("SELECT IF($want.pagename,1,0) as result"
-             . " FROM $link_tbl, $page_tbl linker, $page_tbl linkee, $nonempty_tbl"
-             . " WHERE linkfrom=linker.id AND linkto=linkee.id"
-             . " AND $have.pagename='$qpagename'"
-             . " AND $want.pagename='$qlink'"
-             . " LIMIT 1");
+        $row = $dbh->GetRow("SELECT CASE WHEN $want.pagename='$qlink' THEN 1 ELSE 0 END as result"
+                            . " FROM $link_tbl, $page_tbl linker, $page_tbl linkee, $nonempty_tbl"
+                            . " WHERE linkfrom=linker.id AND linkto=linkee.id"
+                            . " AND $have.pagename='$qpagename'"
+                            . " AND $want.pagename='$qlink'");
         return $row['result'];
     }
 
-    function get_all_pages($include_empty=false, $sortby=false, $limit=false, $exclude='') {
+    function get_all_pages($include_empty=false, $sortby='', $limit='', $exclude='') {
         $dbh = &$this->_dbh;
         extract($this->_table_names);
         $orderby = $this->sortby($sortby, 'db');
@@ -645,8 +673,11 @@ extends WikiDB_backend
         
     /**
      * Title search.
+     * Todo: exclude
      */
-    function text_search($search, $fulltext=false, $sortby=false, $limit=false, $exclude=false) {
+    function text_search($search, $fulltext=false, $sortby='', $limit='', 
+                         $exclude='') 
+    {
         $dbh = &$this->_dbh;
         extract($this->_table_names);
         $orderby = $this->sortby($sortby, 'db');
@@ -840,7 +871,7 @@ extends WikiDB_backend
     /**
      * Find referenced empty pages.
      */
-    function wanted_pages($exclude_from='', $exclude='', $sortby=false, $limit=false) {
+    function wanted_pages($exclude_from='', $exclude='', $sortby='', $limit='') {
         $dbh = &$this->_dbh;
         extract($this->_table_names);
         if ($orderby = $this->sortby($sortby, 'db', array('pagename','wantedfrom')))
@@ -850,16 +881,17 @@ extends WikiDB_backend
             $exclude_from = " AND pp.pagename NOT IN ".$this->_sql_set($exclude_from);
         if ($exclude) // array of pagenames
             $exclude = " AND p.pagename NOT IN ".$this->_sql_set($exclude);
-        $sql = "SELECT p.pagename, pp.pagename as wantedfrom"
-            . " FROM $page_tbl p, $link_tbl linked "
-            . " LEFT JOIN $page_tbl pp ON linked.linkto = pp.id"
-            . " LEFT JOIN $nonempty_tbl ne ON linked.linkto = ne.id" 
-            . " WHERE ne.id is NULL"
-	    .       " AND p.id = linked.linkfrom"
+        $sql = "SELECT p.pagename, pp.pagename AS wantedfrom"
+            . " FROM $page_tbl p, $link_tbl linked"
+            .   " LEFT JOIN $page_tbl pp ON linked.linkto = pp.id"
+            .   " LEFT JOIN $nonempty_tbl ne ON linked.linkto = ne.id"
+            . " WHERE ne.id IS NULL"
+            .       " AND p.id = linked.linkfrom"
             . $exclude_from
             . $exclude
             . $orderby;
         if ($limit) {
+            // oci8 error: WHERE NULL = NULL appended
             list($from, $count) = $this->limit($limit);
             $result = $dbh->limitQuery($sql, $from, $count * 3);
         } else {
@@ -889,13 +921,13 @@ extends WikiDB_backend
                 // Cludge Alert!
                 // This page does not exist (already verified before), but exists in the page table.
                 // So we delete this page.
-                $dbh->query("DELETE FROM $page_tbl WHERE id=$new");
-                $dbh->query("DELETE FROM $version_tbl WHERE id=$new");
-                $dbh->query("DELETE FROM $recent_tbl WHERE id=$new");
                 $dbh->query("DELETE FROM $nonempty_tbl WHERE id=$new");
+                $dbh->query("DELETE FROM $recent_tbl WHERE id=$new");
+                $dbh->query("DELETE FROM $version_tbl WHERE id=$new");
                 // We have to fix all referring tables to the old id
                 $dbh->query("UPDATE $link_tbl SET linkfrom=$id WHERE linkfrom=$new");
                 $dbh->query("UPDATE $link_tbl SET linkto=$id WHERE linkto=$new");
+                $dbh->query("DELETE FROM $page_tbl WHERE id=$new");
             }
             $dbh->query(sprintf("UPDATE $page_tbl SET pagename='%s' WHERE id=$id",
                                 $dbh->escapeSimple($to)));
@@ -1128,6 +1160,7 @@ extends WikiDB_backend
         } else {
             // TODO: try ADODB version?
             trigger_error("Unsupported dbtype and backend. Either switch to ADODB or check it manually.");
+            return false;
         }
     }
 };
@@ -1147,9 +1180,7 @@ extends WikiDB_backend
 class WikiDB_backend_PearDB_generic_iter
 extends WikiDB_backend_iterator
 {
-    function WikiDB_backend_PearDB_generic_iter($backend, $query_result, 
-                                                $field_list = NULL) 
-    {
+    function WikiDB_backend_PearDB_generic_iter($backend, $query_result, $field_list = NULL) {
         if (DB::isError($query_result)) {
             // This shouldn't happen, I thought.
             $backend->_pear_error_callback($query_result);
@@ -1166,7 +1197,6 @@ extends WikiDB_backend_iterator
     }
     
     function next() {
-        $backend = &$this->_backend;
         if (!$this->_result)
             return false;
 
@@ -1185,6 +1215,13 @@ extends WikiDB_backend_iterator
             $this->_result = false;
         }
     }
+
+    function asArray () {
+    	$result = array();
+    	while ($page = $this->next())
+    	    $result[] = $page;
+        return $result;
+    }
 }
 
 class WikiDB_backend_PearDB_iter
@@ -1222,9 +1259,55 @@ class WikiDB_backend_PearDB_search extends WikiDB_backend_search_sql
 }
 
 // $Log: PearDB.php,v $
+// Revision 1.108  2007/06/07 21:37:39  rurban
+// add native asArray methods to generic iters (for DebugInfo)
+//
+// Revision 1.107  2007/05/28 20:13:46  rurban
+// Overwrite all attributes at once at page->save to delete dangling meta
+//
+// Revision 1.106  2007/01/04 16:57:32  rurban
+// Clarify API: sortby,limit and exclude are strings. fix upgrade test connection
+//
+// Revision 1.105  2006/12/23 13:03:32  rurban
+// reorder deletion
+//
+// Revision 1.104  2006/12/23 11:44:56  rurban
+// deal with strict references and the order of deletion
+//
+// Revision 1.103  2006/12/03 17:11:53  rurban
+// #1535832 by matt brown: Check for base 64 encoded version data
+//
+// Revision 1.102  2006/12/02 21:57:27  rurban
+// fix WantedPages SQL: no JOIN
+// clarify first condition in CASE WHEN
+//
+// Revision 1.101  2006/11/29 19:49:05  rurban
+// fix CASE WHEN SQL syntax error from previous commit
+//
+// Revision 1.100  2006/11/19 13:59:11  rurban
+// Replace IF by CASE in exists_link()
+//
+// Revision 1.99  2006/10/08 12:40:51  rurban
+// minor cleanup: remove unused vars
+//
+// Revision 1.98  2006/06/03 08:50:41  rurban
+// revert wrong pear DB nextID() usage, our old is easier
+//
+// Revision 1.97  2006/05/14 12:28:03  rurban
+// mysql 5.x fix for wantedpages join
+//
+// Revision 1.96  2006/04/15 12:28:53  rurban
+// use pear nextID
+//
+// Revision 1.95  2006/02/22 21:52:28  rurban
+// whitespace only
+//
 // Revision 1.94  2005/11/14 22:24:33  rurban
 // fix fulltext search,
 // Eliminate stoplist words,
+// don't extract %pagedate twice in ADODB,
+// add SemanticWeb support: link(relation),
+// major postgresql update: stored procedures, tsearch2 for fulltext
 //
 // Revision 1.93  2005/10/10 19:42:15  rurban
 // fix wanted_pages SQL syntax
@@ -1454,4 +1537,4 @@ class WikiDB_backend_PearDB_search extends WikiDB_backend_search_sql
 // c-hanging-comment-ender-p: nil
 // indent-tabs-mode: nil
 // End:   
-?>
\ No newline at end of file
+?>
