@@ -1,8 +1,10 @@
 <?php // -*-php-*-
-rcs_id('$Id: PearDB_mysql.php,v 1.21 2005/10/10 19:42:15 rurban Exp $');
+rcs_id('$Id: PearDB_mysql.php,v 1.27 2007/01/04 16:41:14 rurban Exp $');
 
 require_once('lib/WikiDB/backend/PearDB.php');
 
+// See http://sql-info.de/mysql/gotchas.html for mysql specific quirks.
+
 // The slowest function overall is mysql_connect with [680ms]
 // 2nd is db_mysql::simpleQuery with [257ms]
 class WikiDB_backend_PearDB_mysql
@@ -13,6 +15,7 @@ extends WikiDB_backend_PearDB
      */
     function WikiDB_backend_PearDB_mysql($dbparams) {
         $this->WikiDB_backend_PearDB($dbparams);
+        if (DB::isError($this->_dbh)) return;
         //$this->_serverinfo = $this->_dbh->ServerInfo();
         $row = $this->_dbh->GetOne("SELECT version()");
         if (!DB::isError($row) and !empty($row)) {
@@ -112,22 +115,23 @@ extends WikiDB_backend_PearDB
     }
 
     /* ISNULL is mysql specific */
-    function wanted_pages($exclude_from='', $exclude='', $sortby=false, $limit=false) {
+    function wanted_pages($exclude_from='', $exclude='', $sortby='', $limit='') {
         $dbh = &$this->_dbh;
         extract($this->_table_names);
         if ($orderby = $this->sortby($sortby, 'db', array('pagename','wantedfrom')))
             $orderby = 'ORDER BY ' . $orderby;
 
         if ($exclude_from) // array of pagenames
-            $exclude_from = " AND linked.pagename NOT IN ".$this->_sql_set($exclude_from);
+            $exclude_from = " AND pp.pagename NOT IN ".$this->_sql_set($exclude_from);
         if ($exclude) // array of pagenames
-            $exclude = " AND $page_tbl.pagename NOT IN ".$this->_sql_set($exclude);
+            $exclude = " AND p.pagename NOT IN ".$this->_sql_set($exclude);
 
-        $sql = "SELECT $page_tbl.pagename,linked.pagename as wantedfrom"
-            . " FROM $link_tbl,$page_tbl as linked "
-            . " LEFT JOIN $page_tbl ON ($link_tbl.linkto=$page_tbl.id)"
-            . " LEFT JOIN $nonempty_tbl ON ($link_tbl.linkto=$nonempty_tbl.id)" 
-            . " WHERE ISNULL($nonempty_tbl.id) AND linked.id=$link_tbl.linkfrom"
+        $sql = "SELECT p.pagename, pp.pagename as wantedfrom"
+            . " FROM $page_tbl p, $link_tbl linked"
+            . " LEFT JOIN $page_tbl pp ON (linked.linkto = pp.id)"
+            . " LEFT JOIN $nonempty_tbl ne ON (linked.linkto = ne.id)" 
+            . " WHERE ISNULL(ne.id)"
+            .       " AND p.id = linked.linkfrom"
             . $exclude_from
             . $exclude
             . $orderby;
@@ -221,6 +225,14 @@ extends WikiDB_backend_PearDB_search
     }
 }
 
+// $Log: PearDB_mysql.php,v $
+// Revision 1.27  2007/01/04 16:41:14  rurban
+// Allow missing connection permissions on upgrade::_try_dbadmin_user.
+//
+// Revision 1.26  2006/12/22 17:49:05  rurban
+// add Log
+//
+
 // (c-file-style: "gnu")
 // Local Variables:
 // mode: php
