@@ -1,4 +1,5 @@
-<?php rcs_id('$Id: dbaBase.php,v 1.23 2005/09/11 13:21:45 rurban Exp $');
+<?php // -*-php-*-
+rcs_id('$Id: dbaBase.php,v 1.29 2007/05/24 18:39:10 rurban Exp $');
 
 require_once('lib/WikiDB/backend.php');
 
@@ -72,6 +73,8 @@ extends WikiDB_backend
     }
 
     function rebuild() {
+    	parent::rebuild();
+    	// rebuild backlink table
         $this->_linkdb->rebuild();
         $this->optimize();
     }
@@ -229,7 +232,7 @@ extends WikiDB_backend
         $pagedb->set($pagename, (int)$latest . ':' . (int)$flags . ":$pagedata");
     }
 
-    function numPages($include_empty=false, $exclude=false) {
+    function numPages($include_empty=false, $exclude='') {
         $pagedb = &$this->_pagedb;
         $count = 0;
         for ($page = $pagedb->firstkey(); $page!== false; $page = $pagedb->nextkey()) {
@@ -250,16 +253,22 @@ extends WikiDB_backend
         return $count;
     }
 
-    function get_all_pages($include_empty=false, $sortby=false, $limit=false, $exclude=false) {
+    function get_all_pages($include_empty=false, $sortby='', $limit='', $exclude='') {
         $pagedb = &$this->_pagedb;
         $pages = array();
+        if ($limit) // extract from,count from limit
+            list($from,$count) = $this->limit($limit);
         for ($page = $pagedb->firstkey(); $page!== false; $page = $pagedb->nextkey()) {
             if (!$page) {
                 assert(!empty($page));
                 continue;
             }
             if ($exclude and in_array($page, $exclude)) continue; 
-            if ($limit and count($pages) > $limit) break;
+            if ($limit and $from) {
+		$i++;
+		if ($i < $from) continue;
+	    }
+            if ($limit and count($pages) > $count) break;
             if (!$include_empty) {
             	if (!($data = $pagedb->get($page))) continue;
                 list($latestversion,$flags,) = explode(':', $data, 3);
@@ -270,8 +279,8 @@ extends WikiDB_backend
             $pages[] = $page;
         }
         return new WikiDB_backend_dbaBase_pageiter($this, $pages, 
-                                                   array('sortby'=>$sortby,
-                                                         'limit' =>$limit));
+                                                   array('sortby'=>$sortby/*,
+                                                         'limit' =>$limit*/));
     }
 
     function set_links($pagename, $links) {
@@ -279,13 +288,214 @@ extends WikiDB_backend
     }
 
     function get_links($pagename, $reversed=true, $include_empty=false,
-                       $sortby=false, $limit=false, $exclude=false) {
-        $links = $this->_linkdb->get_links($pagename, $reversed);
-        return new WikiDB_backend_dbaBase_pageiter($this, $links, 
-                                                   array('sortby'=>$sortby,
-                                                         'limit' =>$limit,
-                                                         'exclude'=>$exclude,
-                                                         ));
+                       $sortby='', $limit='', $exclude='',
+                       $want_relations=false) 
+    {
+    	// optimization: if no relation at all is found, mark it in the iterator.
+        $links = $this->_linkdb->get_links($pagename, $reversed, $want_relations);
+
+        return new WikiDB_backend_dbaBase_pageiter
+	    ($this, $links, 
+	     array('sortby'=>$sortby,
+		   'limit' =>$limit,
+		   'exclude'=>$exclude,
+		   'want_relations'=>$want_relations,
+		   'found_relations' => $want_relations ? $this->_linkdb->found_relations : 0
+		   ));
+    }
+    
+    /**
+     * @access public
+     *
+     * @return array of all linkrelations
+     * Faster than the dumb WikiDB method.
+     */
+    function list_relations($also_attributes=false, $only_attributes=false, $sorted=true) {
+        $linkdb = &$this->_linkdb;
+        $relations = array();
+        for ($link = $linkdb->_db->firstkey(); $link!== false; $link = $linkdb->_db->nextkey()) {
+            if ($link[0] != 'o') continue;	
+            $links = $linkdb->_get_links('o', substr($link,1));
+            foreach ($links as $link) { // linkto => page, linkrelation => page
+                if (is_array($link)
+                    and $link['relation'] 
+                    and !in_array($link['relation'], $relations)) 
+		{
+		    $is_attribute = empty($link['linkto']); // a relation has both
+		    if ($is_attribute) {
+			if ($only_attributes or $also_attributes)
+			    $relations[] = $link['relation'];
+		    } elseif (!$only_attributes) {
+			  $relations[] = $link['relation'];
+		    }
+		}
+            }
+        }
+	if ($sorted) {
+	    sort($relations);
+	    reset($relations);
+	}
+        return $relations;
+    }
+
+    /**
+     * WikiDB_backend_dumb_LinkSearchIter searches over all pages and then all its links.
+     * Since there are less links than pages, and we easily get the pagename from the link key,
+     * we iterate here directly over the linkdb and check the pagematch there.
+     *
+     * @param $pages     object A TextSearchQuery object for the pagename filter.
+     * @param $query     object A SearchQuery object (Text or Numeric) for the linkvalues, 
+     *                          linkto, linkfrom (=backlink), relation or attribute values.
+     * @param $linktype  string One of the 4 linktypes "linkto", "linkfrom" (=backlink), "relation" or "attribute".
+     * 				Maybe also "relation+attribute" for the advanced search.
+     * @param $relation  object A TextSearchQuery for the linkname or false.
+     * @param $options   array Currently ignored. hash of sortby, limit, exclude.
+     * @return object A WikiDB_backend_iterator.
+     * @see WikiDB::linkSearch
+     */
+    function link_search( $pages, $query, $linktype, $relation=false, $options=array() ) {
+        $linkdb = &$this->_linkdb;
+        $links = array();
+	$reverse = false;
+	$want_relations = false;
+	if ($linktype == 'relation') {
+	    $want_relations = true;
+	    $field = 'linkrelation';
+	}
+	if ($linktype == 'attribute') {
+	    $want_relations = true;
+	    $field = 'attribute';
+	}
+	if ($linktype == 'linkfrom') {
+	    $reverse = true;
+	}
+
+        for ($link = $linkdb->_db->firstkey(); $link!== false; $link = $linkdb->_db->nextkey()) {
+	    $type = $reverse ? 'i' : 'o';
+            if ($link[0] != $type) continue;
+	    $pagename = substr($link, 1);
+	    if (!$pages->match($pagename)) continue;
+	    if ($linktype == 'attribute') {
+		$page = $GLOBALS['request']->_dbi->getPage($pagename);
+		$attribs = $page->get('attributes');
+		if ($attribs) {
+		    /* Optimization on expressive searches: 
+		       for queries with multiple attributes.
+		       Just take the defined placeholders from the query(ies)
+		       if there are more attributes than query variables. 
+		    */
+		    if ($query->getType() != 'text'
+		        and !$relation
+			and ((count($vars = $query->getVars()) > 1) 
+			     or (count($attribs) > count($vars))))
+		    {
+			// names must strictly match. no * allowed
+			if (!$query->can_match($attribs)) continue;
+			if (!($result = $query->match($attribs))) continue;
+			foreach ($result as $r) {
+			    $r['pagename'] = $pagename;
+			    $links[] = $r;
+			}
+		    } else {
+			// textsearch or simple value. no strict bind by name needed 
+			foreach ($attribs as $attribute => $value) {
+			    if ($relation and !$relation->match($attribute)) continue;
+			    if (!$query->match($value)) continue; 
+			    $links[] = array('pagename'  => $pagename,
+					     'linkname'  => $attribute,
+					     'linkvalue' => $value);
+			}
+		    }
+		}
+	    }
+	    else {
+		// TODO: honor limits. this can get large.
+		if ($want_relations) {
+		    // MAP linkrelation : pagename => thispagename : linkname : linkvalue  
+		    $_links = $linkdb->_get_links('o', $pagename);
+		    foreach ($_links as $link) { // linkto => page, linkrelation => page
+			if (!isset($link['relation']) or !$link['relation']) continue;
+			if ($relation and !$relation->match($link['relation'])) continue;
+			if (!$query->match($link['linkto'])) continue;
+			$links[] = array('pagename'  => $pagename,
+					 'linkname'  => $link['relation'],
+					 'linkvalue' => $link['linkto']);
+		    }
+		} else {
+		    $_links = $linkdb->_get_links($reverse ? 'i' : 'o', $pagename);
+		    foreach ($_links as $link) { // linkto => page
+			if (is_array($link))
+			    $link = $link['linkto'];
+			if (!$query->match($link)) continue; 
+			$links[] = array('pagename'  => $pagename,
+					 'linkname'  => '',
+					 'linkvalue' => $link);
+		    }
+		}
+	    }
+        }
+	$options['want_relations'] = true; // Iter hack to force return of the whole hash
+        return new WikiDB_backend_dbaBase_pageiter($this, $links, $options);
+    }
+
+    /**
+     * Handle multi-searches for many relations and attributes in one expression.
+     * Bind all required attributes and relations per page together and pass it to one query.
+     *   (is_a::city and population < 20000) and (*::city and area > 1000000)
+     *   (is_a::city or linkto::CategoryCountry) and population < 20000 and area > 1000000
+     * Note that the 'linkto' and 'linkfrom' links are relations, containing an array.
+     *
+     * @param $pages     object A TextSearchQuery object for the pagename filter.
+     * @param $query     object A SemanticSearchQuery object for the links. 
+     * @param $options   array  Currently ignored. hash of sortby, limit, exclude for the pagelist.
+     * @return object A WikiDB_backend_iterator.
+     * @see WikiDB::linkSearch
+     */
+    function relation_search( $pages, $query, $options=array() ) {
+        $linkdb = &$this->_linkdb;
+        $links = array();
+	// We need to detect which attributes and relation names we should look for. NYI
+	$want_attributes = $query->hasAttributes();
+	$want_relation = $query->hasRelations();
+	$linknames = $query->getLinkNames();
+	// create a hash for faster checks
+	$linkcheck = array();
+	foreach ($linknames as $l) $linkcheck[$l] = 1;
+
+        for ($link = $linkdb->_db->firstkey(); $link!== false; $link = $linkdb->_db->nextkey()) {
+	    $type = $reverse ? 'i' : 'o';
+            if ($link[0] != $type) continue;
+	    $pagename = substr($link, 1);
+	    if (!$pages->match($pagename)) continue;
+	    $pagelinks = array();
+	    if ($want_attributes) {
+		$page = $GLOBALS['request']->_dbi->getPage($pagename);
+		$attribs = $page->get('attributes');
+		$pagelinks = $attribs;
+	    }
+	    if ($want_relations) {
+		// all links contain arrays of pagenames, just the attributes 
+		// are guaranteed to be singular
+		if (isset($linkcheck['linkfrom'])) {
+		    $pagelinks['linkfrom'] = $linkdb->_get_links('i', $pagename);
+		}
+		$outlinks = $linkdb->_get_links('o', $pagename);
+		$want_to = isset($linkcheck['linkto']);
+		foreach ($outlinks as $link) { // linkto => page, relation => page
+		    // all named links
+		    if ((isset($link['relation'])) and $link['relation'] 
+		        and isset($linkcheck[$link['relation']]))
+			$pagelinks[$link['relation']][] = $link['linkto'];
+		    if ($want_to)
+			$pagelinks['linkto'][] = is_array($link) ? $link['linkto'] : $link;
+		}
+	    }
+	    if ($result = $query->match($pagelinks)) {
+		$links = array_merge($links, $result);		
+	    }
+        }
+	$options['want_relations'] = true; // Iter hack to force return of the whole hash
+        return new WikiDB_backend_dbaBase_pageiter($this, $links, $options);
     }
 };
 
@@ -329,6 +539,7 @@ function WikiDB_backend_dbaBase_sortby_num($aname, $bname, $field) {
 class WikiDB_backend_dbaBase_pageiter
 extends WikiDB_backend_iterator
 {
+    // fixed for linkrelations
     function WikiDB_backend_dbaBase_pageiter(&$backend, &$pages, $options=false) {
         $this->_backend = $backend;
         $this->_options = $options;
@@ -348,21 +559,20 @@ extends WikiDB_backend_iterator
             $this->_pages = array();
     }
 
+    // fixed for relations
     function next() {
-        if ( ! ($pagename = array_shift($this->_pages)) )
+        if ( ! ($page = array_shift($this->_pages)) )
             return false;
-        if (!empty($options['exclude']) and in_array($pagename, $options['exclude']))
+	if (!empty($this->_options['want_relations'])) {
+	    // $linkrelation = $page['linkrelation'];
+	    $pagename = $page['pagename'];
+            if (!empty($this->_options['exclude']) and in_array($pagename, $this->_options['exclude']))
+        	return $this->next();
+	    return $page;
+    	}
+        if (!empty($this->_options['exclude']) and in_array($page, $this->_options['exclude']))
             return $this->next();
-        return array('pagename' => $pagename);
-    }
-            
-    function count() {
-        return count($this->_pages);
-    }
-
-    function asArray() {
-        reset($this->_pages);
-        return $this->_pages;
+        return array('pagename' => $page);
     }
 
     function free() {
@@ -378,23 +588,57 @@ class WikiDB_backend_dbaBase_linktable
 
     //FIXME: try storing link lists as hashes rather than arrays.
     // (backlink deletion would be faster.)
-    function get_links($page, $reversed=true) {
-        return $this->_get_links($reversed ? 'i' : 'o', $page);
+    function get_links($page, $reversed=true, $want_relations=false) {
+        if ($want_relations) {
+            $this->found_relations = 0;	
+            $links = $this->_get_links($reversed ? 'i' : 'o', $page);
+            $linksonly = array();
+            foreach ($links as $link) { // linkto => page, linkrelation => page
+                if (is_array($link) and isset($link['relation'])) {
+                    if ($link['relation'])
+                        $this->found_relations++;
+                    $linksonly[] = array('pagename'     => $link['linkto'],
+                    		         'linkrelation' => $link['relation']);
+                } else { // empty relations are stripped
+                    $linksonly[] = array('pagename' => $link['linkto']);
+                }
+            }
+            return $linksonly;
+        } else {
+            $links = $this->_get_links($reversed ? 'i' : 'o', $page);
+            $linksonly = array();
+            foreach ($links as $link) {
+                if (is_array($link)) {
+                    $linksonly[] = $link['linkto'];
+                } else
+                    $linksonly[] = $link;
+            }
+            return $linksonly;
+        }
     }
     
-    function set_links($page, $newlinks) {
+    // fixed: relations ready
+    function set_links($page, $links) {
 
-        $oldlinks = $this->_get_links('o', $page);
+        $oldlinks = $this->get_links($page, false, false);
 
-        if (!is_array($newlinks)) {
-            assert(empty($newlinks));
-            $newlinks = array();
+        if (!is_array($links)) {
+            assert(empty($links));
+            $links = array();
         }
-        else {
-            $newlinks = array_unique($newlinks);
+        $this->_set_links('o', $page, $links);
+        
+        /* Now for the backlink update we squash the linkto hashes into a simple array */
+        $newlinks = array();
+        foreach ($links as $hash) {
+            if (!empty($hash['linkto']) 
+                and !in_array($hash['linkto'],$newlinks))
+                 // for attributes it's empty
+                $newlinks[] = $hash['linkto'];		
         }
+        //$newlinks = array_unique($newlinks);
+        sort($oldlinks);
         sort($newlinks);
-        $this->_set_links('o', $page, $newlinks);
 
         reset($newlinks);
         reset($oldlinks);
@@ -432,22 +676,18 @@ class WikiDB_backend_dbaBase_linktable
     function rebuild () {
         $db = &$this->_db;
 
-        // Delete the backlink tables, make a list of page names.
+        // Delete the backlink tables, make a list of lo.page keys.
         $okeys = array();
-        $ikeys = array();
         for ($key = $db->firstkey(); $key; $key = $db->nextkey()) {
             if ($key[0] == 'i')
-                $ikeys[] = $key;
+                $db->delete($key);
             elseif ($key[0] == 'o')
                 $okeys[] = $key;
             else {
                 trigger_error("Bad key in linktable: '$key'", E_USER_WARNING);
-                $ikeys[] = $key;
-            }
-        }
-        foreach ($ikeys as $key) {
             $db->delete($key);
         }
+        }
         foreach ($okeys as $key) {
             $page = substr($key,1);
             $links = $this->_get_links('o', $page);
@@ -470,6 +710,7 @@ class WikiDB_backend_dbaBase_linktable
             if ($key[0] == 'o') {
                 // Forward links.
                 foreach($this->_get_links('o', $page) as $link) {
+                    $link = $link['linkto'];
                     if (!$this->_has_link('i', $link, $page))
                         $errs[] = "backlink entry missing for link '$page'->'$link'";
                 }
@@ -483,10 +724,20 @@ class WikiDB_backend_dbaBase_linktable
                 }
             }
         }
-
+ 	//if ($errs) $this->rebuild();
         return isset($errs) ? $errs : false;
     }
     
+    /* TODO: Add another lrRelationName key for relations.
+     * lrRelationName: frompage => topage
+     */
+
+    function _add_relation($page, $linkedfrom) {
+        $relations = $this->_get_links('r', $page);
+        $backlinks[] = $linkedfrom;
+        sort($backlinks);
+        $this->_set_links('i', $page, $backlinks);
+    }
         
     function _add_backlink($page, $linkedfrom) {
         $backlinks = $this->_get_links('i', $page);
@@ -506,8 +757,9 @@ class WikiDB_backend_dbaBase_linktable
     
     function _has_link($which, $page, $link) {
         $links = $this->_get_links($which, $page);
+        //TODO: since links are always sorted do a binary search or at least break if >
         foreach($links as $l) {
-            if ($l == $link)
+            if ($l['linkto'] == $link)
                 return true;
         }
         return false;
@@ -527,6 +779,20 @@ class WikiDB_backend_dbaBase_linktable
     }
 }
 
+// $Log: dbaBase.php,v $
+// Revision 1.29  2007/05/24 18:39:10  rurban
+// limits for get_all_pages, improved WantedPages
+//
+// Revision 1.28  2007/01/03 21:26:01  rurban
+// Fix dba searching for relations. Optimize link_search for strict attribute search. Add relation_search()
+//
+// Revision 1.27  2007/01/02 13:19:33  rurban
+// faster list_relations method. new native link_search method. additions to rebuild() (still very slow), fix iterator options (for want_relations and exclude). Clarify API: sortby,limit and exclude are strings
+//
+// Revision 1.26  2006/12/22 00:27:37  rurban
+// just add Log
+//
+
 // (c-file-style: "gnu")
 // Local Variables:
 // mode: php
@@ -535,4 +801,4 @@ class WikiDB_backend_dbaBase_linktable
 // c-hanging-comment-ender-p: nil
 // indent-tabs-mode: nil
 // End:   
-?>
\ No newline at end of file
+?>
