@@ -519,7 +519,7 @@ class GroupWikiPage extends WikiGroup{
             return false;
         }
         $contents = $group_revision->getContent();
-        $match = '/^\s*[\*\#]+\s*\[?' . $this->username . '\]?\s*$/';
+        $match = '/^\s*[\*\#]+\s*\[?' . $this->username . '\]?(\s|$)/';
         foreach ($contents as $line){
             if (preg_match($match, $line)) {
                 return true;
