@@ -1,5 +1,5 @@
 <?php //-*-php-*-
-rcs_id('$Id: AdoDb.php,v 1.7 2005/10/10 19:43:49 rurban Exp $');
+rcs_id('$Id: AdoDb.php,v 1.10 2007/05/30 21:53:52 rurban Exp $');
 /* Copyright (C) 2004 ReiniUrban
  * This file is part of PhpWiki. Terms and Conditions see LICENSE. (GPL2)
  */
@@ -125,7 +125,7 @@ extends _DbPassUser
         // Prepare the configured auth statements
         if ($dbi->getAuthParam('auth_check') and empty($this->_authselect)) {
             $this->_authselect = $this->prepare($dbi->getAuthParam('auth_check'), 
-                                                array("userid", "password"));
+                                                array("password", "userid"));
         }
         //NOTE: for auth_crypt_method='crypt' no special auth_user_exists is needed
         if ( !$dbi->getAuthParam('auth_user_exists') 
@@ -160,7 +160,7 @@ extends _DbPassUser
         // reasons
         if (empty($this->_authcreate) and $dbi->getAuthParam('auth_create')) {
             $this->_authcreate = $this->prepare($dbi->getAuthParam('auth_create'),
-                                                array("userid", "password"));
+                                                array("password", "userid"));
         }
         if (!empty($this->_authcreate) and 
             isset($GLOBALS['HTTP_POST_VARS']['auth']) and
@@ -222,7 +222,10 @@ extends _DbPassUser
             elseif (isset($rs->fields[0]))
                 $okay = $rs->fields[0];
             else {
-                $okay = reset($rs->fields);
+                if (is_array($rs->fields))
+                    $okay = reset($rs->fields);
+                else
+                    $okay = false;
             }
             $rs->Close();
             $result = !empty($okay);
@@ -249,7 +252,7 @@ extends _DbPassUser
         $dbi =& $GLOBALS['request']->_dbi;
         if ($dbi->getAuthParam('auth_update') and empty($this->_authupdate)) {
             $this->_authupdate = $this->prepare($dbi->getAuthParam('auth_update'),
-                                                array("userid", "password"));
+                                                array("password", "userid"));
         }
         if (!isset($this->_authupdate)) {
             trigger_error(fmt("Either %s is missing or DATABASE_TYPE != '%s'",
@@ -272,6 +275,15 @@ extends _DbPassUser
 }
 
 // $Log: AdoDb.php,v $
+// Revision 1.10  2007/05/30 21:53:52  rurban
+// add userid to authcreate
+//
+// Revision 1.9  2006/12/22 17:25:23  rurban
+// forgot why
+//
+// Revision 1.8  2006/03/19 16:26:40  rurban
+// fix DBAUTH arguments to be position independent, fixes bug #1358973
+//
 // Revision 1.7  2005/10/10 19:43:49  rurban
 // add DBAUTH_PREF_INSERT: self-creating users. by John Stevens
 //
@@ -307,4 +319,4 @@ extends _DbPassUser
 // c-hanging-comment-ender-p: nil
 // indent-tabs-mode: nil
 // End:
-?>
\ No newline at end of file
+?>
