@@ -1,6 +1,6 @@
 <?php //-*-php-*-
-rcs_id('$Id: LDAP.php,v 1.5 2005/10/10 19:43:49 rurban Exp $');
-/* Copyright (C) 2004 $ThePhpWikiProgrammingTeam
+rcs_id('$Id: LDAP.php,v 1.9 2007/06/13 12:48:14 rurban Exp $');
+/* Copyright (C) 2004,2007 $ThePhpWikiProgrammingTeam
  * This file is part of PhpWiki. Terms and Conditions see LICENSE. (GPL2)
  */
 
@@ -49,16 +49,24 @@ extends _PassUser
         unset($this->_sr);
         unset($this->_ldap);
     }
-    
+
     function checkPass($submitted_password) {
 
         $this->_authmethod = 'LDAP';
         $userid = $this->_userid;
         if (!$this->isValidName()) {
             trigger_error(_("Invalid username."), E_USER_WARNING);
+            $this->_free();
             return $this->_tryNextPass($submitted_password);
         }
         if (!$this->_checkPassLength($submitted_password)) {
+            $this->_free();
+            return WIKIAUTH_FORBIDDEN;
+        }
+        // A LDAP speciality: empty passwords are valid with ldap_bind!!!
+        if (strlen($submitted_password) == 0) {
+            trigger_error(_("Empty password not allowed for LDAP"), E_USER_WARNING);
+            $this->_free();
             return WIKIAUTH_FORBIDDEN;
         }
         if (strstr($userid,'*')) {
@@ -73,11 +81,14 @@ extends _PassUser
                 ? LDAP_SEARCH_FIELD."=$userid"
                 : "uid=$userid";
             if (!$this->_sr = ldap_search($ldap, LDAP_BASE_DN, $st_search)) {
+		trigger_error(_("Could not search in LDAP"), E_USER_WARNING);
  		$this->_free();
                 return $this->_tryNextPass($submitted_password);
             }
             $info = ldap_get_entries($ldap, $this->_sr); 
             if (empty($info["count"])) {
+		if (DEBUG)
+		    trigger_error(_("User not found in LDAP"), E_USER_WARNING);
             	$this->_free();
                 return $this->_tryNextPass($submitted_password);
             }
@@ -86,22 +97,52 @@ extends _PassUser
             for ($i = 0; $i < $info["count"]; $i++) {
                 $dn = $info[$i]["dn"];
                 // The password is still plain text.
+		// LDAP allows all chars but *, (, ), \, NUL
+		// Quoting is done by \xx (two-digit hexcode). * <=> \2a
+		// Handling '?' is unspecified
+		$password = strtr($submitted_password, 
+				array("*" => "\\2a",
+				      "?" => "\\3f",
+				      "(" => "\\28",
+				      ")" => "\\29",
+				      "\\" => "\\5c",
+				      "\0" => "\\00"));
                 // On wrong password the ldap server will return: 
                 // "Unable to bind to server: Server is unwilling to perform"
                 // The @ catches this error message.
-                if ($r = @ldap_bind($ldap, $dn, $submitted_password)) {
+                if ($r = @ldap_bind($ldap, $dn, $password)) {
                     // ldap_bind will return TRUE if everything matches
+		    // Get the mail from ldap
+		    if (!empty($info[$i]["mail"][0])) {
+			$this->_prefs->_prefs['email']->default_value = $info[$i]["mail"][0];
+		    }
             	    $this->_free();
                     $this->_level = WIKIAUTH_USER;
                     return $this->_level;
                 }
             }
+	    if (DEBUG)
+		trigger_error(_("Wrong password: ") . 
+			      str_repeat("*",strlen($submitted_password)), 
+			      E_USER_WARNING);
             $this->_free();
-        }
+        } else {
+            $this->_free();
+	    trigger_error(_("Could not connect to LDAP"), E_USER_WARNING);
+	}
 
         return $this->_tryNextPass($submitted_password);
     }
 
+
+    function isValidName ($userid = false) {
+        if (!$userid) $userid = $this->_userid;
+	// LDAP allows all chars but *, (, ), \, NUL
+	// Quoting is done by \xx (two-digit hexcode). * <=> \2a
+	// We are more restrictive here, but must allow explitly utf-8
+        return preg_match("/^[\-\w_\.@ ]+$/u", $userid) and strlen($userid) < 64;
+    }
+
     function userExists() {
         $userid = $this->_userid;
         if (strstr($userid, '*')) {
@@ -122,6 +163,7 @@ extends _PassUser
 
             if ($info["count"] > 0) {
          	$this->_free();
+		UpgradeUser($GLOBALS['ForbiddenUser'], $this);
                 return true;
             }
         }
@@ -136,6 +178,20 @@ extends _PassUser
 }
 
 // $Log: LDAP.php,v $
+// Revision 1.9  2007/06/13 12:48:14  rurban
+// fix wrong fix from 1.3.13p1
+//
+// Revision 1.8  2007/06/07 16:31:33  rurban
+// Important! Fixes bug #1732882 ldap_bind with empty password
+// Adds diagnostics on other ldap failures
+// Fix password quoting
+//
+// Revision 1.7  2007/05/30 21:56:17  rurban
+// Back to default uid for LDAP
+//
+// Revision 1.6  2007/05/29 16:56:15  rurban
+// Allow more password und userid chars. uid => cn: default for certain testusers
+//
 // Revision 1.5  2005/10/10 19:43:49  rurban
 // add DBAUTH_PREF_INSERT: self-creating users. by John Stevens
 //
