@@ -1,13 +1,13 @@
-<?php rcs_id('$Id: XmlElement.php,v 1.37 2005/01/25 07:04:27 rurban Exp $');
+<?php rcs_id('$Id: XmlElement.php,v 1.42 2007/01/20 11:41:30 rurban Exp $');
 /**
  * Code for writing XML.
  * @package Markup
  * @author: Jeff Dairiki,
  *          Reini Urban (php5 tricks)
  *
- * FIXME: This module is very php5 sensitive: It was fixed for 1.3.9, 
- *          but is again broken with the 1.3.11.
- *        With allow_call_time_pass_reference clean fixes
+ * WARNING: This module is very php5 sensitive. 
+ *          Fixed for 1.3.9, 1.3.11 and 1.3.13 (php-5.2).
+ *          With allow_call_time_pass_reference clean fixes.
  */
 
 /**
@@ -132,14 +132,20 @@ class XmlContent
         return $pdf;
     }
 
+    /* php-5.2 magic */
+    function __toString () {
+        return $this->asString();
+    }
+
     function asString () {
         $val = '';
         foreach ($this->_content as $item) {
             if (is_object($item)) {
-                if (method_exists($item, 'asString'))
-                    $val .= $item->asString();
-                else
+                if (method_exists($item, 'asString')) {
+                    $val .= (string) $item->asString();
+                } else {
                     $val .= sprintf("==Object(%s)==", get_class($item));
+                }
             }
             else
                 $val .= (string) $item;
@@ -422,6 +428,11 @@ class RawXml {
         echo $this->_xml;
     }
 
+    /* php-5.2 magic */
+    function __toString () {
+        return $this->_xml;
+    }
+
     function asXML () {
         return $this->_xml;
     }
@@ -490,18 +501,28 @@ class FormattedText {
             $args[] = AsString($arg);
         return call_user_func_array('sprintf', $args);
     }
+
+    /* php-5.2 magic */
+    function __toString () {
+        return $this->asString();
+    }
 }
 
 /**
  * PHP5 compatibility
  * Error[2048]: Non-static method XmlContent::_quote() should not be called statically
+ * Note: There's lot of room for performance increase if the right charset variant can 
+ * be created on load-time.
  */
 function XmlContent_quote ($string) {
     if (!$string) return $string;
-    if (check_php_version(4,1) and isset($GLOBALS['charset']))
+    if (check_php_version(4,1) and isset($GLOBALS['charset'])
+        and (!defined('IGNORE_CHARSET_NOT_SUPPORTED_WARNING') or !IGNORE_CHARSET_NOT_SUPPORTED_WARNING))
+    {
         return htmlspecialchars($string, ENT_COMPAT, $GLOBALS['charset']);
-    else
+    } else {
         return htmlspecialchars($string);
+    }
 }
 
 function PrintXML ($val /* , ... */ ) {
@@ -604,6 +625,21 @@ function fmt ($fs /* , ... */) {
 }
 
 // $Log: XmlElement.php,v $
+// Revision 1.42  2007/01/20 11:41:30  rurban
+// Add php-5.2.0 note
+//
+// Revision 1.41  2007/01/13 23:34:49  rurban
+// More php-5.2.0 fixes __toString
+//
+// Revision 1.40  2007/01/13 23:28:56  rurban
+// php-5.2.0 fix for string fallback: Provide the magic method __toString
+//
+// Revision 1.39  2006/08/15 13:36:23  rurban
+// support iso-8859-2
+//
+// Revision 1.38  2005/10/10 19:36:09  rurban
+// fix comment
+//
 // Revision 1.37  2005/01/25 07:04:27  rurban
 // case-sensitive for php5
 //
@@ -639,4 +675,4 @@ function fmt ($fs /* , ... */) {
 // c-hanging-comment-ender-p: nil
 // indent-tabs-mode: nil
 // End:   
-?>
\ No newline at end of file
+?>
