@@ -1,7 +1,7 @@
 <?php
-// $Id: XmlRpcServer.php,v 1.14 2005/07/21 19:27:04 rurban Exp $
+// $Id: XmlRpcServer.php,v 1.25 2007/03/18 10:27:11 rurban Exp $
 /* Copyright (C) 2002, Lawrence Akka <lakka@users.sourceforge.net>
- * Copyright (C) 2004, 2005 $ThePhpWikiProgrammingTeam
+ * Copyright (C) 2004,2005,2006,2007 $ThePhpWikiProgrammingTeam
  *
  * LICENCE
  * =======
@@ -45,62 +45,46 @@
 
 /*
 ToDo:
-        Remove all warnings from xmlrpc.inc 
-        Return list of external links in listLinks
-        Support RSS2 cloud subscription
+    * Change to unit tests: XmlRpcTest v1, v2 and private
+    * Return list of external links in listLinks
+    * Support RSS2 cloud subscription: wiki.rssPleaseNotify, pingback.ping
+
+    * API v2 http://www.jspwiki.org/wiki/WikiRPCInterface2 :
+
+    * array listAttachments( utf8 page ) - Lists attachments on a given page. 
+            The array consists of utf8 attachment names that can be fed to getAttachment (or putAttachment).
+    * base64 getAttachment( utf8 attachmentName ) - returns the content of an attachment.
+    * putAttachment( utf8 attachmentName, base64 content ) - (over)writes an attachment. 
+    * array system.listMethods()
+    * string system.methodHelp (string methodName)
+    * array system.methodSignature (string methodName) 
+
 Done:
         Test hwiki.jar xmlrpc interface (java visualization plugin)
      	Make use of the xmlrpc extension if found. http://xmlrpc-epi.sourceforge.net/
 	Resolved namespace conflicts
+	Added various phpwiki specific methods (mailPasswordToUser, getUploadedFileInfo, 
+	putPage, titleSearch, listPlugins, getPluginSynopsis, listRelations)
+	Use client methods in inter-phpwiki calls: SyncWiki, tests/xmlrpc/
 */
 
 // Intercept GET requests from confused users.  Only POST is allowed here!
-// There is some indication that $HTTP_SERVER_VARS is deprecated in php > 4.1.0
-// in favour of $_Server, but as far as I know, it still works.
-if ($GLOBALS['HTTP_SERVER_VARS']['REQUEST_METHOD'] != "POST")
+if (empty($GLOBALS['HTTP_SERVER_VARS']))
+    $GLOBALS['HTTP_SERVER_VARS']  =& $_SERVER;
+if ($GLOBALS['HTTP_SERVER_VARS']['REQUEST_METHOD'] != "POST")  
 {
     die('This is the address of the XML-RPC interface.' .
-        '  You must use XML-RPC calls to access information here');
+        '  You must use XML-RPC calls to access information here.');
 }
-
-// All these global declarations make it so that this file
-// (XmlRpcServer.php) can be included within a function body
-// (not in global scope), and things will still work....
-
-global $xmlrpcI4, $xmlrpcInt, $xmlrpcBoolean, $xmlrpcDouble, $xmlrpcString;
-global $xmlrpcDateTime, $xmlrpcBase64, $xmlrpcArray, $xmlrpcStruct;
-global $xmlrpcTypes;
-global $xmlEntities;
-global $xmlrpcerr, $xmlrpcstr;
-global $xmlrpc_defencoding;
-global $xmlrpcName, $xmlrpcVersion;
-global $xmlrpcerruser, $xmlrpcerrxml;
-global $xmlrpc_backslash;
-global $_xh;
-
+  
+require_once("lib/XmlRpcClient.php");
 if (loadPhpExtension('xmlrpc')) { // fast c lib
-    define('XMLRPC_EXT_LOADED', true);
-
-    global $xmlrpc_util_path;
-    $xmlrpc_util_path = dirname(__FILE__)."/XMLRPC/";
-    include_once("lib/XMLRPC/xmlrpc_emu.inc"); 
-    global $_xmlrpcs_debug;
-    include_once("lib/XMLRPC/xmlrpcs_emu.inc");
-
- } else { // slow php lib
-    define('XMLRPC_EXT_LOADED', true);
-
-    // Include the php XML-RPC library
-    include_once("lib/XMLRPC/xmlrpc.inc");
-
+    require_once("lib/XMLRPC/xmlrpcs_emu.inc");
+} else { // slow php lib
     global $_xmlrpcs_dmap;
-    global $_xmlrpcs_debug;
-    include_once("lib/XMLRPC/xmlrpcs.inc");
+    require_once("lib/XMLRPC/xmlrpcs.inc");
 }
- 
 
-//  API version
-define ("WIKI_XMLRPC_VERSION", 2);
 
 /**
  * Helper function:  Looks up a page revision (most recent by default) in the wiki database
@@ -130,50 +114,6 @@ function _getPageRevision ($params)
     return false;
 } 
 
-/*
- * Helper functions for encoding/decoding strings.
- *
- * According to WikiRPC spec, all returned strings take one of either
- * two forms.  Short strings (page names, and authors) are converted to
- * UTF-8, then rawurlencode()d, and returned as XML-RPC <code>strings</code>.
- * Long strings (page content) are converted to UTF-8 then returned as
- * XML-RPC <code>base64</code> binary objects.
- */
-
-/**
- * Urlencode ASCII control characters.
- *
- * (And control characters...)
- *
- * @param string $str
- * @return string
- * @see urlencode
- */
-function UrlencodeControlCharacters($str) {
-    return preg_replace('/([\x00-\x1F])/e', "urlencode('\\1')", $str);
-}
-
-/**
- * Convert a short string (page name, author) to xmlrpcval.
- */
-function short_string ($str) {
-    return new xmlrpcval(UrlencodeControlCharacters(utf8_encode($str)), 'string');
-}
-
-/**
- * Convert a large string (page content) to xmlrpcval.
- */
-function long_string ($str) {
-    return new xmlrpcval(utf8_encode($str), 'base64');
-}
-
-/**
- * Decode a short string (e.g. page name)
- */
-function short_string_decode ($str) {
-    return utf8_decode(urldecode($str));
-}
-
 /**
  * Get an xmlrpc "No such page" error message
  */
@@ -208,12 +148,14 @@ function getRPCVersionSupported($params)
  * array getRecentChanges(Date timestamp) : Get list of changed pages since 
  * timestamp, which should be in UTC. The result is an array, where each element
  * is a struct: 
- *     name (string) : Name of the page. The name is UTF-8 with URL encoding to make it ASCII. 
+ *     name (string)       : Name of the page. The name is UTF-8 with URL encoding to make it ASCII. 
  *     lastModified (date) : Date of last modification, in UTC. 
- *     author (string) : Name of the author (if available). Again, name is UTF-8 with URL encoding. 
- * 	   version (int) : Current version. 
+ *     author (string)     : Name of the author (if available). Again, name is UTF-8 with URL encoding. 
+ *     version (int)       : Current version. 
+ *     summary (string)    : UTF-8 with URL encoding. 
  * A page MAY be specified multiple times. A page MAY NOT be specified multiple 
  * times with the same modification date.
+ * Additionally to API version 1 and 2 we added the summary field.
  */
 $wiki_dmap['getRecentChanges']
 = array('signature'	=> array(array($xmlrpcArray, $xmlrpcDateTime)),
@@ -241,6 +183,7 @@ function getRecentChanges($params)
         $pages[] = new xmlrpcval(array('name' => $name, 
                                        'lastModified' => $lastmodified,
                                        'author' => $author,
+				       'summary' => short_string($page->get('summary')),
                                        'version' => $version),
                                  'struct');
     } 
@@ -352,7 +295,7 @@ function getAllPages($params)
  *   name (string): the canonical page name 
  *   lastModified (date): Last modification date 
  *   version (int): current version 
- * 	 author (string): author name 
+ *   author (string): author name 
  */
 $wiki_dmap['getPageInfo']
 = array('signature'	=> array(array($xmlrpcStruct, $xmlrpcString)),
@@ -397,8 +340,8 @@ function getPageInfoVersion($params)
 /*  array listLinks( string pagename ): Lists all links for a given page. The
  *  returned array contains structs, with the following elements: 
  *   	 name (string) : The page name or URL the link is to. 
- *       type (int) : The link type. Zero (0) for internal Wiki link,
- *         one (1) for external link (URL - image link, whatever).
+ *       type (int)    : The link type. Zero (0) for internal Wiki link,
+ *                        one (1) for external link (URL - image link, whatever).
  */
 $wiki_dmap['listLinks']
 = array('signature'	=> array(array($xmlrpcArray, $xmlrpcString)),
@@ -470,6 +413,10 @@ function listLinks($params)
     return new xmlrpcresp(new xmlrpcval ($linkstruct, "array"));
 } 
 
+/* End of WikiXMLRpc API v1 */
+/* ======================================================================== */
+/* Start of partial WikiXMLRpc API v2 support */
+
 /** 
  * struct putPage(String pagename, String content, [String author[, String password]})
  * returns a struct with elements: 
@@ -478,6 +425,8 @@ function listLinks($params)
  *   version (int): version of new page
  *
  * @author: Arnaud Fontaine, Reini Urban
+ *
+ * API notes: Contrary to the API v2 specs we dropped attributes and added author + password
  */
 $wiki_dmap['putPage']
 = array('signature'     => array(array($xmlrpcStruct, $xmlrpcString, $xmlrpcString, $xmlrpcString, $xmlrpcString)),
@@ -580,8 +529,48 @@ function putPage($params) {
                                         "struct"));
 }
 
+/* End of WikiXMLRpc API v2 */
+/* ======================================================================== */
+/* Start of private extensions */
+
 /**
- * Publish-Subscribe
+ * struct getUploadedFileInfo( string localpath ) : returns a struct with elements: 
+ *   lastModified (date): Last modification date 
+ *   size (int): current version 
+ * This is to sync uploaded files up to a remote master wiki. (SyncWiki)
+ * Not existing files return both 0.
+ *
+ * API notes: API v2 specs have array listAttachments( utf8 page ), 
+ * base64 getAttachment( utf8 attachmentName ), putAttachment( utf8 attachmentName, base64 content )
+ */
+$wiki_dmap['getUploadedFileInfo']
+= array('signature'	=> array(array($xmlrpcStruct, $xmlrpcString)),
+        'documentation' => 'Gets date and size about an uploaded local file',
+        'function'	=> 'getUploadedFileInfo');
+
+function getUploadedFileInfo($params)
+{
+    // localpath is the relative part after "Upload:"	
+    $ParamPath = $params->getParam(0);
+    $localpath = short_string_decode($ParamPath->scalarval());
+    preg_replace("/^[\\ \/ \.]/", "", $localpath); // strip hacks
+    $file = getUploadFilePath() . $localpath;
+    if (file_exists($file)) {
+        $size = filesize($file);
+        $lastmodified = filemtime($file);
+    } else {
+    	$size = 0;
+    	$lastmodified = 0;
+    }        
+    return new xmlrpcresp(new xmlrpcval
+    	(array('lastModified' => new xmlrpcval(iso8601_encode($lastmodified, 1),
+                                               "dateTime.iso8601"),
+               'size' => new xmlrpcval($size, "int")), 
+        "struct"));
+}
+
+/**
+ * Publish-Subscribe (not yet implemented)
  * Client subscribes to a RecentChanges-like channel, getting a short 
  * callback notification on every change. Like PageChangeNotification, just shorter 
  * and more complicated
@@ -611,7 +600,7 @@ function rssPleaseNotify($params)
 }
 
 /*
- *  String wiki.rssPleaseNotify ( username )
+ *  boolean wiki.mailPasswordToUser ( username )
  *  returns: true or false 
 
  */
@@ -638,14 +627,14 @@ function mailPasswordToUser($params)
 }
 
 /** 
- * struct titleSearch(String substring [, Integer option = 0])
+ * array wiki.titleSearch(String substring [, String option = "0"])
  * returns an array of matching pagenames.
  * TODO: standardize options
  *
  * @author: Reini Urban
  */
 $wiki_dmap['titleSearch']
-= array('signature'     => array(array($xmlrpcArray, $xmlrpcString, $xmlrpcInt)),
+= array('signature'     => array(array($xmlrpcArray, $xmlrpcString, $xmlrpcString)),
         'documentation' => "Return matching pagenames. 
 Option 1: caseexact, 2: regex, 4: starts_with, 8: exact",
         'function'      => 'titleSearch');
@@ -657,9 +646,10 @@ function titleSearch($params)
     $searchstring = short_string_decode($ParamPageName->scalarval());
     if (count($params->params) > 1) {
         $ParamOption = $params->getParam(1);
-        $option = $ParamOption->scalarval();
-    } else $option = 0;
-    // default option: substring, case-inexact
+        $option = (int) $ParamOption->scalarval();
+    } else 
+	$option = 0;
+    	// default option: substring, case-inexact
 
     $case_exact = $option & 1;
     $regex      = $option & 2;
@@ -674,8 +664,8 @@ function titleSearch($params)
         }
     } else {
     	if ($option & 4 or $option & 8) { 
-		    global $xmlrpcerruser;
-    		return new xmlrpcresp(0, $xmlrpcerruser + 1, "Invalid option");
+	    global $xmlrpcerruser;
+    	    return new xmlrpcresp(0, $xmlrpcerruser + 1, "Invalid option");
     	}
     }
     include_once("lib/TextSearchQuery.php");
@@ -689,18 +679,271 @@ function titleSearch($params)
     return new xmlrpcresp(new xmlrpcval($pages, "array"));
 }
 
+/** 
+ * array wiki.listPlugins()
+ *
+ * Returns an array of all available plugins. 
+ * For EditToolbar pluginPulldown via AJAX
+ *
+ * @author: Reini Urban
+ */
+$wiki_dmap['listPlugins']
+= array('signature'     => array(array($xmlrpcArray)),
+        'documentation' => "Return names of all plugins",
+        'function'      => 'listPlugins');
+
+function listPlugins($params)
+{
+    $plugin_dir = 'lib/plugin';
+    if (defined('PHPWIKI_DIR'))
+        $plugin_dir = PHPWIKI_DIR . "/$plugin_dir";
+    $pd = new fileSet($plugin_dir, '*.php');
+    $plugins = $pd->getFiles();
+    unset($pd);
+    sort($plugins);
+    $RetArray = array();
+    if (!empty($plugins)) {
+        require_once("lib/WikiPlugin.php");
+        $w = new WikiPluginLoader;
+        foreach ($plugins as $plugin) {
+            $pluginName = str_replace(".php", "", $plugin);
+            $p = $w->getPlugin($pluginName, false); // second arg?
+            // trap php files which aren't WikiPlugin~s: wikiplugin + wikiplugin_cached only
+            if (strtolower(substr(get_parent_class($p), 0, 10)) == 'wikiplugin') {
+                $RetArray[] = short_string($pluginName);
+            }
+        }
+    }
+  
+    return new xmlrpcresp(new xmlrpcval($RetArray, "array"));
+}
+
+/** 
+ * String wiki.getPluginSynopsis(String plugin)
+ *
+ * For EditToolbar pluginPulldown via AJAX
+ *
+ * @author: Reini Urban
+ */
+$wiki_dmap['getPluginSynopsis']
+= array('signature'     => array(array($xmlrpcArray, $xmlrpcString)),
+        'documentation' => "Return plugin synopsis",
+        'function'      => 'getPluginSynopsis');
+
+function getPluginSynopsis($params)
+{
+    $ParamPlugin = $params->getParam(0);
+    $pluginName = short_string_decode($ParamPlugin->scalarval());
+
+    require_once("lib/WikiPlugin.php");
+    $w = new WikiPluginLoader;
+    $synopsis = '';
+    $p = $w->getPlugin($pluginName, false); // second arg?
+    // trap php files which aren't WikiPlugin~s: wikiplugin + wikiplugin_cached only
+    if (strtolower(substr(get_parent_class($p), 0, 10)) == 'wikiplugin') {
+        $plugin_args = '';
+        $desc = $p->getArgumentsDescription();
+        $src = array("\n",'"',"'",'|','[',']','\\');
+        $replace = array('%0A','%22','%27','%7C','%5B','%5D','%5C');
+        $desc = str_replace("<br />",' ',$desc->asXML());
+        if ($desc)
+            $plugin_args = '\n'.str_replace($src, $replace, $desc);
+        $synopsis = "<?plugin ".$pluginName.$plugin_args."?>"; // args?
+    }
+   
+    return new xmlrpcresp(short_string($synopsis));
+}
+
+/** 
+ * array wiki.callPlugin(String name, String args)
+ *
+ * Returns an array of pages as returned by the plugins PageList call. 
+ * Only valid for plugins returning pagelists, e.g. BackLinks, AllPages, ...
+ * For various AJAX or WikiFormRich calls.
+ *
+ * @author: Reini Urban
+ */
+$wiki_dmap['callPlugin']
+= array('signature'     => array(array($xmlrpcArray, $xmlrpcString, $xmlrpcString)),
+        'documentation' => "Returns an array of pages as returned by the plugins PageList call",
+        'function'      => 'callPlugin');
+
+function callPlugin($params)
+{
+    global $request;
+    $dbi = $request->getDbh();
+    $ParamPlugin = $params->getParam(0);
+    $pluginName = short_string_decode($ParamPlugin->scalarval());
+    $ParamArgs = $params->getParam(1);
+    $plugin_args = short_string_decode($ParamArgs->scalarval());
+
+    $basepage = ''; //$pluginName;
+    require_once("lib/WikiPlugin.php");
+    $w = new WikiPluginLoader;
+    $p = $w->getPlugin($pluginName, false); // second arg?
+    $pagelist = $p->run($dbi, $plugin_args, $request, $basepage);
+    $list = array();
+    if (is_object($pagelist) and isa($pagelist, 'PageList')) {
+	foreach ($pagelist->_pages as $page) {
+	    $list[] = $page->getName();
+	}
+    }
+    return new xmlrpcresp(new xmlrpcval($list, "array"));
+}
+
+/** 
+ * array wiki.listRelations([ Integer option = 1 ])
+ *
+ * Returns an array of all available relation names.
+ *   option: 1 relations only ( with 0 also )
+ *   option: 2 attributes only
+ *   option: 3 both, all names of relations and attributes 
+ *   option: 4 unsorted, this might be added as bitvalue: 7 = 4+3. default: sorted
+ * For some semanticweb autofill methods.
+ *
+ * @author: Reini Urban
+ */
+$wiki_dmap['listRelations']
+= array('signature'     => array(array($xmlrpcArray, $xmlrpcInt)),
+        'documentation' => "Return names of all relations",
+        'function'      => 'listRelations');
+
+function listRelations($params)
+{
+    global $request;
+    $dbh = $request->getDbh();
+    if (count($params->params) > 0) {
+        $ParamOption = $params->getParam(0);
+        $option = (int) $ParamOption->scalarval();
+    } else 
+	$option = 1;
+    $also_attributes = $option & 2; 
+    $only_attributes = $option & 2 and !($option & 1); 
+    $sorted = !($option & 4);
+    return new xmlrpcresp(new xmlrpcval($dbh->listRelations($also_attributes,
+							    $only_attributes,
+							    $sorted), 
+					"array"));
+}
+
+/** 
+ * String pingback.ping(String sourceURI, String targetURI)
+
+Spec: http://www.hixie.ch/specs/pingback/pingback
+
+Parameters
+    sourceURI of type string
+        The absolute URI of the post on the source page containing the
+        link to the target site.
+    targetURI of type string
+        The absolute URI of the target of the link, as given on the source page.
+Return Value
+    A string, as described below.
+Faults
+    If an error condition occurs, then the appropriate fault code from
+    the following list should be used. Clients can quickly determine
+    the kind of error from bits 5-8. 0001x fault codes are used for
+    problems with the source URI, 0002x codes are for problems with
+    the target URI, and 0003x codes are used when the URIs are fine
+    but the pingback cannot be acknowledged for some other reaon.
+
+    0 
+    	A generic fault code. Servers MAY use this error code instead
+    	of any of the others if they do not have a way of determining
+    	the correct fault code.
+    00010 (16)
+        The source URI does not exist.
+    00011 (17)
+        The source URI does not contain a link to the target URI, and
+        so cannot be used as a source.
+    00020 (32)
+        The specified target URI does not exist. This MUST only be
+        used when the target definitely does not exist, rather than
+        when the target may exist but is not recognised. See the next
+        error.
+    00021 (33)
+        The specified target URI cannot be used as a target. It either
+        doesn't exist, or it is not a pingback-enabled resource. For
+        example, on a blog, typically only permalinks are
+        pingback-enabled, and trying to pingback the home page, or a
+        set of posts, will fail with this error.
+    00030 (48)
+        The pingback has already been registered.
+    00031 (49)
+        Access denied.
+    00032 (50)
+        The server could not communicate with an upstream server, or
+        received an error from an upstream server, and therefore could
+        not complete the request. This is similar to HTTP's 402 Bad
+        Gateway error. This error SHOULD be used by pingback proxies
+        when propagating errors.
+
+    In addition, [FaultCodes] defines some standard fault codes that
+    servers MAY use to report higher level errors.
+
+Servers MUST respond to this function call either with a single string
+or with a fault code.
+
+If the pingback request is successful, then the return value MUST be a
+single string, containing as much information as the server deems
+useful. This string is only expected to be used for debugging
+purposes.
+
+If the result is unsuccessful, then the server MUST respond with an
+RPC fault value. The fault code should be either one of the codes
+listed above, or the generic fault code zero if the server cannot
+determine the correct fault code.
+
+Clients MAY ignore the return value, whether the request was
+successful or not. It is RECOMMENDED that clients do not show the
+result of successful requests to the user.
+
+Upon receiving a request, servers MAY do what they like. However, the
+following steps are RECOMMENDED:
+
+   1. The server MAY attempt to fetch the source URI to verify that
+   the source does indeed link to the target.
+   2. The server MAY check its own data to ensure that the target
+   exists and is a valid entry.
+   3. The server MAY check that the pingback has not already been registered.
+   4. The server MAY record the pingback.
+   5. The server MAY regenerate the site's pages (if the pages are static).
+
+ * @author: Reini Urban
+ */
+$wiki_dmap['pingback.ping']
+= array('signature'     => array(array($xmlrpcString, $xmlrpcString, $xmlrpcString)),
+        'documentation' => "",
+        'function'      => 'pingBack');
+function pingBack($params)
+{
+    global $request;
+    $Param0 = $params->getParam(0);
+    $sourceURI = short_string_decode($Param0->scalarval());
+    $Param1 = $params->getParam(1);
+    $targetURI = short_string_decode($Param1->scalarval());
+    // TODO...
+}
+
+/* End of private WikiXMLRpc API extensions */
+/* ======================================================================== */
  
 /** 
  * Construct the server instance, and set up the dispatch map, 
  * which maps the XML-RPC methods on to the wiki functions.
- * Provide the "wiki." prefix to each function
+ * Provide the "wiki." prefix to each function. Besides 
+ * the blog - pingback, ... - functions with a seperate namespace.
  */
 class XmlRpcServer extends xmlrpc_server
 {
     function XmlRpcServer ($request = false) {
         global $wiki_dmap;
-        foreach ($wiki_dmap as $name => $val)
-            $dmap['wiki.' . $name] = $val;
+        foreach ($wiki_dmap as $name => $val) {
+            if ($name == 'pingback.ping') // non-wiki methods
+                $dmap[$name] = $val;
+            else
+                $dmap['wiki.' . $name] = $val;
+        }
 
         $this->xmlrpc_server($dmap, 0 /* delay service*/);
     }
@@ -723,6 +966,46 @@ class XmlRpcServer extends xmlrpc_server
     }
 }
 
+/*
+ $Log: XmlRpcServer.php,v $
+ Revision 1.25  2007/03/18 10:27:11  rurban
+ whitespace only
+
+ Revision 1.24  2007/01/22 23:42:13  rurban
+ Back to WIKI_XMLRPC_VERSION 1. Explain why
+
+ Revision 1.23  2007/01/10 20:47:45  rurban
+ change int to string because of acdropdown
+
+ Revision 1.22  2007/01/07 18:44:11  rurban
+ Add summary to getRecentChanges result
+
+ Revision 1.21  2007/01/04 16:42:13  rurban
+ Use require, not include!
+
+ Revision 1.20  2007/01/03 21:25:52  rurban
+ add option argument to listRelations.
+
+ Revision 1.19  2007/01/02 13:21:21  rurban
+ split client from server. added getUploadedFileInfo (for SyncWiki), callPlugin (for WikiFormRich)
+
+ Revision 1.18  2006/05/18 06:10:45  rurban
+ add xmlrpc listRelations signature
+
+ Revision 1.17  2005/10/31 16:49:31  rurban
+ fix doc
+
+ Revision 1.16  2005/10/29 14:17:51  rurban
+ fix doc
+
+ Revision 1.15  2005/10/29 08:57:12  rurban
+ fix for !register_long_arrays
+ new: array wiki.listPlugins()
+      String wiki.getPluginSynopsis(String plugin)
+      String pingback.ping(String sourceURI, String targetURI) (preliminary)
+
+
+ */
 
 // (c-file-style: "gnu")
 // Local Variables:
@@ -732,4 +1015,4 @@ class XmlRpcServer extends xmlrpc_server
 // c-hanging-comment-ender-p: nil
 // indent-tabs-mode: nil
 // End:   
-?>
\ No newline at end of file
+?>
