@@ -1,5 +1,5 @@
 <?php
-rcs_id('$Id: config.php,v 1.137 2005/08/06 14:31:10 rurban Exp $');
+rcs_id('$Id: config.php,v 1.145 2007/06/02 18:24:39 rurban Exp $');
 /*
  * NOTE: The settings here should probably not need to be changed.
  * The user-configurable settings have been moved to IniConfig.php
@@ -24,7 +24,10 @@ define ('_DEBUG_TRACE',     8); // test php memory usage, prints php debug backt
 define ('_DEBUG_INFO',     16);
 define ('_DEBUG_APD',      32);
 define ('_DEBUG_LOGIN',    64); // verbose login debug-msg (settings and reason for failure)
-define ('_DEBUG_SQL',     128);
+define ('_DEBUG_SQL',     128); // force check db, force optimize, print some debugging logs
+define ('_DEBUG_REMOTE',  256); // remote debug into subrequests (xmlrpc, ajax, wikiwyg, ...) 
+				// or test local SearchHighlight.
+				// internal links have persistent ?start_debug=1 
 
 function isCGI() {
     return (substr(php_sapi_name(),0,3) == 'cgi' and 
@@ -67,16 +70,17 @@ function browserAgent() {
     return $HTTP_USER_AGENT;
 }
 function browserDetect($match) {
-    return strstr(browserAgent(), $match);
+    return strstr(strtolower(browserAgent()), strtolower($match));
 }
 // returns a similar number for Netscape/Mozilla (gecko=5.0)/IE/Opera features.
 function browserVersion() {
-    if (strstr(browserAgent(),    "Mozilla/4.0 (compatible; MSIE"))
-        return (float) substr(browserAgent(),30);
-    elseif (strstr(browserAgent(),"Mozilla/5.0 (compatible; Konqueror/"))
-        return (float) substr(browserAgent(),36);
+    $agent = browserAgent();
+    if (strstr($agent, "Mozilla/4.0 (compatible; MSIE"))
+        return (float) substr($agent, 30);
+    elseif (strstr($agent, "Mozilla/5.0 (compatible; Konqueror/"))
+        return (float) substr($agent, 36);
     else
-        return (float) substr(browserAgent(),8);
+        return (float) substr($agent, 8);
 }
 function isBrowserIE() {
     return (browserDetect('Mozilla/') and 
@@ -112,8 +116,13 @@ function isBrowserKonqueror($version = false) {
 // MacOSX Safari has certain limitations. Need detection and patches.
 // * no <object>, only <embed>
 function isBrowserSafari($version = false) {
-    if ($version) return browserDetect('Safari/') and browserVersion() >= $version; 
-    return browserDetect('Safari/');
+    $found = browserDetect('spoofer') or browserDetect('applewebkit');
+    if ($version) return $found and browserVersion() >= $version; 
+    return $found;
+}
+function isBrowserOpera($version = false) {
+    if ($version) return browserDetect('Opera/') and browserVersion() >= $version; 
+    return browserDetect('Opera/');
 }
 
 
@@ -148,6 +157,7 @@ function guessing_lang ($languages=false) {
         */
     }
 
+    $accept = false; 
     if (isset($GLOBALS['request'])) // in fixup-dynamic-config there's no request yet
         $accept = $GLOBALS['request']->get('HTTP_ACCEPT_LANGUAGE');
     elseif (!empty($_SERVER['HTTP_ACCEPT_LANGUAGE']))
@@ -485,6 +495,26 @@ if (!check_php_version(5)) {
     ');
 }
 
+/**
+ * array_diff_assoc() returns an array containing all the values from array1 that are not
+ * present in any of the other arguments. Note that the keys are used in the comparison 
+ * unlike array_diff(). In core since php-4.3.0
+ * Our fallback here supports only hashes and two args.
+ * $array1 = array("a" => "green", "b" => "brown", "c" => "blue");
+ * $array2 = array("a" => "green", "y" => "yellow", "r" => "red");
+ * => b => brown, c => blue
+ */
+if (!function_exists('array_diff_assoc')) {
+    function array_diff_assoc($a1, $a2) {
+    	$result = array();
+    	foreach ($a1 as $k => $v) {
+    	    if (!isset($a2[$k]) or !$a2[$k])
+    	        $result[$k] = $v;	
+    	}
+    	return $result;
+    }
+}
+
 /** 
  * wordwrap() might crash between 4.1.2 and php-4.3.0RC2, fixed in 4.3.0
  * See http://bugs.php.net/bug.php?id=20927 and 
@@ -551,15 +581,92 @@ function safe_wordwrap($str, $width=80, $break="\n", $cut=false) {
 }
 
 function getUploadFilePath() {
+    if (defined('UPLOAD_FILE_PATH')) return UPLOAD_FILE_PATH;
     return defined('PHPWIKI_DIR') 
         ? PHPWIKI_DIR . "/uploads/" 
         : realpath(dirname(__FILE__) . "/../uploads/");
 }
 function getUploadDataPath() {
-  return SERVER_URL . ((substr(DATA_PATH,0,1)=='/') ? '' : "/") . DATA_PATH . '/uploads/';
+    if (defined('UPLOAD_DATA_PATH')) return UPLOAD_DATA_PATH;
+    return SERVER_URL . ((substr(DATA_PATH,0,1)=='/') ? '' : "/") . DATA_PATH . '/uploads/';
 }
 
+/**
+ * htmlspecialchars doesn't support some special 8bit charsets, which we do want to support.
+ * Well it just prints a warning which we could circumvent.
+ * Note: unused, since php htmlspecialchars does the same, just prints a warning which we silence
+ */
+/*
+function htmlspecialchars_workaround($str, $quote=ENT_COMPAT, $charset='iso-8859-1') {
+    if (in_array(strtolower($charset), 
+                 array('iso-8859-2', 'iso8859-2', 'latin-2', 'latin2'))) 
+    {
+        if (! ($quote & ENT_NOQUOTES)) {
+            $str = str_replace("\"", "&quot;",
+                               $str);
+        }
+        if ($quote & ENT_QUOTES) {
+            $str = str_replace("\'", "&#039;",
+                               $str);
+        }
+        return str_replace(array("<", ">", "&"),
+                           array("&lt;", "&gt;", "&amp;"), $str);
+    }
+    else {
+        return htmlspecialchars($str, $quote, $charset);
+    }
+}
+*/
+
+/**
+ * htmlspecialchars doesn't support some special 8bit charsets, which we do want to support.
+ * Well it just prints a warning which we could circumvent.
+ * Note: unused, since php htmlspecialchars does the same, just prints a warning which we silence
+ */
+/*
+function htmlspecialchars_workaround($str, $quote=ENT_COMPAT, $charset='iso-8859-1') {
+    if (in_array(strtolower($charset), 
+                 array('iso-8859-2', 'iso8859-2', 'latin-2', 'latin2'))) 
+    {
+        if (! ($quote & ENT_NOQUOTES)) {
+            $str = str_replace("\"", "&quot;",
+                               $str);
+        }
+        if ($quote & ENT_QUOTES) {
+            $str = str_replace("\'", "&#039;",
+                               $str);
+        }
+        return str_replace(array("<", ">", "&"),
+                           array("&lt;", "&gt;", "&amp;"), $str);
+    }
+    else {
+        return htmlspecialchars($str, $quote, $charset);
+    }
+}
+*/
+
 // $Log: config.php,v $
+// Revision 1.145  2007/06/02 18:24:39  rurban
+// case-insensitive browser check. Imrpoved isBrowserSafari
+//
+// Revision 1.143  2007/05/24 18:36:56  rurban
+// *** empty log message ***
+//
+// Revision 1.142  2007/03/04 14:07:17  rurban
+// Added SearchHighlight comment
+//
+// Revision 1.141  2007/01/07 18:41:42  rurban
+// Add array_diff_assoc() fallback: fixes Blog with php < 4.3.x. Support UPLOAD_FILE_PATH, UPLOAD_DATA_PATH overrides.
+//
+// Revision 1.140  2007/01/02 13:21:30  rurban
+// added _DEBUG_REMOTE flag, omit want_content if not necessary
+//
+// Revision 1.139  2006/03/19 14:50:42  rurban
+// sf.net patch #1438442 by Matt Brown: Unitialised variable reference in config.php
+//
+// Revision 1.138  2006/03/07 20:45:43  rurban
+// wikihash for php-5.1
+//
 // Revision 1.137  2005/08/06 14:31:10  rurban
 // ensure absolute uploads path
 //
@@ -742,4 +849,4 @@ function getUploadDataPath() {
 // c-hanging-comment-ender-p: nil
 // indent-tabs-mode: nil
 // End:
-?>
\ No newline at end of file
+?>
