@@ -1,18 +1,8 @@
 <?php
-rcs_id('$Id: editpage.php,v 1.96 2005/05/06 17:54:22 rurban Exp $');
+rcs_id('$Id: editpage.php,v 1.111 2007/06/03 17:12:00 rurban Exp $');
 
 require_once('lib/Template.php');
 
-// USE_HTMLAREA - Support for some WYSIWYG HTML Editor
-// Not yet enabled, since we cannot convert HTML to Wiki Markup yet.
-// (See HtmlParser.php for the ongoing efforts)
-// We might use a HTML PageType, which is contra wiki, but some people might prefer HTML markup.
-// TODO: Change from constant to user preference variable (checkbox setting),
-//       when HtmlParser is finished.
-if (!defined('USE_HTMLAREA')) define('USE_HTMLAREA', false);
-if (USE_HTMLAREA) require_once('lib/htmlarea.php');
-if (ENABLE_CAPTCHA)  require_once('lib/Captcha.php'); 
-
 class PageEditor
 {
     function PageEditor (&$request) {
@@ -34,11 +24,19 @@ class PageEditor
                             'mtime' => time());
         
         $this->tokens = array();
+
+        if (ENABLE_WYSIWYG) {
+            $backend = WYSIWYG_BACKEND;
+            // TODO: error message
+            require_once("lib/WysiwygEdit/$backend.php");
+            $class = "WysiwygEdit_$backend";
+            $this->WysiwygEdit = new $class();
+        }
         if (ENABLE_CAPTCHA) {
             require_once('lib/Captcha.php');
             $this->Captcha = new Captcha($this->meta);
         }
-        
+
         $version = $request->getArg('version');
         if ($version !== false) {
             $this->selected = $this->page->getRevision($version);
@@ -88,6 +86,9 @@ class PageEditor
             and ($r->getPref('editHeight') != $r->args['pref']['editHeight'])) {
             $r->_prefs->set('editHeight', $r->args['pref']['editHeight']);
         }
+        
+        if ($this->isModerated())
+            $tokens['PAGE_LOCKED_MESSAGE'] = $this->getModeratedMessage();
 
         if (! $this->canEdit()) {
             if ($this->isInitialEdit())
@@ -101,6 +102,7 @@ class PageEditor
 	    }
             elseif ( $this->savePage()) {
                 // noreturn
+                $request->setArg('action', false);
                 $r->redirect(WikiURL($r->getArg('save_and_redirect_to')));
                 return true;    // Page saved.
             }
@@ -118,6 +120,32 @@ class PageEditor
                 $saveFailed = true;
             }
         }
+	// coming from loadfile conflicts 
+        elseif ($this->editaction == 'keep_old') {
+	    // keep old page and do nothing
+            $this->_redirectToBrowsePage();
+            //$r->redirect(WikiURL($r->getArg('save_and_redirect_to')));
+	    return true;
+        }
+        elseif ($this->editaction == 'overwrite') { 
+            // take the new content without diff
+	    $source = $this->request->getArg('loadfile');
+	    require_once('lib/loadsave.php');
+	    $this->request->setArg('loadfile', 1);
+	    $this->request->setArg('overwrite', 1);
+	    $this->request->setArg('merge', 0);
+	    LoadFileOrDir($this->request);
+            $this->_redirectToBrowsePage();
+            //$r->redirect(WikiURL($r->getArg('save_and_redirect_to')));
+            return true;
+        }
+        elseif ($this->editaction == 'upload') {
+            // run plugin UpLoad
+            $plugin = WikiPluginLoader("UpLoad");
+            $plugin->run();
+            // add link to content	 
+            ;
+        }
 
         if ($saveFailed and $this->isConcurrentUpdate())
         {
@@ -129,7 +157,7 @@ class PageEditor
             $orig_content = $orig->getContent();
             $this_content = explode("\n", $this->_content);
             $other_content = $this->current->getContent();
-            include_once("lib/diff3.php");
+            require_once("lib/diff3.php");
             $diff = new diff3($orig_content, $this_content, $other_content);
             $output = $diff->merged_output(_("Your version"), _("Other version"));
             // Set the content of the textarea to the merged diff
@@ -151,12 +179,14 @@ class PageEditor
             $tokens['PREVIEW_CONTENT'] = $this->getConvertedPreview();
         if ($this->editaction == 'preview')
             $tokens['PREVIEW_CONTENT'] = $this->getPreview(); // FIXME: convert to _MESSAGE?
+        if ($this->editaction == 'diff')
+            $tokens['PREVIEW_CONTENT'] = $this->getDiff();
 
         // FIXME: NOT_CURRENT_MESSAGE?
         $tokens = array_merge($tokens, $this->getFormElements());
 
-        if (ENABLE_EDIT_TOOLBAR) {
-            include_once("lib/EditToolbar.php");
+        if (ENABLE_EDIT_TOOLBAR and !ENABLE_WYSIWYG) {
+            require_once("lib/EditToolbar.php");
             $toolbar = new EditToolbar();
             $tokens = array_merge($tokens, $toolbar->getTokens());
         }
@@ -179,11 +209,15 @@ class PageEditor
         }
 
         $title = new FormattedText ($title_fs, $pagelink);
-        if (USE_HTMLAREA and $template == 'editpage') {
-            $WikiTheme->addMoreHeaders(Edit_HtmlArea_Head());
-            //$tokens['PAGE_SOURCE'] = Edit_HtmlArea_ConvertBefore($this->_content);
+        // not for dumphtml or viewsource
+        if (ENABLE_WYSIWYG and $template == 'editpage') {
+            $WikiTheme->addMoreHeaders($this->WysiwygEdit->Head());
+            //$tokens['PAGE_SOURCE'] = $this->WysiwygEdit->ConvertBefore($this->_content);
         }
         $template = Template($template, $this->tokens);
+	/* Tell google (and others) not to take notice of edit links */
+	if (GOOGLE_LINKS_NOFOLLOW)
+	    $args = array('ROBOTS_META' => "noindex,nofollow");
         GeneratePage($template, $title, $rev);
         return true;
     }
@@ -227,8 +261,9 @@ class PageEditor
             // Save failed. No changes made.
             $this->_redirectToBrowsePage();
             // user will probably not see the rest of this...
-            include_once('lib/display.php');
+            require_once('lib/display.php');
             // force browse of current version:
+            $request->setArg('action', false);
             $request->setArg('version', false);
             displayPage($request, 'nochanges');
             return true;
@@ -241,7 +276,7 @@ class PageEditor
             // Save failed. No changes made.
             $this->_redirectToBrowsePage();
             // user will probably not see the rest of this...
-            include_once('lib/display.php');
+            require_once('lib/display.php');
             // force browse of current version:
             $request->setArg('version', false);
             displayPage($request, 'nochanges');
@@ -276,7 +311,7 @@ class PageEditor
         $this->updateLock();
 
         // Clean out archived versions of this page.
-        include_once('lib/ArchiveCleaner.php');
+        require_once('lib/ArchiveCleaner.php');
         $cleaner = new ArchiveCleaner($GLOBALS['ExpireParams']);
         $cleaner->cleanPageRevisions($page);
 
@@ -297,17 +332,21 @@ class PageEditor
             // been defined.  In this case, the user will most
             // likely not see the rest of the HTML we generate
             // (below).
+            $request->setArg('action', false);
             $this->_redirectToBrowsePage();
         }
 
         // Force browse of current page version.
         $request->setArg('version', false);
-        //$request->setArg('action', false);
+        // testme: does preview and more need action=edit?
+        $request->setArg('action', false);
 
         $template = Template('savepage', $this->tokens);
         $template->replace('CONTENT', $newrevision->getTransformedContent());
-        if (!empty($warnings->_content))
+        if (!empty($warnings->_content)) {
             $template->replace('WARNINGS', $warnings);
+            unset($GLOBALS['ErrorManager']->_postponed_errors); 
+        }
 
         $pagelink = WikiLink($page);
 
@@ -352,12 +391,13 @@ class PageEditor
 
         $oldtext = $current->getPackedContent();
         $newtext =& $this->_content;
-
+        $numlinks = $this->numLinks($newtext);
+        $newlinks = $numlinks - $this->numLinks($oldtext);
         // FIXME: in longer texts the NUM_SPAM_LINKS number should be increased.
         //        better use a certain text : link ratio.
 
         // 1. Not more then 20 new external links
-        if ($this->numLinks($newtext) - $this->numLinks($oldtext) >= NUM_SPAM_LINKS)
+        if ($newlinks >= NUM_SPAM_LINKS)
         {
             // Allow strictly authenticated users?
             // TODO: mail the admin?
@@ -369,7 +409,7 @@ class PageEditor
         // 2. external babycart (SpamAssassin) check
         // This will probably prevent from discussing sex or viagra related topics. So beware.
         if (ENABLE_SPAMASSASSIN) {
-            include_once("lib/spam_babycart.php");
+            require_once("lib/spam_babycart.php");
             if ($babycart = check_babycart($newtext, $request->get("REMOTE_ADDR"), 
                                            $this->user->getId())) {
                 // TODO: mail the admin
@@ -382,24 +422,37 @@ class PageEditor
             }
         }
         // 3. extract (new) links and check surbl for blocked domains
-        if (ENABLE_SPAMBLOCKLIST and $this->numLinks($newtext)) {
-            include_once("lib/SpamBlocklist.php");
-            include_once("lib/InlineParser.php");
+        if (ENABLE_SPAMBLOCKLIST and ($newlinks > 5)) {
+            require_once("lib/SpamBlocklist.php");
+            require_once("lib/InlineParser.php");
             $parsed = TransformLinks($newtext);
-            foreach ($parsed->_content as $link) {
-            	if (isa($link, 'Cached_ExternalLink')) {
-                  $uri = $link->_getURL($this->page->getName());
-                  if ($res = IsBlackListed($uri)) {
-                    // TODO: mail the admin
-                    $this->tokens['PAGE_LOCKED_MESSAGE'] = 
-                        HTML($this->getSpamMessage(),
-                             HTML::p(HTML::strong(_("External links contain blocked domains:")),
-                             HTML::ul(HTML::li(sprintf(_("%s is listed at %s"), 
-                                                       $res[2], $res[0])))));
-                    return true;
-                  }
+            $oldparsed = TransformLinks($oldtext);
+            $oldlinks = array();
+            foreach ($oldparsed->_content as $link) {
+            	if (isa($link, 'Cached_ExternalLink') and !isa($link, 'Cached_InterwikiLink')) {
+                    $uri = $link->_getURL($this->page->getName());
+                    $oldlinks[$uri]++;
             	}
             }
+            unset($oldparsed);
+            foreach ($parsed->_content as $link) {
+            	if (isa($link, 'Cached_ExternalLink') and !isa($link, 'Cached_InterwikiLink')) {
+                    $uri = $link->_getURL($this->page->getName());
+                    // only check new links, so admins may add blocked links.
+                    if (!array_key_exists($uri, $oldlinks) and ($res = IsBlackListed($uri))) {
+                        // TODO: mail the admin
+                        $this->tokens['PAGE_LOCKED_MESSAGE'] = 
+                            HTML($this->getSpamMessage(),
+                                 HTML::p(HTML::strong(_("External links contain blocked domains:")),
+                                         HTML::ul(HTML::li(sprintf(_("%s is listed at %s with %s"), 
+                                                                   $uri." [".$res[2]."]", $res[0], $res[1])))));
+                        return true;
+                    }
+            	}
+            }
+            unset($oldlinks);
+            unset($parsed);
+            unset($oldparsed);
         }
 
         return false;
@@ -423,24 +476,51 @@ class PageEditor
     }
 
     function getPreview () {
-        include_once('lib/PageType.php');
+        require_once('lib/PageType.php');
         $this->_content = $this->getContent();
 	return new TransformedText($this->page, $this->_content, $this->meta);
     }
 
     function getConvertedPreview () {
-        include_once('lib/PageType.php');
+        require_once('lib/PageType.php');
         $this->_content = $this->getContent();
         $this->meta['markup'] = 2.0;
         $this->_content = ConvertOldMarkup($this->_content);
 	return new TransformedText($this->page, $this->_content, $this->meta);
     }
 
+    function getDiff () {
+        require_once('lib/diff.php');
+	$html = HTML();
+	
+	$diff = new Diff($this->current->getContent(), explode("\n", $this->getContent()));
+	if ($diff->isEmpty()) {
+	    $html->pushContent(HTML::hr(),
+			       HTML::p('[', _("Versions are identical"),
+				       ']'));
+	}
+	else {
+	    // New CSS formatted unified diffs (ugly in NS4).
+	    $fmt = new HtmlUnifiedDiffFormatter;
+	    // Use this for old table-formatted diffs.
+	    //$fmt = new TableUnifiedDiffFormatter;
+	    $html->pushContent($fmt->format($diff));
+	}
+        return $html;
+    }
+
     // possibly convert HTMLAREA content back to Wiki markup
     function getContent () {
-        if (USE_HTMLAREA) {
-            $xml_output = Edit_HtmlArea_ConvertAfter($this->_content);
-            $this->_content = join("", $xml_output->_content);
+        if (ENABLE_WYSIWYG) {
+            // don't store everything as html
+            if (!WYSIWYG_DEFAULT_PAGETYPE_HTML) {
+                // Wikiwyg shortcut to avoid the InlineTransformer:
+                if (WYSIWYG_BACKEND == "Wikiwyg") return $this->_content;
+                $xml_output = $this->WysiwygEdit->ConvertAfter($this->_content);
+                $this->_content = join("", $xml_output->_content);
+            } else {
+                $this->meta['pagetype'] = 'html';
+            }
             return $this->_content;
         } else {
             return $this->_content;
@@ -455,6 +535,16 @@ class PageEditor
                  HTML::p(_("Sorry for the inconvenience.")));
     }
 
+    function isModerated() {
+        return $this->page->get('moderation');
+    }
+    function getModeratedMessage() {
+        return
+            HTML(HTML::h2(WikiLink(_("ModeratedPage"))),
+                 HTML::p(fmt("You can edit away, but your changes will have to be approved by the defined moderators at the definition in %s", WikiLink(_("ModeratedPage")))),
+                 HTML::p(fmt("The approval has a grace period of 5 days. If you have your E-Mail defined in your %s, you will get a notification of approval or rejection.", 
+                         WikiLink(_("UserPreferences")))));
+    }
     function getConflictMessage ($unresolved = false) {
         /*
          xgettext only knows about c/c++ line-continuation strings
@@ -491,15 +581,18 @@ class PageEditor
         $request = &$this->request;
 
         $readonly = ! $this->canEdit(); // || $this->isConcurrentUpdate();
-        if (USE_HTMLAREA) {
-            $html = $this->getPreview();
+
+        // WYSIWYG will need two pagetypes: raw wikitest and converted html
+        if (ENABLE_WYSIWYG) {
             $this->_wikicontent = $this->_content;
-            $this->_content = $html->asXML();
+            $this->_content = $this->WysiwygEdit->ConvertBefore($this->_content);
+            //                $this->getPreview();
+            //$this->_htmlcontent = $this->_content->asXML();
         }
 
         $textarea = HTML::textarea(array('class'=> 'wikiedit',
                                          'name' => 'edit[content]',
-                                         'id'   => 'edit[content]',
+                                         'id'   => 'edit-content',
                                          'rows' => $request->getPref('editHeight'),
                                          'cols' => $request->getPref('editWidth'),
                                          'readonly' => (bool) $readonly),
@@ -509,9 +602,10 @@ class PageEditor
          */
         if (isBrowserNS4())
             $textarea->setAttr('wrap', 'virtual');
-        if (USE_HTMLAREA)
-            return Edit_HtmlArea_Textarea($textarea,$this->_wikicontent,'edit[content]');
-        else
+        if (ENABLE_WYSIWYG) {
+            return $this->WysiwygEdit->Textarea($textarea, $this->_wikicontent, 
+                                                $textarea->getAttr('name'));
+        } else
             return $textarea;
     }
 
@@ -534,7 +628,7 @@ class PageEditor
         $el['SUMMARY_INPUT']
             = HTML::input(array('type'  => 'text',
                                 'class' => 'wikitext',
-                                'id' => 'edit[summary]',
+                                'id' => 'edit-summary',
                                 'name'  => 'edit[summary]',
                                 'size'  => 50,
                                 'maxlength' => 256,
@@ -542,7 +636,7 @@ class PageEditor
         $el['MINOR_EDIT_CB']
             = HTML::input(array('type' => 'checkbox',
                                 'name'  => 'edit[minor_edit]',
-                                'id' => 'edit[minor_edit]',
+                                'id' => 'edit-minor_edit',
                                 'checked' => (bool) $this->meta['is_minor_edit']));
         $el['OLD_MARKUP_CB']
             = HTML::input(array('type' => 'checkbox',
@@ -556,36 +650,53 @@ class PageEditor
         $el['LOCKED_CB']
             = HTML::input(array('type' => 'checkbox',
                                 'name' => 'edit[locked]',
-                                'id'   => 'edit[locked]',
+                                'id'   => 'edit-locked',
                                 'disabled' => (bool) !$this->user->isadmin(),
                                 'checked'  => (bool) $this->locked));
 
         $el['PREVIEW_B'] = Button('submit:edit[preview]', _("Preview"),
-                                  'wikiaction');
+                                  'wikiaction',
+                                  array('accesskey'=> 'p', 
+                                     'title' => 'Preview the current content [alt-p]'));
 
         //if (!$this->isConcurrentUpdate() && $this->canEdit())
-        $el['SAVE_B'] = Button('submit:edit[save]', _("Save"), 'wikiaction');
-
+        $el['SAVE_B'] = Button('submit:edit[save]',
+                               _("Save"), 'wikiaction',
+                               array('accesskey'=> 's', 
+                                     'title' => 'Save the current content as wikipage [alt-s]'));
+        $el['CHANGES_B'] = Button('submit:edit[diff]',
+                               _("Changes"), 'wikiaction',
+                               array('accesskey'=> 'c', 
+                                     'title' => 'Preview the current changes as diff [alt-c]'));
+        $el['UPLOAD_B'] = Button('submit:edit[upload]',
+                               _("Upload"), 'wikiaction',
+                                array('title' => 'Select a local file and press Upload to attach into this page'));
+        $el['SPELLCHECK_B'] = Button('submit:edit[SpellCheck]',
+                               _("Spell Check"), 'wikiaction',
+                                array('title' => 'Check the spelling'));
         $el['IS_CURRENT'] = $this->version == $this->current->getVersion();
 
-        $el['WIDTH_PREF'] = HTML::input(array('type' => 'text',
-                                    'size' => 3,
-                                    'maxlength' => 4,
-                                    'class' => "numeric",
-                                    'name' => 'pref[editWidth]',
-                                    'id'   => 'pref[editWidth]',
-                                    'value' => $request->getPref('editWidth'),
-                                    'onchange' => 'this.form.submit();'));
-        $el['HEIGHT_PREF'] = HTML::input(array('type' => 'text',
-                                     'size' => 3,
-                                     'maxlength' => 4,
-                                     'class' => "numeric",
-                                     'name' => 'pref[editHeight]',
-                                     'id'   => 'pref[editHeight]',
-                                     'value' => $request->getPref('editHeight'),
-                                     'onchange' => 'this.form.submit();'));
+        $el['WIDTH_PREF'] 
+            = HTML::input(array('type'     => 'text',
+                                'size'     => 3,
+                                'maxlength'=> 4,
+                                'class'    => "numeric",
+                                'name'     => 'pref[editWidth]',
+                                'id'       => 'pref-editWidth',
+                                'value'    => $request->getPref('editWidth'),
+                                'onchange' => 'this.form.submit();'));
+        $el['HEIGHT_PREF'] 
+            = HTML::input(array('type'     => 'text',
+                                'size'     => 3,
+                                'maxlength'=> 4,
+                                'class'    => "numeric",
+                                'name'     => 'pref[editHeight]',
+                                'id'       => 'pref-editHeight',
+                                'value'    => $request->getPref('editHeight'),
+                                'onchange' => 'this.form.submit();'));
         $el['SEP'] = $WikiTheme->getButtonSeparator();
-        $el['AUTHOR_MESSAGE'] = fmt("Author will be logged as %s.", HTML::em($this->user->getId()));
+        $el['AUTHOR_MESSAGE'] = fmt("Author will be logged as %s.", 
+                                    HTML::em($this->user->getId()));
         
         return $el;
     }
@@ -600,8 +711,9 @@ class PageEditor
         $posted = $request->getArg('edit');
         $request->setArg('edit', false);
 
-        if (!$posted || !$request->isPost()
-            || $request->getArg('action') != 'edit')
+        if (!$posted 
+	    || !$request->isPost()
+            || !in_array($request->getArg('action'),array('edit','loadfile')))
             return false;
 
         if (!isset($posted['content']) || !is_string($posted['content']))
@@ -629,13 +741,13 @@ class PageEditor
         $this->meta = array_merge($this->meta, $meta);
         $this->locked = !empty($posted['locked']);
 
-        if (!empty($posted['preview']))
-            $this->editaction = 'preview';
-        elseif (!empty($posted['save']))
-            $this->editaction = 'save';
-        elseif (!empty($posted['edit_convert']))
-            $this->editaction = 'edit_convert';
-        else
+	foreach (array('preview','save','edit_convert',
+		       'keep_old','overwrite','diff','upload') as $o) 
+	{
+	    if (!empty($posted[$o]))
+		$this->editaction = $o;
+	}
+        if (empty($this->editaction))
             $this->editaction = 'edit';
 
         return true;
@@ -684,13 +796,15 @@ extends PageEditor
         $tokens = &$this->tokens;
 
         if (!$this->canEdit()) {
-            if ($this->isInitialEdit())
+            if ($this->isInitialEdit()) {
                 return $this->viewSource();
+	    }
             $tokens['PAGE_LOCKED_MESSAGE'] = $this->getLockedMessage();
         }
         elseif ($this->editaction == 'save') {
-            if ($this->savePage())
+            if ($this->savePage()) {
                 return true;    // Page saved.
+	    }
             $saveFailed = true;
         }
 
@@ -702,7 +816,7 @@ extends PageEditor
             $orig = $this->page->getRevision($this->_currentVersion);
             $this_content = explode("\n", $this->_content);
             $other_content = $this->current->getContent();
-            include_once("lib/diff.php");
+            require_once("lib/diff.php");
             $diff2 = new Diff($other_content, $this_content);
             $context_lines = max(4, count($other_content) + 1,
                                  count($this_content) + 1);
@@ -727,9 +841,31 @@ extends PageEditor
 
         // FIXME: NOT_CURRENT_MESSAGE?
         $tokens = array_merge($tokens, $this->getFormElements());
+	// we need all GET params for loadfile overwrite
+	if ($this->request->getArg('action') == 'loadfile') {
+		
+	    $this->tokens['HIDDEN_INPUTS'] = 
+		HTML(HiddenInputs
+		    (array('source' => $this->request->getArg('source'),
+		           'merge'  => 1)),
+		     $this->tokens['HIDDEN_INPUTS']);
+	    // add two conflict resolution buttons before preview and save.
+	    $tokens['PREVIEW_B'] = HTML(
+				    Button('submit:edit[keep_old]', 
+					   _("Keep old"), 'wikiaction'),
+				    $tokens['SEP'],
+				    Button('submit:edit[overwrite]', 
+					   _("Overwrite with new"), 'wikiaction'),
+				    $tokens['SEP'],
+				    $tokens['PREVIEW_B']);
+	}
+	if (ENABLE_EDIT_TOOLBAR and !ENABLE_WYSIWYG) {
+            include_once("lib/EditToolbar.php");
+            $toolbar = new EditToolbar();
+            $tokens = array_merge($tokens, $toolbar->getTokens());
+	}
 
         return $this->output('editpage', _("Merge and Edit: %s"));
-        // FIXME: this doesn't display
     }
 
     function output ($template, $title_fs) {
@@ -745,7 +881,14 @@ extends PageEditor
             $pagelink = WikiLink($this->page);
         }
 
-        //$title = new FormattedText ($title_fs, $pagelink);
+        $title = new FormattedText ($title_fs, $pagelink);
+	$this->tokens['HEADER'] = $title;
+	//hack! there's no TITLE in editpage, but in the previous top template
+	if (empty($this->tokens['PAGE_LOCKED_MESSAGE']))
+	    $this->tokens['PAGE_LOCKED_MESSAGE'] = HTML::h3($title);
+	else
+	    $this->tokens['PAGE_LOCKED_MESSAGE'] = HTML(HTML::h3($title),
+							$this->tokens['PAGE_LOCKED_MESSAGE']);
         $template = Template($template, $this->tokens);
 
         //GeneratePage($template, $title, $rev);
@@ -764,12 +907,42 @@ extends PageEditor
 
 /**
  $Log: editpage.php,v $
+ Revision 1.112  2007/06/09 20:05:35  rurban
+ fix and optimize ENABLE_SPAMBLOCKLIST and ($newlinks > 5))
+
+ Revision 1.111  2007/06/03 17:12:00  rurban
+ convenience: only check above 5 external links for blocked domains
+
+ Revision 1.110  2007/01/07 18:42:00  rurban
+ Print ModeratedPage message on edit. Use GOOGLE_LINKS_NOFOLLOW. Improve id: edit: to edit-
+
+ Revision 1.109  2007/01/02 13:21:39  rurban
+ add two merge conflict buttons within loadfile: "Keep Old" and "Overwrite with new". enable edit toolbar there also. fix display of the Merge and Edit header.
+
+ Revision 1.108  2006/12/22 17:47:34  rurban
+ Display Warnings only once.
+ Add button accesskeys
+
+ Revision 1.107  2006/05/13 19:59:54  rurban
+ added wysiwyg_editor-1.3a feature by Jean-Nicolas GEREONE <jean-nicolas.gereone@st.com>
+ converted wysiwyg_editor-1.3a js to WysiwygEdit framework
+ changed default ENABLE_WYSIWYG = true and added WYSIWYG_BACKEND = Wikiwyg
+
  Revision 1.106  2005/11/21 22:03:08  rurban
  fix syntax error inside ENABLE_SPAMBLOCKLIST
 
  Revision 1.105  2005/11/21 20:53:59  rurban
  beautify request pref lines, no antispam if admin (netznetz request), user is a member anyway
 
+ Revision 1.104  2005/10/31 17:20:40  rurban
+ fix ConvertBefore
+
+ Revision 1.103  2005/10/31 17:09:13  rurban
+ use better constant WYSIWYG_DEFAULT_PAGETYPE_HTML
+
+ Revision 1.102  2005/10/31 16:47:14  rurban
+ enable wysiwyg html converters
+
  Revision 1.101  2005/10/30 16:12:28  rurban
  simplify viewsource tokens
 
@@ -782,6 +955,12 @@ extends PageEditor
    Check for links to blocked external tld domains in new edits, against
    multi.surbl.org and bl.spamcop.net.
 
+ Revision 1.98  2005/10/10 19:37:04  rurban
+ change USE_HTMLAREA to ENABLE WYSIWYG, add NUM_SPAM_LINKS=20
+
+ Revision 1.97  2005/09/26 06:32:22  rurban
+ [] is forbidden in id tags. Renamed to use :
+
  Revision 1.96  2005/05/06 17:54:22  rurban
  silence Preview warnings for PAGE_LOCKED_MESSAGE, CONCURRENT_UPDATE_MESSAGE (thanks to schorni)
 
@@ -998,4 +1177,4 @@ extends PageEditor
 // c-hanging-comment-ender-p: nil
 // indent-tabs-mode: nil
 // End:
-?>
\ No newline at end of file
+?>
