@@ -1,8 +1,8 @@
 <?php //-*-php-*-
-rcs_id('$Id: loadsave.php,v 1.137 2005/01/30 23:14:38 rurban Exp $');
+rcs_id('$Id: loadsave.php,v 1.153 2007/05/28 20:54:40 rurban Exp $');
 
 /*
- Copyright 1999,2000,2001,2002,2004,2005 $ThePhpWikiProgrammingTeam
+ Copyright 1999,2000,2001,2002,2004,2005,2006,2007 $ThePhpWikiProgrammingTeam
 
  This file is part of PhpWiki.
 
@@ -129,7 +129,7 @@ function EndLoadDump(&$request)
  */
 function MailifyPage ($page, $nversions = 1)
 {
-    $current = $page->getCurrentRevision();
+    $current = $page->getCurrentRevision(false);
     $head = '';
 
     if (STRICT_MAILABLE_PAGEDUMPS) {
@@ -180,8 +180,11 @@ function MailifyPage ($page, $nversions = 1)
  */
 function FilenameForPage ($pagename)
 {
-    $enc = rawurlencode($pagename);
-    return preg_replace('/^\./', '%2e', $enc);
+    //$enc = rawurlencode($pagename);
+    $enc = preg_replace('/:/', '%3A', $pagename);
+    // For every %2F will need to mkdir -p dirname($pagename)
+    return preg_replace('/^\./', '%2E', $enc);
+    // return preg_replace(array('/^\./','%20'), array('%2E',' '), $enc);
 }
 
 /**
@@ -360,6 +363,23 @@ function _copyMsg($page, $smallmsg) {
     }
 }
 
+function mkdir_p($pathname, $permission = 0777) {
+    $arr = explode("/", $pathname);
+    if (empty($arr)) {
+	return mkdir($pathname, $permission);
+    }
+    $s = array_shift($arr);
+    $ok = TRUE;
+    foreach ($arr as $p) {
+	$curr = "$s/$p";
+	if (!is_dir($curr))
+	    $ok = mkdir($curr, $permission);
+	$s = $curr;
+	if (!$ok) return FALSE;
+    }
+    return TRUE;
+}
+
 /**
  * Dump all pages as XHTML to a directory, as pagename.html.
  * Copies all used css files to the directory, all used images to a 
@@ -429,7 +449,7 @@ function DumpHtmlToDir (&$request)
     } else {
         $prefix = '';
         if (isWindows()) {
-            $prefix = '/' . substr($doc_root,0,2); // add drive where apache is installed
+            $prefix = '/'; // . substr($doc_root,0,2); // add drive where apache is installed
         }
         $link_prefix = "file://".$prefix.$directory."/";
     }
@@ -453,24 +473,39 @@ function DumpHtmlToDir (&$request)
             }
             continue;
         }
-
+        $relative_base = '';
         $request->setArg('pagename', $pagename); // Template::_basepage fix
         $filename = FilenameForPage($pagename) . $WikiTheme->HTML_DUMP_SUFFIX;
-        $msg = HTML();
-
         $revision = $page->getCurrentRevision();
-        $template = new Template('browse', $request,
-                                 array('revision' => $revision,
-                                       'CONTENT' => $revision->getTransformedContent()));
+	$args = array('revision' => $revision,
+		      'CONTENT' => $revision->getTransformedContent(),
+                      'relative_base' => $relative_base);
+	// For every %2F will need to mkdir -p dirname($pagename)
+	if (preg_match("/(%2F|\/)/", $filename)) {
+	    // mkdir -p and set relative base for subdir pages
+	    $count = substr_count($filename, "%2F");
+	    $filename = preg_replace("/%2F/", "/", $filename);
+	    $dirname = dirname($filename);
+	    mkdir_p($directory."/".$dirname);
+	    $relative_base = "../";
+	    while ($count > 1) {
+		$relative_base .= "../";
+		$count--;
+	    }
+	    $args['relative_base'] = $relative_base;
+	}
+        $msg = HTML();
+        $template = new Template('browse', $request, $args);
 
-        $data = GeneratePageasXML($template, $pagename);
+        $data = GeneratePageasXML($template, $pagename, $revision, $args);
 
         if ( !($fd = fopen($directory."/".$filename, "wb")) ) {
             $msg->pushContent(HTML::strong(fmt("couldn't open file '%s' for writing",
                                                "$directory/$filename")));
             $request->finish($msg);
         }
-        $num = fwrite($fd, $data, strlen($data));
+        $len = strlen($data);
+        $num = fwrite($fd, $data, $len);
         if ($page->getName() != $filename) {
             $link = LinkURL($link_prefix.$filename, $filename);
             $msg->pushContent(HTML::small(_("saved as "), $link, " ... "));
@@ -482,7 +517,7 @@ function DumpHtmlToDir (&$request)
         flush();
         $request->chunkOutput();
 
-        assert($num == strlen($data));
+        assert($num == $len);
         fclose($fd);
 
         if (USECACHE) {
@@ -515,6 +550,7 @@ function DumpHtmlToDir (&$request)
                 } else {
                     _copyMsg($from, fmt("... not copied to %s", $target));
                 }
+		//TODO: fix to local path for uploaded images, so that pdf will work
             } else {
                 _copyMsg($from, _("... not found"));
             }
@@ -551,6 +587,7 @@ function DumpHtmlToDir (&$request)
                 } else {
                     _copyMsg($from, fmt("... not copied to %s", $target));
                 }
+		// TODO: fix @import url(main.css);
             } else {
                 _copyMsg($from, _("... not found"));
             }
@@ -627,7 +664,7 @@ function MakeWikiZipHtml (&$request)
         if ($page->get('locked'))
             $attrib['write_protected'] = 1;
 
-        $request->setArg('pagename',$pagename); // Template::_basepage fix
+        $request->setArg('pagename', $pagename); // Template::_basepage fix
         $filename = FilenameForPage($pagename) . $WikiTheme->HTML_DUMP_SUFFIX;
         $revision = $page->getCurrentRevision();
 
@@ -727,10 +764,16 @@ function SavePage (&$request, &$pageinfo, $source, $filename)
     if (empty($versiondata['author_id']))
         $versiondata['author_id'] = $versiondata['author'];
 
-    $pagename = $pageinfo['pagename'];
+    // remove invalid backend specific chars. utf8 issues mostly
+    $pagename_check = new WikiPagename($pageinfo['pagename']);
+    if (!$pagename_check->isValid()) {
+        PrintXML(HTML::dt(HTML::strong(_("Invalid pagename!")." ".$pageinfo['pagename'])));
+        return;
+    }
+    $pagename = $pagename_check->getName();
     $content  = $pageinfo['content'];
 
-    if ($pagename ==_("InterWikiMap"))
+    if ($pagename == _("InterWikiMap"))
         $content = _tryinsertInterWikiMap($content);
 
     $dbi =& $request->_dbi;
@@ -754,27 +797,38 @@ function SavePage (&$request, &$pageinfo, $source, $filename)
     }
 
     $current = $page->getCurrentRevision();
-    if ( $current and (! $current->hasDefaultContents())
-         && ($current->getPackedContent() != $content)
-         && ($merging == true) ) {
-        include_once('lib/editpage.php');
-        $request->setArg('pagename', $pagename);
-        $r = $current->getVersion();
-        $request->setArg('revision', $current->getVersion());
-        $p = new LoadFileConflictPageEditor($request);
-        $p->_content = $content;
-        $p->_currentVersion = $r - 1;
-        $p->editPage($saveFailed = true);
-        return; //early return
+    $skip = false;
+    $edit = $request->getArg('edit');
+    if ($merging) { 
+    	if (isset($edit['keep_old'])) {
+            $merging = false;
+            $skip = true;
+    	}
+    	elseif (isset($edit['overwrite'])) {
+            $merging = false;
+            $overwrite = true;
+    	}
+        elseif ( $current and (! $current->hasDefaultContents())
+         && ($current->getPackedContent() != $content) ) 
+        {
+            include_once('lib/editpage.php');
+            $request->setArg('pagename', $pagename);
+            $v = $current->getVersion();
+            $request->setArg('revision', $current->getVersion());
+            $p = new LoadFileConflictPageEditor($request);
+            $p->_content = $content;
+            $p->_currentVersion = $v - 1;
+            $p->editPage($saveFailed = true);
+            return; //early return
+       }
     }
-
-    foreach ($pagedata as $key => $value) {
+    if (!$skip)
+      foreach ($pagedata as $key => $value) {
         if (!empty($value))
             $page->set($key, $value);
-    }
+      }
 
     $mesg = HTML::dd();
-    $skip = false;
     if ($source)
         $mesg->pushContent(' ', fmt("from %s", $source));
 
@@ -800,8 +854,12 @@ function SavePage (&$request, &$pageinfo, $source, $filename)
                 }
             }
             else {
-                $mesg->pushContent(' ', fmt("has edit conflicts - skipped"));
-                $needs_merge = true; // hackish
+		if (isset($edit['keep_old'])) {
+		    $mesg->pushContent(' ', fmt("keep old"));
+		} else {
+		    $mesg->pushContent(' ', fmt("has edit conflicts - skipped"));
+		    $needs_merge = true; // hackish, to display the buttons
+		}
                 $skip = true;
             }
         }
@@ -888,26 +946,68 @@ function RevertPage (&$request)
     $dbi =& $request->_dbi;
     $page = $dbi->getPage($pagename);
     $current = $page->getCurrentRevision();
-    if ($current->getVersion() == 0) {
+    $currversion = $current->getVersion();
+    if ($currversion == 0) {
         $mesg->pushContent(' ', _("no page content"));
         PrintXML(HTML::dt(fmt("Revert")," ",WikiLink($pagename)),
                  $mesg);
+        flush();
         return;
     }
-    if ($current->getVersion() == $version) {
+    if ($currversion == $version) {
         $mesg->pushContent(' ', _("same version page"));
+        PrintXML(HTML::dt(fmt("Revert")," ",WikiLink($pagename)),
+                 $mesg);
+        flush();
+        return;
+    }
+    if ($request->getArg('cancel')) {
+        $mesg->pushContent(' ', _("Cancelled"));
+        PrintXML(HTML::dt(fmt("Revert")," ",WikiLink($pagename)),
+                 $mesg);
+        flush();
+        return;
+    }
+    if (!$request->getArg('verify')) {
+        $mesg->pushContent(HTML::br(),
+                           _("Are you sure?"),
+                           HTML::br(),
+                           HTML::form(array('action' => $request->getPostURL(),
+                                            'method' => 'post'),
+                                      HiddenInputs($request->getArgs(), false, array('verify')),
+                                      HiddenInputs(array('verify' => 1)),
+                                      Button('submit:verify', _("Yes"), 'button'),
+                                      HTML::Raw('&nbsp;'),
+                                      Button('submit:cancel', _("Cancel"), 'button')),
+                           HTML::hr());
+        $rev = $page->getRevision($version);
+        $html = HTML(HTML::dt(fmt("Revert %s to version $version", WikiLink($pagename))), 
+                     $mesg,
+                     $rev->getTransformedContent()); 
+        $template = Template('browse', 
+                             array('CONTENT' => $html));
+        GeneratePage($template, $pagename, $rev);
+        $request->checkValidators();
+        flush();
         return;
     }
     $rev = $page->getRevision($version);
     $content = $rev->getPackedContent();
     $versiondata = $rev->_data;
     $versiondata['summary'] = sprintf(_("revert to version %d"), $version);
-    $new = $page->save($content, $current->getVersion() + 1, $versiondata);
+    $new = $page->save($content, $currversion + 1, $versiondata);
     $dbi->touch();
-    $mesg->pushContent(' ', fmt("- version %d saved to database as version %d",
-                                $version, $new->getVersion()));
-    PrintXML(HTML::dt(fmt("Revert")," ",WikiLink($pagename)),
-             $mesg);
+    
+    $pagelink = WikiLink($pagename);
+    $mesg->pushContent(fmt("Revert: %s", $pagelink), 
+                       fmt("- version %d saved to database as version %d",
+                           $version, $new->getVersion()));
+    // Force browse of current page version.
+    $request->setArg('version', false);
+    $template = Template('savepage', array());
+    $template->replace('CONTENT', $new->getTransformedContent());
+    
+    GeneratePage($template, $mesg, $new);
     flush();
 }
 
@@ -1004,6 +1104,13 @@ function ParseSerializedPage($text, $default_pagename, $user)
                 break;
         }
     }
+    if (empty($pagehash['charset']))
+        $pagehash['charset'] = 'iso-8859-1';
+    // compare to target charset
+    if (strtolower($pagehash['charset']) != strtolower($GLOBALS['charset'])) {
+        $pageinfo['content'] = charset_convert($params['charset'], $GLOBALS['charset'], $pageinfo['content']);
+        $pageinfo['pagename'] = charset_convert($params['charset'], $GLOBALS['charset'], $pageinfo['pagename']);
+    }
     return $pageinfo;
 }
 
@@ -1040,12 +1147,26 @@ function LoadFile (&$request, $filename, $text = false, $mtime = false)
     if (!$mtime)
         $mtime = time();    // Last resort.
 
+    // DONE: check source - target charset for content and pagename
+    // but only for pgsrc'ed content, not from the browser.
+
     $default_pagename = rawurldecode($basename);
     if ( ($parts = ParseMimeifiedPages($text)) ) {
+    	if (count($parts) > 1)
+    	    $overwrite = $request->getArg('overwrite');
         usort($parts, 'SortByPageVersion');
-        foreach ($parts as $pageinfo)
+        foreach ($parts as $pageinfo) {
+            // force overwrite
+            if (count($parts) > 1)
+                $request->setArg('overwrite', 1);
             SavePage($request, $pageinfo, sprintf(_("MIME file %s"),
                                                   $filename), $basename);
+    }
+        if (count($parts) > 1)
+            if ($overwrite) 
+                $request->setArg('overwrite', $overwrite);
+            else     
+	        unset($request->_args['overwrite']);
     }
     else if ( ($pageinfo = ParseSerializedPage($text, $default_pagename,
                                                $request->getUser())) ) {
@@ -1053,8 +1174,16 @@ function LoadFile (&$request, $filename, $text = false, $mtime = false)
                                               $filename), $basename);
     }
     else {
+        // plain old file
         $user = $request->getUser();
 
+        $file_charset = 'iso-8859-1';
+        // compare to target charset
+        if ($file_charset != strtolower($GLOBALS['charset'])) {
+            $text = charset_convert($file_charset, $GLOBALS['charset'], $text);
+            $default_pagename = charset_convert($file_charset, $GLOBALS['charset'], $default_pagename);
+        }
+
         // Assume plain text file.
         $pageinfo = array('pagename' => $default_pagename,
                           'pagedata' => array(),
@@ -1168,14 +1297,14 @@ function LoadAny (&$request, $file_or_dir, $files = false, $exclude = false)
         // with broken dirname or basename functions.
         // FIXME: windows uses \ and :
         if (is_integer(strpos($file_or_dir, "/"))) {
-            $file_or_dir = FindFile($file_or_dir);
-            // Panic
-            if (!file_exists($file_or_dir))
+            $newfile = FindFile($file_or_dir, true);
+            // Panic. urlencoded by the browser (e.g. San%20Diego => San Diego)
+            if (!$newfile)
                 $file_or_dir = dirname($file_or_dir) . "/"
-                    . urlencode(basename($file_or_dir));
+                    . rawurlencode(basename($file_or_dir));
         } else {
             // This is probably just a file.
-            $file_or_dir = urlencode($file_or_dir);
+            $file_or_dir = rawurlencode($file_or_dir);
         }
     }
 
@@ -1192,7 +1321,7 @@ function LoadAny (&$request, $file_or_dir, $files = false, $exclude = false)
     }
 
     if (! $type) {
-        $request->finish(fmt("Unable to load: %s", $file_or_dir));
+        $request->finish(fmt("Empty or not existing source. Unable to load: %s", $file_or_dir));
     }
     else if ($type == 'dir') {
         LoadDir($request, $file_or_dir, $files, $exclude);
@@ -1270,7 +1399,7 @@ function SetupWiki (&$request)
 
     // Ensure that all mandatory pages are loaded
     $finder = new FileFinder;
-    foreach (array_merge(explode(':','OldTextFormattingRules:TextFormattingRules:PhpWikiAdministration'),
+    foreach (array_merge(explode(':','Help/OldTextFormattingRules:Help/TextFormattingRules:PhpWikiAdministration'),
                          $GLOBALS['AllActionPages'],
                          array(constant('HOME_PAGE'))) as $f) 
     {
@@ -1290,8 +1419,12 @@ function SetupWiki (&$request)
                           E_USER_WARNING);
         }
     }
-
     echo "</dl>\n";
+    
+    $pagename = _("InterWikiMap");
+    $map = $dbi->getPage($pagename);
+    $map->set('locked', true);
+    PrintXML(HTML::dt(HTML::em(WikiLink($pagename))), HTML::dd("locked"));
     EndLoadDump($request);
 }
 
@@ -1319,6 +1452,48 @@ function LoadPostFile (&$request)
 
 /**
  $Log: loadsave.php,v $
+ Revision 1.153  2007/05/28 20:54:40  rurban
+ fix DumpToHtml creating dirs
+
+ Revision 1.152  2007/05/01 16:22:41  rurban
+ lock InterWikiMap on init
+
+ Revision 1.151  2007/02/17 14:17:34  rurban
+ only media=print css for htmldump and pdf
+
+ Revision 1.150  2007/01/20 15:53:42  rurban
+ Use WikiPagename treatment for imported pagenames
+
+ Revision 1.149  2007/01/03 21:25:10  rurban
+ Use convert_charset()
+
+ Revision 1.148  2007/01/02 13:21:57  rurban
+ omit want_content if not necessary. support keep_old and overwrite buttons
+
+ Revision 1.147  2006/12/22 17:44:15  rurban
+ support importing foreign charsets. e.g latin1 => utf8
+
+ Revision 1.146  2006/12/17 18:35:23  rurban
+ Create the right subdirectory name, urlencoded.
+
+ Revision 1.145  2006/09/06 06:01:18  rurban
+ support loadfile multipart archives automatically
+
+ Revision 1.144  2006/08/25 22:06:13  rurban
+ args fix to pass $args to the template
+
+ Revision 1.143  2006/08/25 21:48:39  rurban
+ dumphtml subpages
+
+ Revision 1.142  2006/03/19 17:16:32  rurban
+ remove remaining cruft
+
+ Revision 1.141  2006/03/19 17:11:32  rurban
+ add verify to RevertPage, display reverted page as template
+
+ Revision 1.140  2006/03/07 20:45:43  rurban
+ wikihash for php-5.1
+
  Revision 1.139  2005/08/27 18:02:43  rurban
  fix and expand pages
 
@@ -1623,4 +1798,4 @@ function LoadPostFile (&$request)
 // c-hanging-comment-ender-p: nil
 // indent-tabs-mode: nil
 // End:
-?>
\ No newline at end of file
+?>
