@@ -1,7 +1,7 @@
 <?php //-*-php-*-
-rcs_id('$Id: main.php,v 1.216 2005/08/27 09:40:46 rurban Exp $');
+rcs_id('$Id: main.php,v 1.234 2007/07/01 09:36:10 rurban Exp $');
 /*
- Copyright 1999,2000,2001,2002,2004,2005 $ThePhpWikiProgrammingTeam
+ Copyright 1999,2000,2001,2002,2004,2005,2006 $ThePhpWikiProgrammingTeam
 
  This file is part of PhpWiki.
 
@@ -91,7 +91,9 @@ class WikiRequest extends Request {
         $this->setArg('pagename', $this->_deducePagename());
         $this->setArg('action', $this->_deduceAction());
 
-        if ((DEBUG & _DEBUG_SQL) or (time() % 50 == 0)) {
+        if ((DEBUG & _DEBUG_SQL)
+	    or (DATABASE_OPTIMISE_FREQUENCY > 0 and 
+                (time() % DATABASE_OPTIMISE_FREQUENCY == 0))) {
             if ($this->_dbi->_backend->optimize())
                 trigger_error(_("Optimizing database"), E_USER_NOTICE);
         }
@@ -159,15 +161,21 @@ class WikiRequest extends Request {
         }
     }
 
-    function initializeTheme () {
+    function initializeTheme ($when = 'default') {
         global $WikiTheme;
+	// if when = 'default', then first time init (default theme, ...)
+	// if when = 'login', then check some callbacks
+	//                    and maybe the theme changed (other theme defined in pref)
+	// if when = 'logout', then check other callbacks
+	//                    and maybe the theme changed (back to default theme)
 
-        // Load non-default theme
+        // Load non-default theme (when = login)
         $_theme = @$this->_prefs->_prefs['theme'];
         if ($_theme and isset($_theme->theme))
             $user_theme = $_theme->theme;
         else 
             $user_theme = $this->getPref('theme');
+
         //check changed LANG and THEME inside a session. 
         // (e.g. by using another baseurl)
         if (isset($this->_user->_authhow) 
@@ -191,6 +199,21 @@ class WikiRequest extends Request {
         if (empty($WikiTheme))
             include_once("themes/default/themeinfo.php");
         assert(!empty($WikiTheme));
+
+	// When all themeinfo.php files do not execute global init code anymore
+	$WikiTheme->load();
+
+	// callbacks
+	if ($when == 'login') {
+	    if (!$this->_user->hasHomePage()) { // NewUser
+		$WikiTheme->CbNewUserLogin($this, $this->_user->_userid);
+		if (in_array($this->getArg('action'), array('edit','create')))
+		    $WikiTheme->CbNewUserEdit($this, $this->_user->_userid);
+	    }
+	}
+	elseif ($when == 'logout') {
+	    $WikiTheme->CbUserLogout($this, $this->_user->_userid);
+	}
     }
 
     // This really maybe should be part of the constructor, but since it
@@ -220,7 +243,7 @@ class WikiRequest extends Request {
 
         // Save preferences in session and cookie
         if ((defined('WIKI_XMLRPC') and !WIKI_XMLRPC) or $action != 'xmlrpc') {
-            if (isset($this->_user)) {
+            if (isset($this->_user) and $this->_user->_userid) {
             	if (!isset($this->_user->_authhow) or $this->_user->_authhow != 'session') {
                     $this->_user->setPreferences($this->_prefs, true);
             	}
@@ -238,7 +261,7 @@ class WikiRequest extends Request {
             else
                 $action = 'edit';
   	}
-        if (0) {
+        if (! ENABLE_PAGEPERM) { // Bug #1438392 by Matt Brown
             $require_level = $this->requiredAuthority($action);
             if (! $this->_user->hasAuthority($require_level))
                 $this->_notAuthorized($require_level); // NORETURN
@@ -313,7 +336,7 @@ class WikiRequest extends Request {
      * the URL.  (These should be ignored when we receive the POST
      * request.)
      */
-    function getPostURL ($pagename=false) {
+    function getPostURL ($pagename = false) {
         global $HTTP_GET_VARS;
 
         if ($pagename === false)
@@ -321,6 +344,8 @@ class WikiRequest extends Request {
         $action = $this->getArg('action');
         if (!empty($HTTP_GET_VARS['start_debug'])) // zend ide support
             return WikiURL($pagename, array('action' => $action, 'start_debug' => 1));
+        elseif ($action == 'edit')
+            return WikiURL($pagename);
         else
             return WikiURL($pagename, array('action' => $action));
     }
@@ -335,11 +360,7 @@ class WikiRequest extends Request {
 
         $olduser = $this->_user;
         $user = $this->_user->AuthCheck($auth_args);
-        if (isa($user, WikiUserClassname())) {
-            // Successful login (or logout.)
-            $this->_setUser($user);
-        }
-        elseif (is_string($user)) {
+        if (is_string($user)) {
             // Login attempt failed.
             $fail_message = $user;
             $auth_args['pass_required'] = true;
@@ -357,6 +378,10 @@ class WikiRequest extends Request {
             $olduser->PrintLoginForm($this, $auth_args, $fail_message, 'newpage');
             $this->finish();    //NORETURN
         }
+        elseif (isa($user, WikiUserClassname())) {
+            // Successful login (or logout.)
+            $this->_setUser($user);
+        }
         else {
             // Login request cancelled.
         }
@@ -388,10 +413,12 @@ class WikiRequest extends Request {
     function _setUser (&$user) {
         $this->_user =& $user;
         if (defined('MAIN_setUser')) return; // don't set cookies twice
-        $this->setCookieVar('WIKI_ID', $user->getAuthenticatedId(),
+        $this->setCookieVar(getCookieName(), $user->getAuthenticatedId(),
                             COOKIE_EXPIRATION_DAYS, COOKIE_DOMAIN);
-        if ($user->isSignedIn())
+	$isSignedIn = $user->isSignedIn();
+        if ($isSignedIn) {
             $user->_authhow = 'signin';
+	}
 
         // Save userid to prefs..
         if ( empty($this->_user->_prefs)) {
@@ -401,8 +428,8 @@ class WikiRequest extends Request {
         $this->_user->_group = $this->getGroup();
         $this->setSessionVar('wiki_user', $user);
         $this->_prefs->set('userid',
-                           $user->isSignedIn() ? $user->getId() : '');
-        $this->initializeTheme();
+                           $isSignedIn ? $user->getId() : '');
+        $this->initializeTheme($isSignedIn ? 'login' : 'logout');
         define('MAIN_setUser', true);
     }
 
@@ -471,14 +498,17 @@ class WikiRequest extends Request {
         }
         elseif ($require_level == WIKIAUTH_BOGO)
             $msg = fmt("You must sign in to %s.", $what);
-        elseif ($require_level == WIKIAUTH_USER)
+        elseif ($require_level == WIKIAUTH_USER) {
             $msg = fmt("You must log in to %s.", $what);
-        elseif ($require_level == WIKIAUTH_ANON)
+	    if (!ALLOW_ANON_USER)
+		$msg = fmt("You must log in first", $what);
+        } elseif ($require_level == WIKIAUTH_ANON)
             $msg = fmt("Access for you is forbidden to %s.", $what);
         else
             $msg = fmt("You must be an administrator to %s.", $what);
 
-        $this->_user->PrintLoginForm($this, compact('require_level','pass_required'), $msg);
+        $this->_user->PrintLoginForm($this, compact('require_level','pass_required'), 
+				     $msg);
         $this->finish();    // NORETURN
     }
 
@@ -505,7 +535,7 @@ class WikiRequest extends Request {
                     'xmlrpc'     => _("access this wiki via XML-RPC"),
                     'soap'       => _("access this wiki via SOAP"),
                     'zip'        => _("download a zip dump from this wiki"),
-                    'ziphtml'    => _("download an html zip dump from this wiki")
+                    'ziphtml'    => _("download a html zip dump from this wiki")
                     );
         }
         if (in_array($action, array_keys($actionDescriptions)))
@@ -595,6 +625,8 @@ class WikiRequest extends Request {
             case 'search':
             case 'pdf':
             case 'captcha':
+            case 'wikitohtml':
+            case 'setpref':
                 return WIKIAUTH_ANON;
 
             case 'zip':
@@ -670,7 +702,12 @@ class WikiRequest extends Request {
     // [574ms] mainly template:printexpansion: 393ms and template::expandsubtemplate [100+70+60ms]
     function handleAction () {
         $action = $this->getArg('action');
-        if ($this->isPost() and !$this->_user->isAdmin() and $action != 'browse') {
+        if ($this->isPost()  
+            and !$this->_user->isAdmin()
+            and $action != 'browse' 
+            and $action != 'wikitohtml' 
+            )
+        {
             $page = $this->getPage();
             if ( $page->get('moderation') ) {
                 require_once("lib/WikiPlugin.php");
@@ -802,11 +839,12 @@ class WikiRequest extends Request {
         if (!($action = $this->getArg('action'))) {
             // TODO: improve this SOAP.php hack by letting SOAP use index.php 
             // or any other virtual url as with xmlrpc
-            if (defined('WIKI_SOAP')   and WIKI_SOAP)
+            if (defined('WIKI_SOAP') and WIKI_SOAP)
                 return 'soap';
             // Detect XML-RPC requests.
             if ($this->isPost()
-                && $this->get('CONTENT_TYPE') == 'text/xml'
+                && ($this->get('CONTENT_TYPE') == 'text/xml' 
+                    or $this->get('CONTENT_TYPE') == 'application/xml')
                 && strstr($GLOBALS['HTTP_RAW_POST_DATA'], '<methodCall>')
                )
             {
@@ -843,9 +881,9 @@ class WikiRequest extends Request {
             return $this->args['auth']['userid'];
 
         if ($user = $this->getSessionVar('wiki_user')) {
-            // switched auth between sessions. 
+            // Switched auth between sessions. 
             // Note: There's no way to demandload a missing class-definition 
-            // afterwards! (Stupid php)
+            // afterwards! Stupid php.
             if (isa($user, WikiUserClassname())) {
                 $this->_user = $user;
                 $this->_user->_authhow = 'session';
@@ -862,7 +900,7 @@ class WikiRequest extends Request {
         if (!empty($HTTP_ENV_VARS['REMOTE_USER']))
             return $HTTP_ENV_VARS['REMOTE_USER'];
 
-        if ($userid = $this->getCookieVar('WIKI_ID')) {
+        if ($userid = $this->getCookieVar(getCookieName())) {
             if (!empty($userid) and substr($userid,0,2) != 's:') {
                 $this->_user->authhow = 'cookie';
                 return $userid;
@@ -870,6 +908,8 @@ class WikiRequest extends Request {
         }
 
         if ($this->getArg('action') == 'xmlrpc') { // how about SOAP?
+	    if (empty($GLOBALS['HTTP_RAW_POST_DATA']))
+		trigger_error("Wrong always_populate_raw_post_data = Off setting in your php.ini\nCannot use xmlrpc!", E_USER_ERROR);
             // wiki.putPage has special otional userid/passwd arguments. check that later.
             $userid = '';
             if (isset($HTTP_SERVER_VARS['REMOTE_USER']))
@@ -886,7 +926,7 @@ class WikiRequest extends Request {
         return false;
     }
     
-    function _isActionPage ($pagename) {
+    function _isActionPage ($pagename, $verbose = true) {
         $dbi = $this->getDbh();
         $page = $dbi->getPage($pagename);
         if (!$page) return false;
@@ -894,13 +934,14 @@ class WikiRequest extends Request {
         // FIXME: more restrictive check for sane plugin?
         if (strstr($rev->getPackedContent(), '<?plugin'))
             return true;
-        if (!$rev->hasDefaultContents())
+        if ($verbose and !$rev->hasDefaultContents())
             trigger_error("$pagename: Does not appear to be an 'action page'", E_USER_NOTICE);
         return false;
     }
 
     function findActionPage ($action) {
         static $cache;
+        if (!$action) return false;
 
         // check for translated version, as per users preferred language
         // (or system default in case it is not en)
@@ -910,7 +951,7 @@ class WikiRequest extends Request {
             return $cache[$translation];
 
         // check for cached translated version
-        if ($this->_isActionPage($translation))
+        if ($translation and $this->_isActionPage($translation, false))
             return $cache[$action] = $translation;
 
         // Allow for, e.g. action=LikePages
@@ -925,7 +966,7 @@ class WikiRequest extends Request {
             $trans = new WikiPlugin__WikiTranslation();
             $trans->lang = $LANG;
 	    $default = $trans->translate_to_en($action, $LANG);
-            if ($this->_isActionPage($default))
+            if ($default and $this->_isActionPage($default, false))
                 return $cache[$action] = $default;
         } else {
             $default = $translation;
@@ -1039,7 +1080,7 @@ class WikiRequest extends Request {
         $page->set('locked', true);
         $this->_dbi->touch();
         // check ModeratedPage hook
-        if ($moderated = $page->get('moderated')) {
+        if ($moderated = $page->get('moderation')) {
             require_once("lib/WikiPlugin.php");
             $plugin = WikiPluginLoader::getPlugin("ModeratedPage");
             if ($retval = $plugin->lock_check($this, $page, $moderated))
@@ -1080,6 +1121,15 @@ class WikiRequest extends Request {
         $xmlrpc->service();
     }
     
+    function action_soap () {
+	if (defined("WIKI_SOAP") and WIKI_SOAP) // already loaded
+	    return;
+	/*
+	  allow VIRTUAL_PATH or action=soap SOAP access
+	 */
+	include_once("SOAP.php");
+    }
+
     function action_revert () {
         include_once "lib/loadsave.php";
         RevertPage($this);
@@ -1138,6 +1188,19 @@ class WikiRequest extends Request {
         $captcha->image ( $captcha->captchaword() ); 
     }
     
+    function action_wikitohtml () {
+       include_once("lib/WysiwygEdit/Wikiwyg.php");
+       $wikitohtml = new WikiToHtml( $this->getArg("content") , $this);
+       $wikitohtml->send();
+    }
+
+    function action_setpref () {
+	$what = $this->getArg('pref');
+	$value = $this->getArg('value');
+	$prefs =& $this->_user->_prefs;
+	$prefs->set($what, $value);
+	$num = $this->_user->setPreferences($prefs);
+    }
 }
 
 //FIXME: deprecated with ENABLE_PAGEPERM (?)
@@ -1216,9 +1279,8 @@ function main () {
     }
     
     // Initialize with system defaults in case user not logged in.
-    // Should this go into constructor?
-    $request->initializeTheme();
-
+    // Should this go into the constructor?
+    $request->initializeTheme('default');
     $request->updateAuthAndPrefs();
     $request->initializeLang();
     
@@ -1234,8 +1296,8 @@ function main () {
     $request->possiblyDeflowerVirginWiki();
     
 // hack! define proper actions for these.
-if (defined('WIKI_XMLRPC') and WIKI_XMLRPC) return;
-if (defined('WIKI_SOAP')   and WIKI_SOAP)   return;
+//if (defined('WIKI_XMLRPC') and WIKI_XMLRPC) return;
+//if (defined('WIKI_SOAP')   and WIKI_SOAP)   return;
 
     $validators = array('wikiname' => WIKI_NAME,
                         'args'     => wikihash($request->getArgs()),
@@ -1252,8 +1314,10 @@ if (defined('WIKI_SOAP')   and WIKI_SOAP)   return;
     //
     // (If DEBUG if off, this may be a strong validator, but I'm going
     // to go the paranoid route here pending further study and testing.)
-    //
-    $validators['%weak'] = true;
+    // access hits and edit stats in the footer violate strong ETags also. 
+    if (1 or DEBUG) {
+	$validators['%weak'] = true;
+    }
     $request->setValidators($validators);
    
     $request->handleAction();
@@ -1273,10 +1337,64 @@ if (!defined('PHPWIKI_NOMAIN') or !PHPWIKI_NOMAIN)
 
 
 // $Log: main.php,v $
+// Revision 1.234  2007/07/01 09:36:10  rurban
+// themes are now easier derivable classes from other themes.
+// removed global code setters, switched to $WikiTheme->load() in main
+//
+// Revision 1.233  2007/06/02 21:07:21  rurban
+// _isActionPage ($pagename, $verbose = true)
+//
+// Revision 1.232  2007/05/13 18:13:20  rurban
+// improve prefs: only save with userid
+//
+// Revision 1.231  2007/02/17 14:16:44  rurban
+// action_soap, action_setpref, anon perms for wikitohtml, setpref
+//
+// Revision 1.230  2007/01/27 21:52:15  rurban
+// Improved login header for !ALLOW_ANON_USER
+//
+// Revision 1.229  2007/01/07 18:44:30  rurban
+// Fix typo for moderation
+//
+// Revision 1.228  2007/01/02 13:22:17  rurban
+// allow application/xml for xmlrpc detection: acdropdown uses this type. silence findActionPage warnings
+//
+// Revision 1.227  2006/12/22 17:53:55  rurban
+// Remove action from edited url
+// move wikitohtml to lib/WysiwygEdit/Wikiwyg.php
+//
+// Revision 1.226  2006/09/06 06:01:45  rurban
+// minor cleanup
+//
+// Revision 1.225  2006/05/31 19:59:57  jeannicolas
+//
+//
+// Added wysiwyg_editor 1.1b
+//
+// Revision 1.224  2006/05/13 19:59:55  rurban
+// added wysiwyg_editor-1.3a feature by Jean-Nicolas GEREONE <jean-nicolas.gereone@st.com>
+// converted wysiwyg_editor-1.3a js to WysiwygEdit framework
+// changed default ENABLE_WYSIWYG = true and added WYSIWYG_BACKEND = Wikiwyg
+//
+// Revision 1.223  2006/03/19 15:01:00  rurban
+// sf.net patch #1333957 by Matt Brown: Authentication cookie identical across all wikis on a host
+//
+// Revision 1.222  2006/03/19 14:53:12  rurban
+// sf.net patch #1438392 by Matt Brown: Bogo Login is broken when ENABLE_PAGEPERM=false
+//
+// Revision 1.221  2006/03/19 14:23:51  rurban
+// sf.net patch #1377011 by Matt Brown: add DATABASE_OPTIMISE_FREQUENCY
+//
+// Revision 1.220  2006/03/07 21:04:15  rurban
+// wikihash for php-5.1
+//
 // Revision 1.219  2005/10/30 14:20:42  rurban
 // move Captcha specific vars and methods into a Captcha object
 // randomize Captcha chars positions and angles (smoothly)
 //
+// Revision 1.218  2005/10/29 14:18:06  rurban
+// fix typo
+//
 // Revision 1.217  2005/09/18 12:44:00  rurban
 // novatrope patch to let only _AUTHENTICATED view pages
 //
@@ -1839,4 +1957,4 @@ if (!defined('PHPWIKI_NOMAIN') or !PHPWIKI_NOMAIN)
 // c-hanging-comment-ender-p: nil
 // indent-tabs-mode: nil
 // End:
-?>
\ No newline at end of file
+?>
