@@ -76,7 +76,7 @@ extends WikiPlugin
         extract($args);
 
         $content = HTML();
-        $subpages = array_reverse($subpages);
+        //$subpages = array_reverse($subpages); // TODO: why?
         if ($maxpages) {
             $subpages = array_slice ($subpages, 0, $maxpages);
         }
