@@ -1,5 +1,5 @@
 <?php // -*-php-*-
-rcs_id('$Id: PluginManager.php,v 1.17 2005/01/25 06:58:22 rurban Exp $');
+rcs_id('$Id: PluginManager.php,v 1.20 2007/01/03 21:23:57 rurban Exp $');
 /**
  Copyright 1999, 2000, 2001, 2002 $ThePhpWikiProgrammingTeam
 
@@ -37,7 +37,7 @@ extends WikiPlugin
 
     function getVersion() {
         return preg_replace("/[Revision: $]/", '',
-                            "\$Revision: 1.17 $");
+                            "\$Revision: 1.20 $");
     }
 
     function getDefaultArguments() {
@@ -126,28 +126,13 @@ extends WikiPlugin
             }
             $desc = $p->getDescription();
             $ver = $p->getVersion();
-            $args = $p->getDefaultArguments();
+            $arguments = $p->getArgumentsDescription();
             unset($p); //done querying plugin object, release from memory
 
-            $arguments = HTML();
-            foreach ($args as $arg => $default) {
-                // Word around UserPreferences plugin to avoid error
-                if ((is_array($default))) {
-                    $default = '(array)';
-                    // This is a bit flawed with UserPreferences object
-                    //$default = sprintf("array('%s')",
-                    //                   implode("', '", array_keys($default)));
-                }
-                else
-                    if (stristr($default, ' '))
-                        $default = "'$default'";
-                $arguments->pushcontent("$arg=$default", HTML::br());
-            }
-
             // This section was largely improved by Pierrick Meignen:
             // make a link if an actionpage exists
             $pluginNamelink = $pluginName;
-            $pluginDocPageName = $pluginName . "Plugin";
+            $pluginDocPageName = _("Help")."/" . $pluginName . "Plugin";
 
             $pluginDocPageNamelink = false;
             $localizedPluginName = '';
@@ -203,6 +188,15 @@ extends WikiPlugin
 };
 
 // $Log: PluginManager.php,v $
+// Revision 1.20  2007/01/03 21:23:57  rurban
+// Change to "Help/"
+//
+// Revision 1.19  2005/10/12 06:15:25  rurban
+// just aesthetics
+//
+// Revision 1.18  2005/09/26 06:38:00  rurban
+// use the new method
+//
 // Revision 1.17  2005/01/25 06:58:22  rurban
 // reformatting
 //
