@@ -1,7 +1,7 @@
 <?php // -*-php-*-
-rcs_id('$Id: RecentChanges.php,v 1.109 2006/03/19 14:26:29 rurban Exp $');
+rcs_id('$Id: RecentChanges.php,v 1.119 2007/07/01 09:17:45 rurban Exp $');
 /**
- Copyright 1999, 2000, 2001, 2002 $ThePhpWikiProgrammingTeam
+ Copyright 1999,2000,2001,2002,2007 $ThePhpWikiProgrammingTeam
 
  This file is part of PhpWiki.
 
@@ -20,6 +20,8 @@ rcs_id('$Id: RecentChanges.php,v 1.109 2006/03/19 14:26:29 rurban Exp $');
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+include_once("lib/WikiPlugin.php");
+
 /**
  */
 class _RecentChanges_Formatter
@@ -125,14 +127,22 @@ extends _RecentChanges_Formatter
 {
     function diffLink ($rev) {
         global $WikiTheme;
-        $button = $WikiTheme->makeButton(_("(diff)"), $this->diffURL($rev), 'wiki-rc-action');
+        $button = $WikiTheme->makeButton(_("diff"), $this->diffURL($rev), 'wiki-rc-action');
         $button->setAttr('rel', 'nofollow');
-        return $button;
+        return HTML("(",$button,")");
+    }
+
+    /* deletions: red, additions: green */
+    function diffSummary ($rev) {
+        $html = $this->diffURL($rev);
+        return '';
     }
 
     function historyLink ($rev) {
         global $WikiTheme;
-        return $WikiTheme->makeButton(_("(hist)"), $this->historyURL($rev), 'wiki-rc-action');
+        $button = $WikiTheme->makeButton(_("hist"), $this->historyURL($rev), 'wiki-rc-action');
+        $button->setAttr('rel', 'nofollow');
+        return HTML("(",$button,")");
     }
 
     function pageLink ($rev, $link_text=false) {
@@ -160,35 +170,67 @@ extends _RecentChanges_Formatter
     }
 
     function authorLink ($rev) {
-        $author = $rev->get('author');
-        if ( $this->authorHasPage($author) ) {
-            return WikiLink($author);
-        } else
-            return $author;
+        return WikiLink($rev->get('author'), 'if_known');
+    }
+
+    /* Link to all users contributions (contribs and owns) */
+    function authorContribs ($rev) {
+	$author = $rev->get('author');
+	if (preg_match("/^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/", $author)) return '';
+        return HTML('(',
+		    Button(array('action' => _("RecentChanges"), 
+				 'format' => 'contribs',
+				 'author' => $author,
+				 'days' => 360),
+			   _("contribs"),
+			   $author),
+		    ' | ',
+		    Button(array('action' => _("RecentChanges"), 
+				 'format' => 'contribs',
+				 'owner' => $author,
+				 'days' => 360),
+			   _("new pages"),
+			   $author),
+		    ')');
     }
 
     function summaryAsHTML ($rev) {
         if ( !($summary = $this->summary($rev)) )
             return '';
-        return  HTML::strong( array('class' => 'wiki-summary'),
-                              "[",
-                              TransformLinks($summary, $rev->get('markup'), $rev->getPageName()),
-                              "]");
+        return  HTML::span( array('class' => 'wiki-summary'),
+			    "(",
+			    TransformLinks($summary, $rev->get('markup'), $rev->getPageName()),
+			    ")");
     }
 
     function rss_icon () {
         global $request, $WikiTheme;
 
-        $rss_url = $request->getURLtoSelf(array('format' => 'rss'));
+        $rss_url = $request->getURLtoSelf(array('action' => 'RecentChanges', 'format' => 'rss'));
         return HTML::small(array('style' => 'font-weight:normal;vertical-align:middle;'), 
                            $WikiTheme->makeButton("RSS", $rss_url, 'rssicon'));
     }
     function rss2_icon () {
         global $request, $WikiTheme;
 
-        $rss_url = $request->getURLtoSelf(array('format' => 'rss2'));
+        $rss_url = $request->getURLtoSelf(array('action' => 'RecentChanges', 'format' => 'rss2'));
         return HTML::small(array('style' => 'font-weight:normal;vertical-align:middle;'), 
-                           $WikiTheme->makeButton("RSS2", $rss_url, 'rssicon'));
+                           $WikiTheme->makeButton("xml", $rss_url, 'rssicon'));
+    }
+    function atom_icon () {
+        global $request, $WikiTheme;
+	if (DEBUG) {
+	    $rss_url = $request->getURLtoSelf(array('action' => 'RecentChanges', 'format' => 'atom'));
+	    return HTML::small(array('style' => 'font-weight:normal;vertical-align:middle;'), 
+                           $WikiTheme->makeButton("atom", $rss_url, 'rssicon'));
+	}
+    }
+    function grazr_icon () {
+        global $request, $WikiTheme;
+        $rss_url = 'http://grazr.com/gzpanel.html?' . 
+	    WikiURL($request->getArg('pagename'),array('action' => 'RecentChanges', 'format' => 'rss2'),true);
+        return HTML::small(array('style' => 'font-weight:normal;vertical-align:middle;'), 
+                           $WikiTheme->makeButton("grazr-it", $rss_url, 'rssicon'));
     }
 
     function pre_description () {
@@ -202,7 +244,21 @@ extends _RecentChanges_Formatter
             $edits = _("minor edits");
         if (isset($caption) and $caption == _("Recent Comments"))
             $edits = _("comments");
-
+	if (!empty($only_new)) {
+            $edits = _("created new pages");
+	}
+	if (!empty($author)) {
+            global $request;
+            if ($author == '[]')
+                $author = $request->_user->getID();
+            $edits .= sprintf(_(" for pages changed by %s"), $author);
+	}
+	if (!empty($owner)) {
+            global $request;
+            if ($owner == '[]')
+                $owner = $request->_user->getID();
+            $edits .= sprintf(_(" for pages owned by %s"), $owner);
+	}
         if ($timespan = $days > 0) {
             if (intval($days) != $days)
                 $days = sprintf("%.1f", $days);
@@ -223,7 +279,7 @@ extends _RecentChanges_Formatter
          * fr: 1 jour   "Les %d %s les plus rcentes pendant [le dernier (d'une] jour) sont numres ci-dessous."
          * fr: %s jours "Les %d %s les plus rcentes pendant [les derniers (%s] jours) sont numres ci-dessous."
          */
-        if ($limit > 0) {
+	if ($limit > 0) {
             if ($timespan) {
                 if (intval($days) == 1)
                     $desc = fmt("The %d most recent %s during the past day are listed below.",
@@ -271,11 +327,14 @@ extends _RecentChanges_Formatter
         extract($this->_args);
         return array($show_minor ? _("RecentEdits") : _("RecentChanges"),
                      ' ',
-                     $this->rss_icon(), HTML::raw('&nbsp;'), $this->rss2_icon(),
+                     $this->rss_icon(), 
+		     HTML::raw('&nbsp;'), $this->rss2_icon(),
+		     HTML::raw('&nbsp;'), $this->grazr_icon(),
+		     HTML::raw('&nbsp;'), $this->atom_icon(),
                      $this->sidebar_link());
     }
 
-    function empty_message () {
+    function empty_message() {
         if (isset($this->_args['caption']) and $this->_args['caption'] == _("Recent Comments"))
             return _("No comments found");
         else 
@@ -326,6 +385,7 @@ extends _RecentChanges_Formatter
             if (($date = $this->date($rev)) != $last_date) {
                 if ($lines)
                     $html->pushContent($lines);
+		// for user contributions no extra date line
                 $html->pushContent(HTML::h3($date));
                 $lines = HTML::ul();
                 $last_date = $date;
@@ -334,7 +394,6 @@ extends _RecentChanges_Formatter
             // enforce view permission
             if (mayAccessPage('view', $rev->_pagename)) {
                 $lines->pushContent($this->format_revision($rev));
-
                 if ($first)
                     $this->setValidators($rev);
                 $first = false;
@@ -350,33 +409,107 @@ extends _RecentChanges_Formatter
     }
 
     function format_revision ($rev) {
+        global $WikiTheme;
         $args = &$this->_args;
 
         $class = 'rc-' . $this->importance($rev);
 
         $time = $this->time($rev);
         if (! $rev->get('is_minor_edit'))
-            $time = HTML::strong(array('class' => 'pageinfo-majoredit'), $time);
+            $time = HTML::span(array('class' => 'pageinfo-majoredit'), $time);
 
         $line = HTML::li(array('class' => $class));
 
-
         if ($args['difflinks'])
             $line->pushContent($this->diffLink($rev), ' ');
 
         if ($args['historylinks'])
             $line->pushContent($this->historyLink($rev), ' ');
 
-        $line->pushContent($this->pageLink($rev), ' ',
-                           $time, ' ',
-                           $this->summaryAsHTML($rev),
-                           ' ... ',
-                           $this->authorLink($rev));
+	if (isa($WikiTheme, 'Theme_MonoBook')) {
+	    $line->pushContent(
+			       $args['historylinks'] ? '' : $this->historyLink($rev),
+			       ' . . ', $this->pageLink($rev), '; ',
+			       $time, ' . . ',
+			       $this->authorLink($rev),' ',
+			       $this->authorContribs($rev),' ',
+			       $this->summaryAsHTML($rev));
+	} else {
+	    $line->pushContent($this->pageLink($rev), ' ',
+			       $time, ' ',
+			       $this->summaryAsHTML($rev),
+			       ' ... ',
+			       $this->authorLink($rev));
+	}
+        return $line;
+    }
+
+}
+
+/* format=contribs: no seperation into extra dates
+ * 14:41, 3 December 2006 (hist) (diff) Talk:PhpWiki (added diff link)  (top)
+ */
+class _RecentChanges_UserContribsFormatter
+extends _RecentChanges_HtmlFormatter
+{
+    function title () {
+        extract($this->_args);
+        return array(_("UserContribs"),":",$author ? $author : $owner,
+                     ' ',
+                     $this->rss_icon(), 
+		     HTML::raw('&nbsp;'), $this->rss2_icon(),
+		     //HTML::raw('&nbsp;'), $this->grazr_icon(),
+		     HTML::raw('&nbsp;'), $this->atom_icon());
+    }
+
+    function format ($changes) {
+        include_once('lib/InlineParser.php');
+        
+        $html = HTML(HTML::h2(false, $this->title()));
+	$lines = HTML::ol();
+        $first = true; $count = 0;
+        while ($rev = $changes->next()) {
+            if (mayAccessPage('view', $rev->_pagename)) {
+                $lines->pushContent($this->format_revision($rev));
+                if ($first)
+                    $this->setValidators($rev);
+                $first = false;
+            }
+            $count++;
+        }
+        $this->_args['limit'] = $count;
+        if (($desc = $this->description()))
+            $html->pushContent($desc);
+        if ($this->_args['daylist'])
+            $html->pushContent(new DayButtonBar($this->_args));
+        if ($first)
+            $html->pushContent(HTML::p(array('class' => 'rc-empty'),
+                                       $this->empty_message()));
+        else                               
+            $html->pushContent($lines);
+        
+        return $html;
+    }
+
+    function format_revision ($rev) {
+        $args = &$this->_args;
+        $class = 'rc-' . $this->importance($rev);
+        $time = $this->time($rev);
+        if (! $rev->get('is_minor_edit'))
+            $time = HTML::span(array('class' => 'pageinfo-majoredit'), $time);
+
+        $line = HTML::li(array('class' => $class));
+
+	$line->pushContent($this->time($rev),", ");
+	$line->pushContent($this->date($rev)," ");
+	$line->pushContent($this->diffLink($rev), ' ');
+	$line->pushContent($this->historyLink($rev), ' ');
+	$line->pushContent($this->pageLink($rev), ' ',
+			   $this->summaryAsHTML($rev));
         return $line;
     }
 }
 
-
 class _RecentChanges_SideBarFormatter
 extends _RecentChanges_HtmlFormatter
 {
@@ -443,10 +576,10 @@ extends _RecentChanges_HtmlFormatter
     function summaryAsHTML ($rev) {
         if ( !($summary = $this->summary($rev)) )
             return '';
-        return HTML::strong(array('class' => 'wiki-summary'),
-                                "[",
-                                /*TransformLinks(*/$summary,/* $rev->get('markup')),*/
-                                "]");
+        return HTML::span(array('class' => 'wiki-summary'),
+			  "[",
+			  /*TransformLinks(*/$summary,/* $rev->get('markup')),*/
+			  "]");
     }
 
 
@@ -691,7 +824,8 @@ extends _RecentChanges_RssFormatter {
                                  ));
     }
 
-    function cloud_properties () { return false; } // xml-rpc registerProcedure not yet implemented
+    // xml-rpc registerProcedure not yet implemented
+    function cloud_properties () { return false; } 
     function cloud_properties_test () {
         return array('protocol' => 'xml-rpc', // xml-rpc or soap or http-post
                      'registerProcedure' => 'wiki.rssPleaseNotify',
@@ -701,6 +835,72 @@ extends _RecentChanges_RssFormatter {
     }
 }
 
+/** Explicit application/atom+xml Content-Type
+ *  A weird, questionable format
+ */
+class _RecentChanges_AtomFormatter
+extends _RecentChanges_RssFormatter {
+
+    function format ($changes) {
+        global $request;
+        include_once('lib/RssWriter.php');
+        $rss = new AtomFeed;
+
+	// "channel" is called "feed" in atom
+        $rc_url = WikiURL($request->getArg('pagename'), false, 'absurl');
+        $feed_props = array('title' => WIKI_NAME,
+			    'link' => array('rel'=>"alternate",
+			    		    'type'=>"text/html",
+			                    'href' => $rc_url),
+			    'id' => md5($rc_url),                
+			    'modified' => Iso8601DateTime(time()),
+			    'generator' => 'PhpWiki-'.PHPWIKI_VERSION,
+			    'tagline' => '');
+        $rss->feed($feed_props);
+        $first = true;
+        while ($rev = $changes->next()) {
+            // enforce view permission
+            if (mayAccessPage('view', $rev->_pagename)) {
+		$props = $this->item_properties($rev);
+                $rss->addItem($props,
+                              false,
+                              $this->pageURI($rev));
+                if ($first)
+                    $this->setValidators($rev);
+                $first = false;
+            }
+        }
+
+        $request->discardOutput();
+        $rss->finish();
+        printf("\n<!-- Generated by PhpWiki-%s:\n%s-->\n", PHPWIKI_VERSION, $GLOBALS['RCS_IDS']);
+        // Flush errors in comment, otherwise it's invalid XML.
+        global $ErrorManager;
+        if (($errors = $ErrorManager->getPostponedErrorsAsHTML()))
+            printf("\n<!-- PHP Warnings:\n%s-->\n", AsXML($errors));
+
+        $request->finish();     // NORETURN!!!!
+    }
+
+    function item_properties ($rev) {
+        $page = $rev->getPage();
+        $pagename = $page->getName();
+        return array( 'title'           => $pagename,
+                      'link'            => array('rel' => 'alternate',
+						 'type' => 'text/html',
+						 'href' => $this->pageURL($rev)),
+                      'summary'         => $this->summary($rev),
+                      'modified'        => $this->time($rev)."Z",
+                      'issued'          => $this->time($rev),
+                      'created'         => $this->time($rev)."Z",
+                      'author'          => new XmlElement('author', new XmlElement('name', $rev->get('author')))
+                      );
+    }
+}
+
+/**
+ * Filter by non-empty
+ */
 class NonDeletedRevisionIterator extends WikiDB_PageRevisionIterator
 {
     /** Constructor
@@ -730,6 +930,71 @@ class NonDeletedRevisionIterator extends WikiDB_PageRevisionIterator
 
 }
 
+/**
+ * Filter by only_new.
+ * Only new created pages
+ */
+class NewPageRevisionIterator extends WikiDB_PageRevisionIterator
+{
+    /** Constructor
+     *
+     * @param $revisions object a WikiDB_PageRevisionIterator.
+     */
+    function NewPageRevisionIterator ($revisions) {
+        $this->_revisions = $revisions;
+    }
+
+    function next () {
+        while (($rev = $this->_revisions->next())) {
+            if ($rev->getVersion() == 1)
+                return $rev;
+        }
+        $this->free();
+        return false;
+    }
+}
+
+/**
+ * Filter by author
+ */
+class AuthorPageRevisionIterator extends WikiDB_PageRevisionIterator
+{
+    function AuthorPageRevisionIterator ($revisions, $author) {
+        $this->_revisions = $revisions;
+        $this->_author = $author;
+    }
+
+    function next () {
+        while (($rev = $this->_revisions->next())) {
+            if ($rev->get('author_id') == $this->_author)
+                return $rev;
+        }
+        $this->free();
+        return false;
+    }
+}
+
+/**
+ * Filter by owner
+ */
+class OwnerPageRevisionIterator extends WikiDB_PageRevisionIterator
+{
+    function OwnerPageRevisionIterator ($revisions, $owner) {
+        $this->_revisions = $revisions;
+        $this->_owner = $owner;
+    }
+
+    function next () {
+        while (($rev = $this->_revisions->next())) {
+	    $page = $rev->getPage();
+            if ($page->getOwner() == $this->_owner)
+                return $rev;
+        }
+        $this->free();
+        return false;
+    }
+}
+
 class WikiPlugin_RecentChanges
 extends WikiPlugin
 {
@@ -739,7 +1004,7 @@ extends WikiPlugin
 
     function getVersion() {
         return preg_replace("/[Revision: $]/", '',
-                            "\$Revision: 1.109 $");
+                            "\$Revision: 1.119 $");
     }
 
     function managesValidators() {
@@ -766,6 +1031,9 @@ extends WikiPlugin
                      'show_major'   => true,
                      'show_all'     => false,
                      'show_deleted' => 'sometimes',
+		     'only_new'     => false,
+		     'author'       => false,
+		     'owner'        => false,
                      'limit'        => false,
                      'format'       => false,
                      'daylist'      => false,
@@ -794,7 +1062,9 @@ extends WikiPlugin
         return $args;
     }
 
-    function getMostRecentParams ($args) {
+    function getMostRecentParams (&$args) {
+    	$show_all = false; $show_minor = false; $show_major = false;
+    	$limit = false;
         extract($args);
 
         $params = array('include_minor_revisions' => $show_minor,
@@ -802,11 +1072,24 @@ extends WikiPlugin
                         'include_all_revisions' => !empty($show_all));
         if ($limit != 0)
             $params['limit'] = $limit;
-
-        if ($days > 0.0)
+        if (!empty($args['author'])) {
+            global $request;
+            if ($args['author'] == '[]')
+                $args['author'] = $request->_user->getID();
+            $params['author'] = $args['author'];
+        }
+        if (!empty($args['owner'])) {
+            global $request;
+            if ($args['owner'] == '[]')
+                $args['owner'] = $request->_user->getID();
+            $params['owner'] = $args['owner'];
+        }
+        if (!empty($days)) {
+          if ($days > 0.0)
             $params['since'] = time() - 24 * 3600 * $days;
-        elseif ($days < 0.0)
+          elseif ($days < 0.0)
             $params['since'] = 24 * 3600 * $days - time();
+        }
 
         return $params;
     }
@@ -814,12 +1097,19 @@ extends WikiPlugin
     function getChanges ($dbi, $args) {
         $changes = $dbi->mostRecent($this->getMostRecentParams($args));
 
-        $show_deleted = $args['show_deleted'];
+        $show_deleted = @$args['show_deleted'];
+        $show_all = @$args['show_all'];
         if ($show_deleted == 'sometimes')
-            $show_deleted = $args['show_minor'];
+            $show_deleted = @$args['show_minor'];
 
+        if (!empty($args['only_new']))
+            $changes = new NewPageRevisionIterator($changes);
+        if (!empty($args['author']))
+            $changes = new AuthorPageRevisionIterator($changes, $args['author']);
+        if (!empty($args['owner']))
+            $changes = new OwnerPageRevisionIterator($changes, $args['owner']);
         if (!$show_deleted)
-            $changes = new NonDeletedRevisionIterator($changes, !$args['show_all']);
+            $changes = new NonDeletedRevisionIterator($changes, !$show_all);
 
         return $changes;
     }
@@ -834,6 +1124,8 @@ extends WikiPlugin
                 $fmt_class = '_RecentChanges_RssFormatter';
             elseif ($format == 'rss2')
                 $fmt_class = '_RecentChanges_Rss2Formatter';
+            elseif ($format == 'atom')
+                $fmt_class = '_RecentChanges_AtomFormatter';
             elseif ($format == 'rss091') {
                 include_once "lib/RSSWriter091.php";
                 $fmt_class = '_RecentChanges_RssFormatter091';
@@ -842,6 +1134,8 @@ extends WikiPlugin
                 $fmt_class = '_RecentChanges_SideBarFormatter';
             elseif ($format == 'box')
                 $fmt_class = '_RecentChanges_BoxFormatter';
+            elseif ($format == 'contribs')
+                $fmt_class = '_RecentChanges_UserContribsFormatter';
             else
                 $fmt_class = '_RecentChanges_HtmlFormatter';
         }
@@ -872,8 +1166,10 @@ extends WikiPlugin
         $args['show_deleted'] = 'sometimes';
         $args['show_all'] = false;
         $args['days'] = 90;
-        return $this->makeBox(WikiLink($this->getName(),'',SplitPagename($this->getName())),
-                              $this->format($this->getChanges($request->_dbi, $args), $args));
+        return $this->makeBox(WikiLink($this->getName(),'',
+                                       SplitPagename($this->getName())),
+                              $this->format
+                              ($this->getChanges($request->_dbi, $args), $args));
     }
 
 };
@@ -895,6 +1191,8 @@ class DayButtonBar extends HtmlElement {
                 $caption = _("Show all changes for:");
             else
                 $caption = _("Show changes for:");
+	    if ($only_new)
+                $caption = _("All new pages since:");
         }
 
         $this->pushContent($caption, ' ');
@@ -927,6 +1225,36 @@ class DayButtonBar extends HtmlElement {
 }
 
 // $Log: RecentChanges.php,v $
+// Revision 1.119  2007/07/01 09:17:45  rurban
+// add ATOM support, a very questionable format
+//
+// Revision 1.118  2007/06/02 18:24:59  rurban
+// global WikiTheme
+//
+// Revision 1.117  2007/05/30 20:43:31  rurban
+// added MonoBook UserContribs
+//
+// Revision 1.116  2007/05/13 18:13:41  rurban
+// use all filters, not just the first, ignoring the others. improve wording a bit
+//
+// Revision 1.115  2007/04/08 16:24:10  rurban
+// Remove redundant code in ->authorLink(): 'if_known' does the same
+//
+// Revision 1.114  2007/02/17 22:39:44  rurban
+// format=rss overhaul
+//
+// Revision 1.113  2007/02/17 14:15:52  rurban
+// new argument owner=[] for RecentChangesMyPages
+//
+// Revision 1.112  2007/01/28 20:32:07  rurban
+// silence empty warning
+//
+// Revision 1.111  2007/01/25 21:48:52  rurban
+// new argument author=[]
+//
+// Revision 1.110  2007/01/22 23:51:36  rurban
+// new arg: only_new
+//
 // Revision 1.109  2006/03/19 14:26:29  rurban
 // sf.net patch by Matt Brown: Add rel=nofollow to more actions
 //
