@@ -1,7 +1,7 @@
 <?php // -*-php-*-
-rcs_id('$Id: _BackendInfo.php,v 1.24 2005/01/29 19:47:43 rurban Exp $');
+rcs_id('$Id: _BackendInfo.php,v 1.28 2007/05/15 16:32:43 rurban Exp $');
 /**
- Copyright 1999, 2000, 2001, 2002 $ThePhpWikiProgrammingTeam
+ Copyright 1999,2000,2001,2002,2006,2007 $ThePhpWikiProgrammingTeam
 
  This file is part of PhpWiki.
 
@@ -36,11 +36,12 @@ extends WikiPlugin
 
     function getVersion() {
         return preg_replace("/[Revision: $]/", '',
-                            "\$Revision: 1.24 $");
+                            "\$Revision: 1.28 $");
     }
 
     function getDefaultArguments() {
-        return array('page' => '[pagename]');
+        return array('page' => '[pagename]',
+		     'notallversions' => 0);
     }
 
     function run($dbi, $argstr, &$request, $basepage) {
@@ -50,11 +51,13 @@ extends WikiPlugin
             return '';
 
         $backend = &$dbi->_backend;
+        $this->chunk_split = true;
+        $this->readonly_pagemeta = array();
+        $this->hidden_pagemeta = array ('_cached_html');
 
         $html = HTML(HTML::h3(fmt("Querying backend directly for '%s'",
                                   $page)));
 
-
         $table = HTML::table(array('border' => 1,
                                    'cellpadding' => 2,
                                    'cellspacing' => 0));
@@ -67,17 +70,38 @@ extends WikiPlugin
             $this->_fixupData($pagedata);
             $table->pushContent($this->_showhash("get_pagedata('$page')", $pagedata));
         }
+	if (!$notallversions) {
+	    $version = $backend->get_latest_version($page);
+	    $vdata = $backend->get_versiondata($page, $version, true);
+	    $this->_fixupData($vdata);
+	    $table->pushContent(HTML::tr(HTML::td(array('colspan' => 2))));
+	    $table->pushContent($this->_showhash("get_versiondata('$page',$version)",
+						 $vdata));
+	} else {
+	    for ($version = $backend->get_latest_version($page);
+		 $version;
+		 $version = $backend->get_previous_version($page, $version))
+		{
+		    $vdata = $backend->get_versiondata($page, $version, true);
+		    $this->_fixupData($vdata);
+		    $table->pushContent(HTML::tr(HTML::td(array('colspan' => 2))));
+		    $table->pushContent($this->_showhash("get_versiondata('$page',$version)",
+							 $vdata));
+		}
+	}
 
-        for ($version = $backend->get_latest_version($page);
-             $version;
-             $version = $backend->get_previous_version($page, $version))
-            {
-                $vdata = $backend->get_versiondata($page, $version, true);
-                $this->_fixupData($vdata);
-                $table->pushContent(HTML::tr(HTML::td(array('colspan' => 2))));
-                $table->pushContent($this->_showhash("get_versiondata('$page',$version)",
-                                                     $vdata));
-            }
+        $linkdata = $backend->get_links($page, false);
+        if ($linkdata->count())
+            $table->pushContent($this->_showhash("get_links('$page')", $linkdata->asArray()));
+        $relations = $backend->get_links($page, false, false, false, false, false, true);
+        if ($relations->count()) {
+            $table->pushContent($this->_showhash("get_relations('$page')", array()));
+            while ($rel = $relations->next())
+                $table->pushContent($this->_showhash(false, $rel));
+        }
+        $linkdata = $backend->get_links($page, true);
+        if ($linkdata->count())
+            $table->pushContent($this->_showhash("get_backlinks('$page')", $linkdata->asArray()));
 
         $html->pushContent($table);
         return $html;
@@ -85,12 +109,15 @@ extends WikiPlugin
 
     /**
      * Really should have a _fixupPagedata and _fixupVersiondata, but this works.
+     * also used in plugin/EditMetaData
      */
-    function _fixupData(&$data) {
+    function _fixupData(&$data, $prefix='') {
+	if (!is_array($data)) return;
+	
         global $request;
         $user = $request->getUser();
-
         foreach ($data as $key => $val) {
+            $fullkey = $prefix . '[' . $key . ']';
             if (is_integer($key)) {
             	;
             } elseif ($key == 'passwd' and !$user->isAdmin()) {
@@ -103,24 +130,36 @@ extends WikiPlugin
                 ob_end_clean();
             }
             elseif (is_bool($val)) {
-            	$data[$key] = $val ? "<true>" : "<false>";
+            	$data[$key] = $this->_showvalue($key, $val ? "true" : "false", $prefix);
             }
-            elseif (is_string($val) && (substr($val, 0, 2) == 'a:')) {
+            elseif (is_string($val) && ((substr($val, 0, 2) == 'a:' 
+					 or (substr($val, 0, 2) == 'O:')))) 
+	    {
                 // how to indent this table?
                 $val = unserialize($val);
-                $this->_fixupData($val);
+                $this->_fixupData($val, $fullkey);
                 $data[$key] = HTML::table(array('border' => 1,
                                                 'cellpadding' => 2,
                                                 'cellspacing' => 0),
-                                          $this->_showhash(false, $val));
+                                          $this->_showhash(false, $val, $fullkey));
             }
             elseif (is_array($val)) {
                 // how to indent this table?
-                $this->_fixupData($val);
+                $this->_fixupData($val, $fullkey);
                 $data[$key] = HTML::table(array('border' => 1,
                                                 'cellpadding' => 2,
                                                 'cellspacing' => 0),
-                                          $this->_showhash(false, $val));
+                                          $this->_showhash(false, $val, $fullkey));
+            } elseif (is_object($val)) {
+                // how to indent this table?
+                ob_start();
+                print_r($val);
+                $val = HTML::pre(ob_get_contents());
+                ob_end_clean();
+                $data[$key] = HTML::table(array('border' => 1,
+                                                'cellpadding' => 2,
+                                                'cellspacing' => 0),
+                                          $this->_showhash(false, $val, $fullkey));
             }
             elseif ($key and $key == '%content') {
                 if ($val === true)
@@ -132,8 +171,9 @@ extends WikiPlugin
         }
         unset($data['%pagedata']); // problem in backend
     }
-            
-    function _showhash ($heading, $hash, $pagename = '') {
+
+    /* also used in plugin/EditMetaData */
+    function _showhash ($heading, $hash, $prefix='') {
         $rows = array();
         if ($heading)
             $rows[] = HTML::tr(array('bgcolor' => '#ffcccc',
@@ -141,8 +181,10 @@ extends WikiPlugin
                                HTML::td(array('colspan' => 2,
                                               'style' => 'color:#000000'),
                                         $heading));
+	if (!is_array($hash)) return array();
         ksort($hash);
         foreach ($hash as $key => $val) {
+            if ($this->chunk_split and is_string($val)) $val = chunk_split($val);	
             $rows[] = HTML::tr(HTML::td(array('align' => 'right',
                                               'bgcolor' => '#cccccc',
                                               'style' => 'color:#000000'),
@@ -150,14 +192,32 @@ extends WikiPlugin
                                              HTML::raw('&nbsp;'))),
                                HTML::td(array('bgcolor' => '#ffffff',
                                               'style' => 'color:#000000'),
-                                        $val ? $val : HTML::raw('&nbsp;'))
+                                        $this->_showvalue($key, $val, $prefix))
                                );
         }
         return $rows;
     }
+
+    /* also used in plugin/EditMetaData */
+    function _showvalue ($key, $val, $prefix='') {
+	return $val ? $val : HTML::raw('&nbsp;');
+    }
+
 };
 
 // $Log: _BackendInfo.php,v $
+// Revision 1.28  2007/05/15 16:32:43  rurban
+// Improve baseclass for EditMetaData
+//
+// Revision 1.27  2007/05/13 18:13:48  rurban
+// base class for EditmetaData
+//
+// Revision 1.26  2007/01/07 18:44:47  rurban
+// Add notallversion argument. Support more types: objects, serialized objects. split overlong strings
+//
+// Revision 1.25  2006/09/06 06:02:36  rurban
+// print linkinfo also
+//
 // Revision 1.24  2005/01/29 19:47:43  rurban
 // support bool
 //
