@@ -1,4 +1,4 @@
-<?php //rcs_id('$Id: stdlib.php,v 1.244 2005/09/11 13:24:33 rurban Exp $');
+<?php //rcs_id('$Id: stdlib.php,v 1.262 2007/06/07 17:02:01 rurban Exp $');
 /*
  Copyright 1999,2000,2001,2002,2004,2005 $ThePhpWikiProgrammingTeam
 
@@ -55,7 +55,7 @@
     file_mtime ($filename)
     sort_file_mtime ($a, $b)
     class fileSet {fileSet($directory, $filepattern = false), 
-                   getFiles($exclude=false, $sortby=false, $limit=false) }
+                   getFiles($exclude='', $sortby='', $limit='') }
     class ListRegexExpand { listMatchCallback($item, $key),  
                             expandRegex ($index, &$pages) }
 
@@ -82,6 +82,8 @@
     firstNWordsOfContent ($n, $content)
     extractSection ($section, $content, $page, $quiet = false, $sectionhead = false)
     isExternalReferrer()
+    
+    charset_convert($from, $to, $data)
 
   function: LinkInterWikiLink($link, $linktext)
   moved to: lib/interwiki.php
@@ -137,6 +139,16 @@ function UnMangleXmlIdentifier($str) {
                         $str);
 }
 
+/**
+ * Returns a name for the WIKI_ID cookie that should be unique on the host.
+ * But for it to be unique you must have set a unique WIKI_NAME in your
+ * configuration file.
+ * @return string The name of the WIKI_ID cookie to use for this wiki.
+ */
+function GetCookieName() {
+    return preg_replace("/[^\d\w]/", "_", WIKI_NAME) . "_WIKI_ID";
+}
+
 /**
  * Generates a valid URL for a given Wiki pagename.
  * @param mixed $pagename If a string this will be the name of the Wiki page to link to.
@@ -370,7 +382,7 @@ function LinkURL($url, $linktext = '') {
  *
  * Syntax: [image.png size=50% border=n align= hspace= vspace= width= height=]
  * Disallows sizes which are too small. 
- * Spammers may use such (typically invisible) image attributes to higher their GoogleRank.
+ * Spammers may use such (typically invisible) image attributes to raise their GoogleRank.
  *
  * Handle embeddable objects, like svg, class, vrml, swf, svgz, pdf, avi, wmv especially.
  */
@@ -383,8 +395,8 @@ function LinkImage($url, $alt = false) {
                                      _("BAD URL -- remove all of <, >, \"")));
     } else {
         // support new syntax: [image.jpg size=50% border=n]
-        if (!preg_match("/\.(".$force_img.")/i", $url))
-            $ori_url = $url;
+        //if (!preg_match("/\.(".$force_img.")/i", $url))
+        $ori_url = $url;
         $arr = split(' ',$url);
         if (count($arr) > 1) {
             $url = $arr[0];
@@ -646,14 +658,15 @@ class WikiPageName
         if (is_string($name)) {
             $this->shortName = $name;
             if (strstr($name, ':')) {
-                list($moniker, $this->shortName) = split (":", $name, 2);
+                list($moniker, $shortName) = split (":", $name, 2);
 	  	$map = getInterwikiMap(); // allow overrides to custom maps
                 if (isset($map->_map[$moniker])) {
                     $url = $map->_map[$moniker];
                     if (strstr($url, '%s'))
-                        $url = sprintf($url, $this->shortName);
+                        $url = sprintf($url, $shortName);
                     else
-                        $url .= $this->shortName;
+                        $url .= $shortName;
+                    $this->shortName = $shortName;
                     // expand Talk or User, but not to absolute urls!
                     if (strstr($url, '//')) {
                         if ($moniker == 'Talk')
@@ -746,7 +759,13 @@ class WikiPageName
         return substr($name, 1);
     }
 
-
+    /**
+     * Compress internal white-space to single space character.
+     *
+     * This leads to problems with loading a foreign charset pagename, 
+     * which cannot be deleted anymore, because unknown chars are compressed.
+     * So BEFORE importing a file _check must be done !!!
+     */
     function _check($pagename) {
         // Compress internal white-space to single space character.
         $pagename = preg_replace('/[\s\xa0]+/', ' ', $orig = $pagename);
@@ -754,7 +773,7 @@ class WikiPageName
             $this->_warnings[] = _("White space converted to single space");
     
         // Delete any control characters.
-        if (DATABASE_TYPE == 'cvs' or DATABASE_TYPE == 'file') {
+        if (DATABASE_TYPE == 'cvs' or DATABASE_TYPE == 'file' or DATABASE_TYPE == 'flatfile') {
             $pagename = preg_replace('/[\x00-\x1f\x7f\x80-\x9f]/', '', $orig = $pagename);
             if ($pagename != $orig)
                 $this->_errors[] = _("Control characters not allowed");
@@ -782,8 +801,11 @@ class WikiPageName
         }
 
         // disallow some chars only on file and cvs
-        if ((DATABASE_TYPE == 'cvs' or DATABASE_TYPE == 'file') 
-            and preg_match('/(:|\.\.)/', $pagename, $m)) {
+        if ((DATABASE_TYPE == 'cvs' 
+            or DATABASE_TYPE == 'file' 
+            or DATABASE_TYPE == 'flatfile') 
+            and preg_match('/(:|\.\.)/', $pagename, $m)) 
+        {
             $this->_warnings[] = sprintf(_("Illegal chars %s removed"), $m[1]);
             $pagename = str_replace('..', '', $pagename);
             $pagename = str_replace(':', '', $pagename);
@@ -979,7 +1001,7 @@ function ConvertOldMarkup ($text, $markup_type = "block") {
                 preg_match('/^\[\s*(\d+)\s*\]/', $block, $m);
                 $footnum = $m[1];
                 $block = substr($block, strlen($m[0]));
-                $prefix = "#[|ftnt_${footnum}]~[[${footnum}|#ftnt_ref_${footnum}]~] ";
+                $prefix = "#[|ftnt_".${footnum}."]~[[".${footnum}."|#ftnt_ref_".${footnum}."]~] ";
             }
             elseif ($block[0] == '<') {
                 // Plugin.
@@ -1080,6 +1102,7 @@ function SplitPagename ($page) {
         $RE[] = '/(\d)([[:alpha:]])/';
         // Split at subpage seperators. TBD in Theme.php
         $RE[] = "/([^${sep}]+)(${sep})/";
+        $RE[] = "/(${sep})([^${sep}]+)/";
         
         foreach ($RE as $key)
             $RE[$key] = pcre_fix_posix_classes($key);
@@ -1374,7 +1397,7 @@ class fileSet {
      * (This was a function LoadDir in lib/loadsave.php)
      * See also http://www.php.net/manual/en/function.readdir.php
      */
-    function getFiles($exclude=false, $sortby=false, $limit=false) {
+    function getFiles($exclude='', $sortby='', $limit='') {
         $list = $this->_fileList;
 
         if ($sortby) {
@@ -1527,7 +1550,7 @@ function explodeList($input, $allnames, $glob_style = true, $case_sensitive = tr
 
 // echo implode(":",explodeList("Test*",array("xx","Test1","Test2")));
 function explodePageList($input, $include_empty=false, $sortby='pagename', 
-			 $limit=false, $exclude=false) {
+			 $limit='', $exclude='') {
     include_once("lib/PageList.php");
     return PageList::explodePageList($input, $include_empty, $sortby, $limit, $exclude);
 }
@@ -1843,13 +1866,7 @@ function fixTitleEncoding( $s ) {
     $locharset = strtolower($charset);
 
     if( $locharset != "utf-8" and $ishigh and $isutf )
-        // if charset == 'iso-8859-1' then simply use utf8_decode()
-        if ($locharset == 'iso-8859-1')
-            return utf8_decode( $s );
-        else
-            // TODO: check for iconv support
-            return iconv( "UTF-8", $charset, $s );
-
+	$s = charset_convert('UTF-8', $locharset, $s);
     if ($locharset == "utf-8" and $ishigh and !$isutf )
         return utf8_encode( $s );
 
@@ -1984,6 +2001,7 @@ function isExternalReferrer(&$request) {
         $se = new SearchEngines();
         return $se->parseSearchQuery($referrer);
     }
+    //if (DEBUG) return array('query' => 'wiki');
     return false;
 }
 
@@ -1992,19 +2010,46 @@ function isExternalReferrer(&$request) {
  */
 function loadPhpExtension($extension) {
     if (!extension_loaded($extension)) {
-        $soname = (isWindows() ? 'php_' : '') . $extension . (isWindows() ? '.dll' : '.so');
+	$isWindows = (substr(PHP_OS,0,3) == 'WIN');
+        $soname = ($isWindows ? 'php_' : '') 
+	        . $extension 
+	        . ($isWindows ? '.dll' : '.so');
         if (!@dl($soname))
             return false;
     }
     return extension_loaded($extension);
 }
 
+function charset_convert($from, $to, $data) {
+    //global $CHARSET;
+    //$wikicharset = strtolower($CHARSET);
+    //$systemcharset = strtolower(get_cfg_var('iconv.internal_encoding')); // 'iso-8859-1';
+    if (strtolower($from) == 'utf-8' and strtolower($to) == 'iso-8859-1')
+	return utf8_decode($data);
+    if (strtolower($to) == 'utf-8' and strtolower($from) == 'iso-8859-1')
+	return utf8_encode($data);
+
+    if (loadPhpExtension("iconv")) {
+	$tmpdata = iconv($from, $to, $data);
+	if (!$tmpdata)
+	    trigger_error("charset conversion $from => $to failed. Wrong source charset?", E_USER_WARNING);
+	else
+	    $data = $tmpdata;
+    } else {
+	trigger_error("The iconv extension cannot be loaded", E_USER_WARNING);
+    }
+    return $data;
+}
+
 function string_starts_with($string, $prefix) {
     return (substr($string, 0, strlen($prefix)) == $prefix);
 }
 function string_ends_with($string, $suffix) {
     return (substr($string, -strlen($suffix)) == $suffix);
 }
+function array_remove($arr,$value) {
+   return array_values(array_diff($arr,array($value)));
+}
 
 /** 
  * Ensure that the script will have another $secs time left. 
@@ -2021,13 +2066,17 @@ function longer_timeout($secs = 30) {
 
 function printSimpleTrace($bt) {
     //print_r($bt);
-    echo "Traceback:\n";
-    foreach ($bt as $i => $elem) {
-        if (!array_key_exists('file', $elem)) {
-            continue;
-        }
-        echo join(" ",array_values($elem)),"\n";
-        //print "  " . $elem['file'] . ':' . $elem['line'] . " " .$elem['function']"\n";
+    echo "\nTraceback:\n";
+    if (function_exists('debug_print_backtrace')) { // >= 5
+	debug_print_backtrace();
+    } else {
+	foreach ($bt as $i => $elem) {
+	    if (!array_key_exists('file', $elem)) {
+		continue;
+	    }
+	    //echo join(" ",array_values($elem)),"\n";
+	    echo "  ",$elem['file'],':',$elem['line']," ",$elem['function'],"\n";
+	}
     }
 }
 
@@ -2037,11 +2086,12 @@ function printSimpleTrace($bt) {
  * Special quirks for Windows: Requires cygwin.
  */
 function getMemoryUsage() {
+    //if (!(DEBUG & _DEBUG_VERBOSE)) return;
     if (function_exists('memory_get_usage') and memory_get_usage()) {
         return memory_get_usage();
-    } elseif (function_exists('getrusage') and ($u = getrusage()) and !empty($u['ru_maxrss'])) {
+    } elseif (function_exists('getrusage') and ($u = @getrusage()) and !empty($u['ru_maxrss'])) {
         $mem = $u['ru_maxrss'];
-    } elseif (substr(PHP_OS,0,3) == 'WIN') { // requires a newer cygwin
+    } elseif (substr(PHP_OS,0,3) == 'WIN') { // may require a newer cygwin
         // what we want is the process memory only: apache or php (if CGI)
         $pid = getmypid();
         $memstr = '';
@@ -2049,17 +2099,17 @@ function getMemoryUsage() {
  	if (function_exists('win32_ps_list_procs')) {
 	    $info = win32_ps_stat_proc($pid);
 	    $memstr = $info['mem']['working_set_size'];
-	} else {
-	    // This works only if it's a cygwin process (apache or php)
-	    // requires a newer cygwin
+	} elseif(0) {
+	    // This works only if it's a cygwin process (apache or php).
+	    // Requires a newer cygwin
 	    //$memstr = exec("cat /proc/$pid/statm |cut -f1");
 
 	    // if it's native windows use something like this: 
 	    //   (requires pslist from sysinternals.com)
-	    //$memstr = exec("pslist $pid|grep -A1 Mem|sed 1d|perl -ane\"print \$"."F[5]\"");
+	    $memstr = exec("pslist $pid|grep -A1 Mem|sed 1d|perl -ane\"print \$"."F[5]\"");
         }
         return (integer) trim($memstr);
-    } elseif (1) {
+    } elseif (0) {
         $pid = getmypid();
         //%MEM: Percentage of total memory in use by this process
         //VSZ: Total virtual memory size, in 1K blocks.
@@ -2072,6 +2122,47 @@ function getMemoryUsage() {
 }
 
 // $Log: stdlib.php,v $
+// Revision 1.262  2007/06/07 17:02:01  rurban
+// fix display of pagenames containing ":" in certain lists
+//
+// Revision 1.261  2007/06/01 06:37:53  rurban
+// use native debug_print_backtrace
+//
+// Revision 1.260  2007/02/17 14:15:21  rurban
+// fix ImgLink params: $ori_url
+//
+// Revision 1.259  2007/01/20 11:41:38  rurban
+// WikiPagename::_check: add flatfile support
+//
+// Revision 1.258  2007/01/07 18:43:51  rurban
+// Remove FileFinder dependency for loadPhpExtension()
+//
+// Revision 1.257  2007/01/03 21:24:56  rurban
+// Disable noisy ps subprocess on DEBUG. Turn it on explicitly on memory debugging. Add convert_charset helper. Use convert_charset().
+//
+// Revision 1.256  2007/01/02 13:23:49  rurban
+// ftnt_${footnum}: do not confuse old php string definition parsers. Clarify API: sortby,limit and exclude are strings.
+//
+// Revision 1.255  2006/12/22 16:53:38  rurban
+// Try to dl() load the iconv extension, if not already loaded
+//
+// Revision 1.254  2006/12/02 19:53:05  rurban
+// Simplify DISABLE_MARKUP_WIKIWORD handling by adding the new function
+// stdlib: array_remove(). Hopefully PHP will not add this natively sooner
+// or later.
+//
+// Revision 1.253  2006/08/25 22:20:52  rurban
+// better split subpages at sep
+//
+// Revision 1.252  2006/06/18 11:03:36  rurban
+// support rc version
+//
+// Revision 1.251  2006/03/19 15:01:00  rurban
+// sf.net patch #1333957 by Matt Brown: Authentication cookie identical across all wikis on a host
+//
+// Revision 1.250  2006/03/07 20:45:44  rurban
+// wikihash for php-5.1
+//
 // Revision 1.249  2005/10/30 14:24:33  rurban
 // move rand_ascii_readable from Captcha to stdlib
 //
@@ -2450,4 +2541,4 @@ function getMemoryUsage() {
 // c-hanging-comment-ender-p: nil
 // indent-tabs-mode: nil
 // End:   
-?>
\ No newline at end of file
+?>
