@@ -22,8 +22,6 @@ $locale[" Keep up to %d minor edits, but keep them no longer than %d days."] =
    "Garder jusqu' %d dition mineures, mais pas plus que %d jours.";
 $locale[" Sorry, cannot merge."] =
    " Dsol, impossible de fusionner.";
-$locale[" This is for small text"] =
-   "C'est pour des petits textes";
 $locale[" replace "] =
    " remplace ";
 $locale[" skipped"] =
@@ -90,14 +88,14 @@ $locale["%s is missing"] =
    "%s est manquant";
 $locale["%s is not locked!"] =
    "%s n'est pas verrouill!";
-$locale["%s is not writable"] =
-   "%s n'est pas accessible en criture";
 $locale["%s is not writable."] =
    "%s n'est pas accessible en criture.";
 $locale["%s is only allowed in locked pages."] =
    "%s n'est autoris que dans les pages verrouilles.";
 $locale["%s k"] =
    "%s k";
+$locale["%s not defined"] =
+   "%s : non dfini";
 $locale["%s not found"] =
    "%s non trouv";
 $locale["%s of this page"] =
@@ -170,8 +168,6 @@ $locale["(diff)"] =
    "(diff)";
 $locale["(disable individual page permissions, enable inheritance)?"] =
    "(dsactive les permissions individuelles par page, activer l'hritage) ?";
-$locale["(hist)"] =
-   "(hist)";
 $locale["(max. recursion level: %d)"] =
    "(niveau maximum de rcursivit: %d)";
 $locale["(ugo : rwx)"] =
@@ -380,8 +376,6 @@ $locale["Bad url in src: remove all of <, >, \""] =
    "Mauvaise url dans la source : retirez tous les <, >, \"";
 $locale["Blog page"] =
    "Page de Blog";
-$locale["Blogs"] =
-   "Blogs";
 $locale["Bogo Users"] =
    "Utilisateurs Bogo";
 $locale["Bold text"] =
@@ -420,6 +414,8 @@ $locale["Cannot open AUTH_GROUP_FILE %s"] =
    "Impossible d'ouvrir AUTH_GROUP_FILE %s";
 $locale["Case insensitive?"] =
    "Sensible  la casse ?";
+$locale["Category"] =
+   "Catgorie";
 $locale["CategoryCategory"] =
    "CatgorieCatgorie";
 $locale["CategoryGroup"] =
@@ -520,8 +516,6 @@ $locale["Couldn't rename page '%s' to '%s'."] =
    "Impossible de renommer la page '%s' en '%s'.";
 $locale["Couldn't start commandline '%s'"] =
    "Impossible de lancer la ligne de commande '%s'";
-$locale["Count"] =
-   "Compte";
 $locale["Create a Wiki page by the provided name."] =
    "Crer une PageWiki avec le nom demand.";
 $locale["Create a Wiki page."] =
@@ -730,8 +724,6 @@ $locale["Every"] =
    "Toutes";
 $locale["Example.jpg"] =
    "Example.jpg";
-$locale["Existing page-level metadata for %s:"] =
-   "Mtadonnes existantes pour niveau de pages pour %s :";
 $locale["Expiry parameters"] =
    "Expiration des paramtres";
 $locale["External Links"] =
@@ -752,8 +744,6 @@ $locale["Fatal PhpWiki Error"] =
    "Erreur Fatale de PhpWiki";
 $locale["File successfully uploaded."] =
    "Fichier transfr avec succs.";
-$locale["FileInfo"] =
-   "FichierInfo";
 $locale["Files with extension %s are not allowed."] =
    "%s: extension interdit.";
 $locale["FindPage"] =
@@ -798,8 +788,6 @@ $locale["Go?"] =
    "Allons-y ?";
 $locale["GoTo"] =
    "AllerVers";
-$locale["GoodStyle"] =
-   "StyleCorrect";
 $locale["Google:%s"] =
    "Google: %s";
 $locale["GooglePlugin"] =
@@ -840,12 +828,8 @@ $locale["History of all major and minor edits by %s to page %s."] =
    "Historique de toutes les ditions majeures et mineures par %s sur la page %s.";
 $locale["History of all major and minor modifications for any page edited by %s."] =
    "Historique des ditions majeures ou mineures pour n'importe quelle page dite par %s.";
-$locale["History of all major edits by %s to page %s."] =
-   "Historique de toutes les ditions majeures par %s sur la page %s.";
 $locale["History of changes."] =
    "Historique des changements.";
-$locale["History of major modifications for any page edited by %s."] =
-   "Historique des ditions majeures de toutes les pages dites par %s.";
 $locale["Hit statistics"] =
    "Statistiques de visites";
 $locale["Hits"] =
@@ -1180,8 +1164,6 @@ $locale["Nothing found"] =
    "Aucun rsultat";
 $locale["OK"] =
    "OK";
-$locale["Ok"] =
-   "Ok";
 $locale["Okay"] =
    "OK";
 $locale["Old UserPage %s without stored password updated with empty password. Set a password in your UserPreferences."] =
@@ -1192,8 +1174,6 @@ $locale["OldStyleTable"] =
    "TableauAncienStyle";
 $locale["OldStyleTablePlugin"] =
    "PluginTableauAncienStyle";
-$locale["OldTextFormattingRules"] =
-   "RglesDeFormatageDesTextes";
 $locale["Older page:"] =
    "Ancienne page :";
 $locale["One page links to %s:"] =
@@ -1334,8 +1314,6 @@ $locale["PhotoAlbum"] =
    "AlbumPhotos";
 $locale["PhotoAlbumPlugin"] =
    "PluginAlbumPhotos";
-$locale["Photos"] =
-   "Photos";
 $locale["PhpHighlight"] =
    "ColorationPhp";
 $locale["PhpHighlightPlugin"] =
@@ -1660,8 +1638,6 @@ $locale["SignOut"] =
    "Dconnexion";
 $locale["Signed Users"] =
    "Utilisateurs signs";
-$locale["Signed as %s"] =
-   "Connect comme %s";
 $locale["Simple RSS Feed aggregator Plugin"] =
    "Importer des ressources RSS";
 $locale["Simple Sample Plugin"] =
@@ -1754,12 +1730,10 @@ $locale["TODO: Google can only translate from english and french. Find a transla
    "A FAIRE: Google peut uniquement traduire l'anglais et le franais. Trouver un service pour traduire %s en %s";
 $locale["Table Of Contents"] =
    "Table des matires";
-$locale["TeX directory not writable."] =
-   "Rpertoire TeX non accessible en criture.";
 $locale["TeX2png"] =
    "TeX2png";
-$locale["TextFormattingRules"] =
-   "RglesDeFormatageDesTextes";
+$locale["Template"] =
+   "Thme";
 $locale["Thank you for editing %s."] =
    "Merci d'avoir dit %s.";
 $locale["Thanks for adding this translation!"] =
@@ -1864,6 +1838,8 @@ $locale["Too many external links."] =
    "Trop de liens exterieurs.";
 $locale["Top & bottom toolbars"] =
    "Barres d'outils en haut et en bas";
+$locale["Topic"] =
+   "Sujet";
 $locale["Total %d plugins: "] =
    "%d plugins au total : ";
 $locale["Total of %d languages: "] =
@@ -1897,13 +1873,13 @@ $locale["UNOBTAINABLE"] =
 $locale["USER"] =
    "UTILISATEUR";
 $locale["Unable to bind LDAP server %s using %s %s"] =
-   "Impossible de se connecter au serveur LDAP %s, avec %s %s";
+   "Impossible de se connecter au serveur LDAP %s";
 $locale["Unable to connect to IMAP server "] =
    "Impossible de se connecter au serveur IMAP ";
 $locale["Unable to connect to LDAP server %s"] =
    "Impossible de se connecter au serveur LDAP %s";
-$locale["Unable to load: %s"] =
-   "Impossible de charger : %s";
+$locale["Unable to find src='%s'"] =
+   "Impossible de lire src=%s";
 $locale["Unable to open directory '%s' for reading"] =
    "Impossible d'ouvrir le rpertoire '%s' en lecture";
 $locale["Unable to read src='%s'"] =
@@ -1932,8 +1908,6 @@ $locale["Unlocking pages"] =
    "Dverrouiller les pages";
 $locale["Unsupported ACL access type %s ignored."] =
    "Le type d'accs ACL %s n'est pas support. -- Ignor.";
-$locale["Unsupported argument: %s=%s"] =
-   "Argument %s=%s n'est pas support !";
 $locale["Up"] =
    "Haut";
 $locale["UpLoad"] =
@@ -2022,18 +1996,12 @@ $locale["WabiSabi"] =
    "WabiSabi";
 $locale["Wanted From"] =
    "Recherches par";
-$locale["Wanted Pages for %s (%d total):"] =
-   "Pages recherches pour %s  dans ce wiki (%d au total) :";
 $locale["Wanted Pages for %s:"] =
    "Pages recherches pour %s :";
-$locale["Wanted Pages in this wiki (%d total):"] =
-   "Pages recherches dans ce wiki (%d au total) :";
 $locale["Wanted Pages in this wiki:"] =
    "Pages recherches dans ce wiki :";
 $locale["WantedPages"] =
    "PagesRecherches";
-$locale["Warning!"] =
-   "Attention !";
 $locale["Warning:"] =
    "Attention:";
 $locale["Warning: The search string cannot be empty!"] =
@@ -2078,8 +2046,6 @@ $locale["WikiAdminSetAcl"] =
    "WikiAdminSetAcl";
 $locale["WikiAdminUtils"] =
    "WikiAdminUtils";
-$locale["WikiAdminUtils says:"] =
-   "WikiAdminUtils dit :";
 $locale["WikiBlog"] =
    "WikiBlog";
 $locale["WikiBlogPlugin"] =
@@ -2164,6 +2130,8 @@ $locale["Your home page has not been created yet so your preferences cannot not
    "Cette page a t verrouille par l'administrateur et vos changements n'ont pas pu tre enregistre.";
 $locale["Your home page is locked so your preferences cannot not be saved."] =
    "Cette page a t verrouille par l'administrateur et vos changements n'ont pas pu tre enregistre.";
+$locale["Your rating "] =
+   "Votre note ";
 $locale["Your rating was %.1f"] =
    "Votre note est %.1f";
 $locale["Your signature"] =
@@ -2214,16 +2182,6 @@ $locale["check for mysql page.id auto_increment flag"] =
    "vrifie le marqueur auto_increment pour page.id";
 $locale["check for mysql session.sess_id sanity"] =
    "recherche une nouvelle colonne session.sess_ip";
-$locale["check for necessary ActionPage updates"] =
-   "vrifie les mises  jour d'ActionPage ncessaire";
-$locale["check for necessary config updates"] =
-   "vrifie les mises  jour de configuration ncessaire";
-$locale["check for necessary database updates"] =
-   "rechercher des mises  jour ncessaire dans la base de donnes";
-$locale["check for necessary pgsrc updates"] =
-   "recherche les mises  jour ncessaire des pages sources";
-$locale["check for necessary plugin argument updates"] =
-   "recherche les mises  jour ncessaire des plugin argument";
 $locale["check for new session.sess_ip column"] =
    "recherche une nouvelle colonne session.sess_ip";
 $locale["check for table %s"] =
@@ -2252,6 +2210,8 @@ $locale["diff"] =
    "diff";
 $locale["diff this page"] =
    "diff de cette page";
+$locale["download a html zip dump from this wiki"] =
+   "tlcharger une sauvegarde HTML compresse de ce wiki";
 $locale["download a zip dump from this wiki"] =
    "tlcharger une sauvegarde compresse de ce wiki";
 $locale["dump html pages"] =
@@ -2281,7 +2241,7 @@ $locale["home page"] =
 $locale["invalid %s ignored"] =
    "%s non valable ignor";
 $locale["invalid argument %s"] =
-   "Argument non valable %s";
+   "Argument non valable : %s";
 $locale["invidual page permission"] =
    "Permission de page individuelle";
 $locale["latest comment by "] =
