@@ -20,8 +20,6 @@ $locale["'%s': Bad page name"] =
    "'%s': ڡ̾ޤ";
 $locale["(diff)"] =
    "(ʬ)";
-$locale["(hist)"] =
-   "()";
 $locale["<no matches>"] =
    "<פޤ>";
 $locale["<none>"] =
@@ -80,8 +78,6 @@ $locale["Complete."] =
    "λޤ";
 $locale["Contents"] =
    "ƥ";
-$locale["Count"] =
-   "";
 $locale["Create: %s"] =
    ": %s";
 $locale["Creating pages"] =
@@ -424,8 +420,6 @@ $locale["View the current version"] =
    "ߤΥСɽ";
 $locale["W"] =
    "";
-$locale["Warning!"] =
-   "ٹ!";
 $locale["Warning: You are editing an old revision."] =
    "ٹ: ʤϸŤӥԽƤޤ";
 $locale["Width"] =
