@@ -128,8 +128,6 @@ $locale["(diff)"] =
    "(差異)";
 $locale["(disable individual page permissions, enable inheritance)?"] =
    "(禁制個別網頁的權限,使用繼承)?";
-$locale["(hist)"] =
-   "(歷程)";
 $locale["(max. recursion level: %d)"] =
    "(最大遞迴層級: %d)";
 $locale["(ugo : rwx)"] =
@@ -170,8 +168,6 @@ $locale["ACCESS DENIED: You must log in to upload files."] =
    "拒絕存取: 請先登入才能上傳檔案";
 $locale["ACL"] =
    "ACL";
-$locale["AUTH_GROUP_FILE not defined"] =
-   "AUTH_GROUP_FILE 未定義";
 $locale["Action page"] =
    "動作頁面";
 $locale["Add %s hours to the server's local time when reporting times."] =
@@ -360,8 +356,6 @@ $locale["Couldn't chmod page '%s' to '%s'."] =
    "無法為頁面 '%s' 變更權限為 '%s'.";
 $locale["Couldn't rename page '%s' to '%s'."] =
    "無法更名頁面 '%s' 為 '%s'.";
-$locale["Count"] =
-   "計數";
 $locale["Create a Wiki page by the provided name."] =
    "以提供的名稱建立 Wiki 頁面.";
 $locale["Create a Wiki page."] =
@@ -534,8 +528,6 @@ $locale["Error: The user HomePage must be a valid WikiWord. Sorry, UserPreferenc
    "錯誤: 使用者的首頁必須是一個合法的 WikiWord. 抱歉, UserPreferences 無法儲存.";
 $locale["Every"] =
    "每";
-$locale["Existing page-level metadata for %s:"] =
-   "已存在的頁面層級中繼資料 %s:";
 $locale["Expiry parameters"] =
    "過期參數";
 $locale["External Links"] =
@@ -584,8 +576,6 @@ $locale["Go to or create page."] =
    "前往或建立頁面.";
 $locale["GoTo"] =
    "GoTo";
-$locale["GoodStyle"] =
-   "GoodStyle";
 $locale["GooglePlugin"] =
    "GooglePlugin";
 $locale["Group %s does not exist"] =
@@ -614,12 +604,8 @@ $locale["History of all major and minor edits by %s to page %s."] =
    "所有被 %s 修改的主要和次要修訂歷程 - 頁面 %s.";
 $locale["History of all major and minor modifications for any page edited by %s."] =
    "被 %s 修改的所有頁面的主要與次要變更歷程.";
-$locale["History of all major edits by %s to page %s."] =
-   "被 %s 修改的所有主要修訂歷程 - 頁面 %s.";
 $locale["History of changes."] =
    "變更的歷程.";
-$locale["History of major modifications for any page edited by %s."] =
-   "被 %s 修改的主要所有頁面變更歷程.";
 $locale["Hit statistics"] =
    "擊閱數統計";
 $locale["Hits"] =
@@ -688,8 +674,6 @@ $locale["JabberPresence"] =
    "JabberPresence";
 $locale["Keep up to %d major edits, but keep them no longer than %d days."] =
    "保持 %d 份主要編輯,但不超過 %d 天.";
-$locale["LDAP_AUTH_HOST not defined"] =
-   "LDAP_AUTH_HOST 未定義";
 $locale["Language"] =
    "語言";
 $locale["Last"] =
@@ -848,8 +832,6 @@ $locale["Note, that user accounts with bouncing emails will be disabled."] =
    "注意, 使用者帳號的回信(bouncing emails)將被關閉.";
 $locale["Note:"] =
    "注意:";
-$locale["Ok"] =
-   "確定";
 $locale["Okay"] =
    "Okay";
 $locale["Old UserPage %s without stored password updated with empty password. Set a password in your UserPreferences."] =
@@ -860,8 +842,6 @@ $locale["OldStyleTable"] =
    "OldStyleTable";
 $locale["OldStyleTablePlugin"] =
    "OldStyleTablePlugin";
-$locale["OldTextFormattingRules"] =
-   "OldTextFormattingRules";
 $locale["Older page:"] =
    "較舊的頁面:";
 $locale["One page links to %s:"] =
@@ -1290,8 +1270,6 @@ $locale["TODO: Google can only translate from english and french. Find a transla
    "TODO: Google 只能從英文與法文轉換. 為 %s 尋找轉換到語言 %s 的服務.";
 $locale["Table Of Contents"] =
    "目錄";
-$locale["TextFormattingRules"] =
-   "TextFormattingRules";
 $locale["Thank you for editing %s."] =
    "謝謝你編輯了 %s.";
 $locale["Thanks for adding this translation!"] =
@@ -1404,8 +1382,6 @@ $locale["Unable to connect to IMAP server "] =
    "無法連結到 IMAP 伺服器";
 $locale["Unable to connect to LDAP server %s"] =
    "無法連結到 LDAP 伺服器 %s";
-$locale["Unable to load: %s"] =
-   "無法載入: %s";
 $locale["Unable to open directory '%s' for reading"] =
    "無法開啟目錄 '%s' 進行讀取";
 $locale["Undefined method %s for special group %s"] =
@@ -1492,14 +1468,8 @@ $locale["W"] =
    "寬";
 $locale["WabiSabi"] =
    "WabiSabi";
-$locale["Wanted Pages for %s (%d total):"] =
-   "%s 所需要的頁面 (共 %d):";
-$locale["Wanted Pages in this wiki (%d total):"] =
-   "在此 wiki 所需要的頁面 (共 %d):";
 $locale["WantedPages"] =
    "WantedPages";
-$locale["Warning!"] =
-   "警告!";
 $locale["Warning:"] =
    "警告:";
 $locale["Warning: The search string cannot be empty!"] =
@@ -1536,8 +1506,6 @@ $locale["WikiAdminSetAcl"] =
    "WikiAdminSetAcl";
 $locale["WikiAdminUtils"] =
    "WikiAdminUtils";
-$locale["WikiAdminUtils says:"] =
-   "WikiAdminUtils 說:";
 $locale["WikiBlog"] =
    "WikiBlog";
 $locale["WikiBlogPlugin"] =
@@ -1646,8 +1614,6 @@ $locale["diff this page"] =
    "比對此頁面";
 $locale["download a zip dump from this wiki"] =
    "從此 wiki 下載 zip dump";
-$locale["download an html zip dump from this wiki"] =
-   "從此 wiki 下載 html zip dump";
 $locale["dump html pages"] =
    "傾印 html 頁面";
 $locale["dump serial pages"] =
