@@ -1,4 +1,4 @@
--- $Id: mysql-initialize.sql,v 1.8 2005/06/21 05:59:18 rurban Exp $
+-- $Id: mysql-initialize.sql,v 1.14 2007/01/04 16:48:05 rurban Exp $
 
 CREATE TABLE page (
 	id              INT NOT NULL AUTO_INCREMENT,
@@ -14,7 +14,7 @@ CREATE TABLE page (
 	cached_html 	MEDIUMBLOB,
         PRIMARY KEY (id),
 	UNIQUE KEY (pagename)
-) CHARSET=latin1;
+);
 
 CREATE TABLE version (
 	id              INT NOT NULL,
@@ -43,8 +43,10 @@ CREATE TABLE nonempty (
 CREATE TABLE link (
 	linkfrom        INT NOT NULL,
         linkto          INT NOT NULL,
+        relation        INT DEFAULT 0,
 	INDEX (linkfrom),
-        INDEX (linkto)
+        INDEX (linkto),
+        INDEX (relation)
 );
 
 CREATE TABLE session (
@@ -70,30 +72,33 @@ CREATE TABLE session (
 -- in the DBAuthParam SQL statements also.
 
 CREATE TABLE pref (
-  	userid 	CHAR(48) BINARY NOT NULL UNIQUE,
+  	userid 	VARCHAR(48) BINARY NOT NULL UNIQUE,
   	prefs  	TEXT NULL DEFAULT '',
-  	passwd 	CHAR(48) BINARY DEFAULT '',
-	groupname CHAR(48) BINARY DEFAULT 'users',
+  	passwd 	VARCHAR(48) BINARY DEFAULT '',
+	groupname VARCHAR(48) BINARY DEFAULT 'users',
   	PRIMARY KEY (userid)
 );
 
 -- update to 1.3.12: (see lib/upgrade.php)
+
 -- ALTER TABLE pref ADD passwd 	CHAR(48) BINARY DEFAULT '';
 -- ALTER TABLE pref ADD groupname CHAR(48) BINARY DEFAULT 'users';
 
 -- deprecated since 1.3.12. only useful for seperate databases.
 -- better use the extra pref table where such users can be created easily 
 -- without password.
---CREATE TABLE user (
+
+-- CREATE TABLE user (
 --  	userid 	CHAR(48) BINARY NOT NULL UNIQUE,
 --  	passwd 	CHAR(48) BINARY DEFAULT '',
 --	prefs  	TEXT NULL DEFAULT '',
 --	groupname CHAR(48) BINARY DEFAULT 'users',
 --  	PRIMARY KEY (userid)
---);
+-- );
 
 -- Use the member table, if you need it for n:m user-group relations,
 -- and adjust your DBAUTH_AUTH_ SQL statements.
+
 CREATE TABLE member (
 	userid    CHAR(48) BINARY NOT NULL,
    	groupname CHAR(48) BINARY NOT NULL DEFAULT 'users',
@@ -121,7 +126,7 @@ CREATE INDEX rating_rateepage ON rating (rateepage);
 -- see http://www.outoforder.cc/projects/apache/mod_log_sql/docs-2.0/#id2756178
 CREATE TABLE accesslog (
         time_stamp    INT UNSIGNED,
-	remote_host   VARCHAR(50),
+	remote_host   VARCHAR(100),
 	remote_user   VARCHAR(50),
         request_method VARCHAR(10),
 	request_line  VARCHAR(255),
@@ -137,4 +142,7 @@ CREATE TABLE accesslog (
 );
 CREATE INDEX log_time ON accesslog (time_stamp);
 CREATE INDEX log_host ON accesslog (remote_host);
--- create extra indices on demand (usually referer. see plugin/AccessLogSql)
\ No newline at end of file
+-- create extra indices on demand (usually referer. see plugin/AccessLogSql)
+
+-- upgrade to 1.3.13:
+-- ALTER TABLE accesslog CHANGE remote_host VARCHAR(100);
\ No newline at end of file
