@@ -1,4 +1,4 @@
--- $Id: oci8-initialize.sql,v 1.4 2005/06/21 05:59:18 rurban Exp $
+-- $Id: oci8-initialize.sql,v 1.7 2006/12/23 11:47:40 rurban Exp $
 
 set verify off
 set feedback off
@@ -7,7 +7,7 @@ set feedback off
 -- Prefix for table names.
 --
 -- You should set this to the same value you specify for
--- $DBParams['prefix'] in index.php.
+-- DATABASE_PREFIX in config.ini
 --
 -- You have to use a prefix, because some phpWiki tablenames are 
 -- Oracle reserved words!
@@ -53,6 +53,7 @@ define nonmt_id=&prefix.nonmt_id
 define link_tbl=&prefix.link
 define link_from=&prefix.link_from
 define link_to=&prefix.link_to
+define link_rel=&prefix.link_rel
 
 define session_tbl=&prefix.session
 define sess_id=&prefix.sess_id
@@ -62,8 +63,8 @@ define sess_ip=&prefix.sess_ip
 define pref_tbl=&prefix.pref
 define pref_id=&prefix.pref_id
 
-define user_tbl=&prefix.user
-define user_id=&prefix.user_id
+--define user_tbl=&prefix.user
+--define user_id=&prefix.user_id
 
 define member_tbl=&prefix.member
 define member_userid=&prefix.member_userid
@@ -90,6 +91,9 @@ CREATE TABLE &page_tbl (
 	CONSTRAINT &page_nm UNIQUE (pagename)
 );
 
+-- we use 0 <=> global_data to satisfy the relation = 0 constraint
+INSERT INTO &page_tbl VALUES (0,'global_data',0,'','');
+
 prompt Creating &version_tbl
 CREATE TABLE &version_tbl (
 	id		INT NOT NULL,
@@ -120,10 +124,12 @@ CREATE TABLE &nonempty_tbl (
 prompt Creating &link_tbl
 CREATE TABLE &link_tbl (
         linkfrom	INT NOT NULL,
-        linkto		INT NOT NULL
+        linkto		INT NOT NULL,
+        relation  	INT 
 );
 CREATE INDEX &link_from ON &link_tbl (linkfrom);
 CREATE INDEX &link_to   ON &link_tbl (linkto);
+CREATE INDEX &link_rel  ON &link_tbl (relation);
 
 prompt Creating &session_tbl
 CREATE TABLE &session_tbl (
@@ -144,21 +150,18 @@ prompt Creating &pref_tbl
 CREATE TABLE &pref_tbl (
   	userid 	CHAR(48) NOT NULL,
   	prefs  	CLOB DEFAULT '',
-  	passwd 	CHAR(48) DEFAULT '',
+	passwd  CHAR(48) DEFAULT '',
 	groupname CHAR(48) DEFAULT 'users',
 	CONSTRAINT &pref_id PRIMARY KEY (userid)
 );
--- ALTER TABLE pref ADD passwd 	CHAR(48) DEFAULT '';
--- ALTER TABLE pref ADD groupname CHAR(48) DEFAULT 'users';
 
 -- better use the extra pref table where such users can be created easily 
 -- without password.
-
---prompt Creating &pref_tbl
+--prompt Creating &user_tbl
 --CREATE TABLE &user_tbl (
 --  	userid 	CHAR(48) NOT NULL,
---	prefs  	CLOB DEFAULT '',
 --  	passwd 	CHAR(48) DEFAULT '',
+--	prefs  	CLOB DEFAULT '',
 --	groupname CHAR(48) DEFAULT 'users',
 --  	CONSTRAINT &user_id PRIMARY KEY (userid)
 --);
@@ -193,16 +196,16 @@ prompt Creating &accesslog_tbl
 CREATE TABLE &accesslog_tbl (
 -- for OCI 9i+ use:   time_stamp TIMESTAMP,
         time_stamp    DATE,
-	remote_host   VARCHAR2(50),
+	remote_host   VARCHAR2(100),
 	remote_user   VARCHAR2(50),
         request_method VARCHAR2(10),
 	request_line  VARCHAR2(255),
 	request_args  VARCHAR2(255),
 	request_file  VARCHAR2(255),
 	request_uri   VARCHAR2(255),
-	request_time  DATE,
+	request_time  VARCHAR2(28),
 	status 	      NUMBER(4),
-	bytes_sent    NUMBER(4),
+	bytes_sent    NUMBER,
         referer       VARCHAR(255), 
 	agent         VARCHAR(255),
 	request_duration FLOAT
